/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file AppTimer.cpp

\brief This class contains attributes and functions that can be used to manipulate time.
*/

// TerraAmazon
#include "AppTimer.h"

// STL
#include <time.h>

ta::common::AppTimer::AppTimer() :
  m_currentSleep_Stop(0),
  m_timerSleepMilliseconds(0)
{
  cleanAttributs();
}

ta::common::AppTimer::~AppTimer()
{
  cleanAttributs();
}

void ta::common::AppTimer::cleanAttributs(){

  m_startAc = 0;
  m_startTimer = 0;
  m_stopTimer = 0;

  m_sleepAc = 0;
  m_startSleep = 0;
  m_stopSleep = 0;
  m_auxAntSleep = 0;

  m_statusTimer = false;
  m_statusSleep = false;
}

void ta::common::AppTimer::startTimer(double timerSleepMilliseconds)
{
  if (!m_statusTimer){
    m_timerSleepMilliseconds = timerSleepMilliseconds;
    m_startTimer = clock();
    m_statusTimer = true;
  }
}

void ta::common::AppTimer::stopTimer()
{
  m_stopTimer = clock();
  m_startAc = m_stopTimer - m_startTimer;
  m_statusTimer = false;
  cleanAttributs();
}

double ta::common::AppTimer::currentTimer()
{
  if (!m_statusTimer)
    return m_startAc;
  else{
    double difference = (clock() - m_startTimer);
    return difference;
  }
}

double ta::common::AppTimer::getTimerMil()
{
  m_startAc = currentTimer();
  return m_startAc;
}

double ta::common::AppTimer::getTimerSec()
{
  m_startAc = currentTimer();
  return m_startAc / 1000;
}

double ta::common::AppTimer::getTimerMin()
{
  m_startAc = currentTimer();
  return m_startAc / 60000;
}

double ta::common::AppTimer::getTimerHou()
{
  m_startAc = currentTimer();
  return m_startAc / 3600000;
}

bool ta::common::AppTimer::getStatusTimer(){
  return m_statusTimer;
}

void ta::common::AppTimer::startSleep()
{
  if (!m_statusTimer)
    return;

  if (!m_statusSleep){
    m_startSleep = clock();
    m_currentSleep_Stop = m_startSleep;
    m_statusSleep = true;
  }
}

void ta::common::AppTimer::stopSleep()
{
  m_stopSleep = clock();
  sleepAc(m_stopSleep, m_startSleep);
  m_statusSleep = false;
  m_auxAntSleep = 0;
}

double ta::common::AppTimer::timerSleep()
{
  if (!m_statusSleep){
    m_currentSleep_Stop = m_sleepAc;
    return m_sleepAc;
  }
  else{
    if (m_currentSleep_Stop == 0)
      return 0;

    sleepAc(clock(), m_startSleep);
    return m_sleepAc;
  }
}

void ta::common::AppTimer::sleepAc(double stop, double start)
{
  double difference = (stop - start);
  double sleepInter = 0;
  if (m_timerSleepMilliseconds <= difference)
  {
    sleepInter = difference - m_timerSleepMilliseconds;
    if (sleepInter > m_sleepAc)
      m_sleepAc = sleepInter;
    else
      m_sleepAc += sleepInter - m_auxAntSleep;

    m_auxAntSleep = sleepInter;
  }
}

double ta::common::AppTimer::getSleepMil()
{
  return timerSleep();
}

double ta::common::AppTimer::getSleepSec()
{
  return timerSleep() / 1000;
}

double ta::common::AppTimer::getSleepMin()
{
  return timerSleep() / 60000;
}

double ta::common::AppTimer::getSleepHou()
{
  return timerSleep() / 3600000;
}

bool ta::common::AppTimer::getStatusSleep(){
  return m_statusSleep;
}

double ta::common::AppTimer::getRealTimerMil()
{
  return (m_startAc - m_sleepAc);
}

double ta::common::AppTimer::getRealTimerSec()
{
  return (getTimerSec() - getSleepSec());
}

double ta::common::AppTimer::getRealTimerMin()
{
  return (getTimerMin() - getSleepMin());
}

double ta::common::AppTimer::getRealTimerHou()
{
  return (getTimerHou() - getSleepHou());
}