/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file AppTimer.h

  \brief This class contains attributes and functions that can be used to manipulate time.
*/

#ifndef __TERRAAMAZON_COMMON_APPTIMER_H
#define __TERRAAMAZON_COMMON_APPTIMER_H

namespace ta
{
  namespace common
  {
    /*!
    \class AppTimer

    \brief 

    */
    class AppTimer
    {

      public:

        /*!
          \brief Constructor.
        */
        AppTimer();

        /*!
          \brief Destructor.
        */
        ~AppTimer();

      public:

        /** This method clean attributs of class.
        \return Returns Nothing. */
        void cleanAttributs();

        //@{

        /** This method start the timer of work.
        \param timerSleepMilliseconds	- Maximum timer that user can to do without interaction of sistem. (Input)
        \return Returns Nothing. */
        void startTimer(double timerSleepMilliseconds);

        /** This method stop the timer of work.
        \return Returns Nothing. */
        void stopTimer();

        /** This method get the execution timer in millisecond.
        \return Returns Timer in Milliseconds. */
        double getTimerMil();

        /** This method get the execution timer in Second.
        \return Returns Timer in Seconds. */
        double getTimerSec();

        /** This method get the execution timer in minute.
        \return Returns Timer in Minute. */
        double getTimerMin();

        /** This method get the execution timer in hour.
        \return Returns Timer in Hour. */
        double getTimerHou();

        /** This method informs if the timer was started.
        \return Return true if the timer was started.*/
        bool getStatusTimer();

        //@}

        //@{

        // Methods Sleep Timer

        /** This method start the timer in sleep.
        \return Returns Nothing. */
        void startSleep();

        /** This method stop the timer in sleep.
        \return Returns Nothing. */
        void stopSleep();

        /** This method get the sleep timer in millisecond.
        \return Returns Timer in Millisecond. */
        double getSleepMil();

        /** This method get the sleep timer in second.
        \return Returns Timer in Second. */
        double getSleepSec();

        /** This method get the sleep timer in minute.
        \return Returns Timer in Minute. */
        double getSleepMin();

        /** This method get the sleep timer in hour.
        \return Returns Timer in Hour. */
        double getSleepHou();

        /** This method informs if the sleep timer was started.
        \return Return true if the sleep timer was started.*/
        bool getStatusSleep();

        //@}

        //@{

        // Methods Real Timer

        /** This method get the real timer in millisecond.
        \return Returns Timer in Millisecond. */
        double getRealTimerMil();

        /** This method get the real timer in second.
        \return Returns Timer in Second. */
        double getRealTimerSec();

        /** This method get the real timer in minute.
        \return Returns Timer in Minute. */
        double getRealTimerMin();

        /** This method get the real timer in hour.
        \return Returns Timer in Hour. */
        double getRealTimerHou();

        //@}

      protected:

        /** This method get current timer of sleep in milliseconds.
        \return Returns the current timer of sleep in milliseconds. */
        double timerSleep();

        /** This method get timer in milliseconds.
        \return Returns the timer in milliseconds. */
        double currentTimer();

        /** This method subtract the sleep timer end with sleep timer initial.
        \return Returns Nothing. */
        void sleepAc(double stop, double start);

      protected:

        // Attributes Timer
        double m_startAc; //!< Time End less Time Start.   
        double m_startTimer; //!< Time start of operation. 
        double m_stopTimer; //!< Time end of operation. 
        bool m_statusTimer; //!< Return the status of current Timer.	

        // Attributes Sleep Timer
        double m_sleepAc; //!< Time end less Timer Start.
        double m_startSleep; //!< Timer start of sleep.
        double m_stopSleep; //!< Timer start of sleep.
        double m_currentSleep_Stop;  //!< Stops the current sleep
        double m_timerSleepMilliseconds; //!< Maximum timer that user can to do without interaction of sistem.
        double m_auxAntSleep; //!< The last timer
        bool m_statusSleep; //!< Return the status of Sleep Timer.
    };
  } // end namespace common
}   // end namespace ta

#endif // __TERRAAMAZON_COMMON_APPTIMER_H

