/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file CalculateLayerArea.h

  \brief This file has the class used to calculate the geometries area of a layer.
*/

#ifndef __TERRAAMAZON_COMMON_CALCULATELAYERAREA_H
#define __TERRAAMAZON_COMMON_CALCULATELAYERAREA_H


// Terralib
#include <terralib/common/UnitsOfMeasureManager.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/maptools/AbstractLayer.h>


namespace ta
{
  namespace common
  {
    /*!
    \class CalculateLayerArea

    \brief Class used to calculate the geometries area of a layer.

    */
    class CalculateLayerArea
    {

      public:

        /*!
          \brief Constructor.
        */
        CalculateLayerArea();

        /*!
          \brief Destructor.
        */
         ~CalculateLayerArea();

      public:

        void setInputParameters(te::map::AbstractLayerPtr layer, te::common::UnitOfMeasurePtr unit, int srid, bool createColumnArea, std::string columnName);

        void getOutputParameters(std::size_t& polyCount, double& totArea);

        void execute();

      protected:

        bool checkParameters();

        double calculateArea(te::gm::Geometry* geom);

        double converteArea(double value);

        bool convertAngleToPlanar(double& val, te::common::UnitOfMeasurePtr planar);

        bool convertPlanarToAngle(double& val, te::common::UnitOfMeasurePtr planar);

      protected:

        te::da::DataSourcePtr m_ds;

        te::map::AbstractLayerPtr m_layer;

        te::common::UnitOfMeasurePtr m_unit;

        int m_srid;

        bool m_createColumnArea;

        std::string m_columnName;

        std::string m_errorMessage;

        std::size_t m_polyCount;

        double m_totArea;
    };
  } // end namespace common
}   // end namespace ta

#endif // __TERRAAMAZON_COMMON_CALCULATELAYERAREA_H

