/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file TerraAmazon.h

\brief The main class of TerraAmazon.
*/

#ifndef __TERRAAMAZON_COMMON_TERRAAMAZON_H
#define __TERRAAMAZON_COMMON_TERRAAMAZON_H

// TerraAmazon
#include "../common/Enums.h"
#include "../edition/qt/EditToolBar.h"
#include "Config.h"

// TerraLib
#include <terralib/qt/af/BaseApplication.h>
#include "TerraAmazonController.h"

// STL
#include <string>

namespace te
{
  namespace qt
  {
    namespace af
    {
      class DataSetTableDockWidget;
    }
    namespace widgets
    {
      class ChartDisplayWidget;
      class CompositionModeMenuWidget;
      class HelpManagerImpl;
    }
  }
}

namespace ta
{
  namespace workspace
  {
    struct WorkspaceMetadata;
  }

  namespace common
  {
    namespace qt { class ConnectLayersDockWidget; }
    namespace qt { class StatusBarInfoWidget; }

    /*!
    \brief The main class of TerraAmazon.

    \sa te::qt::af::BaseApplication
    */
    class TerraAmazon : public te::qt::af::BaseApplication
    {
      Q_OBJECT

    public:

      TerraAmazon(QWidget* parent = 0);

      ~TerraAmazon();

      void init();

      void startWorkspace(const QString& wsFileName);

      bool showWelcome();

    protected:

      virtual void makeDialog();

      virtual void initActions();

      virtual void initMenus();

      virtual void initSlotsConnections();

      void addMenusActions();

      void addPopUpMenu();

      void initToolbars();

      void initInfoStatusBar();

      void addActions(const QString& name, const QString& category, const QList<QAction*>& acts);

      bool checkServer();

    protected slots:

      void onApplicationTriggered(te::qt::af::evt::Event* e);

      void onNewWorkspaceTriggered();

      void onOpenWorkspaceTriggered();

      void onSaveWorkspaceTriggered();

      void onSaveWorkspaceAsTriggered();

      void onRecentWorkspacesTriggered(QAction* ws);

      void onRestartSystemTriggered();

      void onWorkspaceServerControlTriggered();

      void onWorkspacePropertiesTriggered();

      void onUserControlTriggered();

      void onAddServerTriggered();

      void onGroupControlTriggered();

      void onPhaseControlTriggered();

      void onAttributeControlTriggered();

      void onProjectControlTriggered();

      void onRulesControlTriggered();

      void onClassControlTriggered();

      void onProjectManagementWizardTriggered();

      void onAOIManagementWizardTriggered();

      void onProjCreateFolderTriggered();
      
      void onProjInformationTriggered();


      void onCreateDraftLayerTriggered();


      void onPluginsManagerTriggered();

      void onAddDataSetLayerTriggered();

      void onAddQueryLayerTriggered();

      void onAddTabularLayerTriggered();

      void onAddFolderLayerTriggered();


      void onProjectActivationTriggered();

      void onProcessControlTriggered();

      void onProcessCellManagerTriggered();

      void onCreateCanceptualModelTriggered();

      void onAdminDataExchangerTriggered();

      void onAdminQueryDatasourceTriggered();
      
      void onHelpContentsTriggered();

      void onHelpAboutTriggered();
      
      void onShowWelcomeDialogToggled(bool status);


      void onCatalogManagerTriggered();

      void closeEvent(QCloseEvent *event);

      void onEnableEditTriggered(bool checked);
      
      void onLayerDataExchangerTriggered();

      void onLayerEditLegendTriggered();
      
      void onLayerQueryTriggered();

      void onLayerCompositionModeTriggered();

      void onLayerCalculateAreaTriggered();

      void onCreateNewLayer(te::map::AbstractLayerPtr layer);

      void onEnableVisualizationTools(bool enable);

    protected:

      void workspaceChanged();

      void checkAndSaveWorkspace();

      void openWorkspace(const QString& wsFileName);

      void configureTerraAmazon(const ta::common::TaUserType& type);

      void setWindowTitle(bool hasChanged = false);

    protected:

      // File Menu
      QMenu* m_fileMenu;

      QAction* m_fileNewWorkspace;
      QAction* m_fileOpenWorkspace;
      QAction* m_fileSaveWorkspace;
      QAction* m_fileSaveWorkspaceAs;
      QAction* m_filePrintPreview;
      QAction* m_filePrint;
      QMenu* m_fileRecentWorkspacesMenu;
      QAction* m_fileRestartSystem;
      QAction* m_fileExit;

      // Show Menu
      QMenu* m_showMenu;
      QMenu* m_showToolBarsMenu;
      QAction* m_showConnectLayers;

      // Map Menu
      //QMenu* m_mapMenu;

      // Layer Menu
      //QMenu* m_layerMenu;
      QAction* m_layerQuery;
      QAction* m_layerDataExchangerDirectPopUp;

      // Workspace
      QMenu* m_workspaceMenu;

      QAction* m_workspaceAddLayerDataset;
      QAction* m_workspaceAddLayerQueryDataSet;
      QAction* m_workspaceAddLayerTabularDataSet;
      QAction* m_workspaceAddLayerGraph;
      QAction* m_workspaceAddFolderLayer;
      QAction* m_workspaceChangeLayerDataSource;
      QAction* m_workspaceUpdateLayerDataSource;
      QAction* m_workspaceProperties;

      QMenu* m_workspaceAddLayerMenu;

      // Process Menu
      QMenu* m_processMenu;
      QAction* m_processServerControl;
      QAction* m_processProjectActivation;
      QAction* m_processControl;
      QAction* m_processCellsManager;

      // Administration Menu
      QMenu* m_administrationMenu;

      QAction* m_adminCreateConceptualModel;
      QAction* m_adminDataExchanger;
      QAction* m_adminQueryDataSource;

      //Project Management Menu
      QMenu* m_projManagementMenu;

      QAction* m_projManagementUserControl;
      QAction* m_projManagementGroupControl;
      QAction* m_projManagementPhaseControl;
      QAction* m_projManagementAttributeControl;
      QAction* m_projProjectControl;
      QAction* m_projRulesControl;
      QAction* m_projClassControl;
      QAction* m_projProjectManagementWizard;
      QAction* m_projAOIManagementWizard;
      QAction* m_projCatalogManager;
      QAction* m_projCreateFolder;
      QAction* m_projInformation;

      QAction* m_createDraftLayer;


      //Settings Menu
      QMenu* m_settingsMenu;

      QAction* m_settingsShowWelcomeDlg;

      // Plugins Menu
      QMenu* m_pluginsMenu;
      QAction* m_pluginsManager;

      // Help Menu
      QMenu* m_helpMenu;
      QAction* m_helpAbout;
      QAction* m_helpContents;
      QAction* m_helpUpdate;

      // Edition Tools Action
      QAction* m_editionToolsAction;

      // Only popup menu
      QAction* m_layerEditLegend;
      QAction* m_layerCompositionMode;
      QAction* m_layerCalculateArea;

      QSize m_mapCursorSize;

      std::vector<te::qt::af::DataSetTableDockWidget*> m_tables;

      TerraAmazonController* m_taController;

      te::qt::widgets::CompositionModeMenuWidget* m_compModeMenu;

      ta::workspace::WorkspaceMetadata* m_workspace;

      ta::edition::qt::EditToolBar* m_editToolbar;

      ta::common::qt::ConnectLayersDockWidget* m_connectLayersDock;

      ta::common::qt::StatusBarInfoWidget* m_statusBarInfoWidget;

    };
  } // end namespace common
}   // end namespace ta

#endif  // __TERRAAMAZON_COMMON_TERRAAMAZON_H
