/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file AboutDialog.cpp

  \brief A Qt dialog showing TerraAmazon about window.
*/

// TerraAmazon
#include "../../Version.h"
#include "../Utils.h"
#include "../Version.h"
#include "AboutDialog.h"
#include "ui_AboutDialogForm.h"

// TerraLib
#include <terralib/common/Version.h>
#include <terralib/qt/af/ApplicationController.h>
#include <terralib/Version.h>

// Qt
#include <QtGui/QPixmap>

ta::common::qt::AboutDialog::AboutDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
    m_ui(new Ui::AboutDialogForm)
{
  m_ui->setupUi(this);

  std::string logoTVLargeFileName = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  QPixmap pixmapTVLarge(logoTVLargeFileName.c_str());
  m_ui->m_applicationLargeLogo->setPixmap(pixmapTVLarge.scaled(QSize(206, 67), Qt::KeepAspectRatio, Qt::SmoothTransformation));

  std::string copyrightStr = tr("<p>Copyright &copy; 2005-2015 INPE<BR>").toStdString();
  m_ui->m_copyrightLabel->setText(copyrightStr.c_str());

  //TerraLib Info

  std::string logoTEFileName = ta::common::FindInTerraAmazonPath("share/terralib/images/png/terralib-globe.png");
  QPixmap pixmapTE(logoTEFileName.c_str());

  pixmapTE = pixmapTE.scaled(64, 64, Qt::KeepAspectRatio, Qt::SmoothTransformation);

  m_ui->m_terralibLogo->setPixmap(pixmapTE);

  std::string terralibVersionStr = tr("TerraLib Version: ").toStdString() + te::common::Version::asString();
  m_ui->m_terralibVersionLabel->setText(terralibVersionStr.c_str());

  std::string buildDateStr = tr("Build Date: ").toStdString() + te::common::Version::buildDate();
  m_ui->m_buildDateLabel->setText(buildDateStr.c_str());

  //TerraAmazon info

  std::string logoTAFileName = ta::common::FindInTerraAmazonPath("share/icons/png/terraAmazon-logo.png");
  QPixmap pixmapTA(logoTAFileName.c_str());

  pixmapTA = pixmapTA.scaled(64, 64, Qt::KeepAspectRatio, Qt::SmoothTransformation);

  m_ui->m_terraamazonLogo->setPixmap(pixmapTA);

  std::string terraamazonVersionStr = tr("TerraAmazon Version: ").toStdString() + ta::common::Version::asString();
  m_ui->m_terraamazonVersionLabel->setText(terraamazonVersionStr.c_str());

  std::string terraAmazonbuildDateStr = tr("Build Date: ").toStdString() + ta::common::Version::buildDate();
  m_ui->m_buildTADateLabel->setText(terraAmazonbuildDateStr.c_str());
}

ta::common::qt::AboutDialog::~AboutDialog()
{
}
