/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

#ifndef __TERRAAMAZON_COMMON_QT_CONNECTLAYERSDOCKWIDGET_H
#define __TERRAAMAZON_COMMON_QT_CONNECTLAYERSDOCKWIDGET_H

// TerraAmazon
#include "Config.h"

// TerraLib
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/qt/af/connectors/MapDisplay.h>
#include <terralib/qt/widgets/layer/explorer/LayerItemView.h>

// STL
#include <string>
#include <map>
#include <memory>

// Qt
#include <QDockWidget>
#include <QListWidgetItem>

namespace Ui { class ConnectLayersDockWidgetForm; }

namespace ta
{
  namespace common
  {
    namespace qt
    {
      class ConnectLayersDockWidget : public QDockWidget
      {
        Q_OBJECT

      public:

        ConnectLayersDockWidget(QWidget* parent = 0);

        ~ConnectLayersDockWidget();

        void setApplicationComponents(te::qt::widgets::LayerItemView* layerExplorer, te::qt::af::MapDisplay* mapDisplay);

        void setLayers(std::list<te::map::AbstractLayerPtr> layers);

      public slots:

        void onMapDisplayExtentChanged();

        void onDisplayPaintEvent(QPainter* painter);

        void onConnectLayersToolButtonClicked();

        void onDrawToolButtonClicked();

        void onUpToolButtonClicked();

        void onDownToolButtonClicked();

        void onAlphaSliderChanged(int value);

        void onVSliderChanged(int value);

        void onHSliderChanged(int value);

        void onFlickerTolButtonClicked(bool flag);

        void onBoxToolButtonClicked(bool flag);

        void onConnectLayerBoxDefined(QRect rect);

        void onConnectLayerBoxUpLayer();

        void onConnectLayerBoxDownLayer();

        void onConnectLayerBoxFlickLayer();

      protected:

        void drawLayers();

        void updateSliderBarSize();

        void clearPixmaps();

        void checkLayerItem();

      private:

        std::auto_ptr<Ui::ConnectLayersDockWidgetForm> m_ui;

        te::qt::widgets::LayerItemView* m_layerExplorer;

        te::qt::af::MapDisplay* m_mapDisplay;

        std::vector<QPixmap*> m_layersPixmaps;

        std::map<int, QListWidgetItem*> m_layersItems;

        bool m_isConnected;

        bool m_flick;

        std::size_t m_curLayerPix;

        QRect m_rect;
      };
    } // end namespace qt
  }   // end namespace common
}     // end namespace ta



#endif  // __TERRAAMAZON_COMMON_QT_CONNECTLAYERSDOCKWIDGET_H
