/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

// TerraAmazon
#include "CreateCellLayerUtils.h"
#include "../Utils.h"
#include "../../model/facade/UserFacade.h"
#include "../../model/facade/CatalogFacade.h"
#include "../../model/item/User.h"
#include "../../model/item/Catalog.h"
#include "../../model/Globals.h"

// TerraLib
#include <terralib/cellspace/CellSpaceOperations.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/datatype/StringProperty.h>
#include <terralib/datatype/TimeInstant.h>
#include <terralib/geometry/Envelope.h>
#include <terralib/memory/DataSet.h>
#include <terralib/se/Utils.h>


// STL
#include <fstream>
#include <memory>

// Boost
#include <boost/lexical_cast.hpp>


void ta::common::CreateCellLayer(te::da::DataSourceInfoPtr targetDs, const std::string& newCellLayerName, const te::gm::Envelope& env, const double& res, const int& srid)
{
  te::cellspace::CellularSpacesOperations cellOp;
  cellOp.createCellSpace(targetDs, newCellLayerName, res, res, env, srid, te::cellspace::CellularSpacesOperations::CELLSPACE_POLYGONS, 0);
}

void ta::common::AdaptCellDataSet(te::da::DataSourcePtr source, const std::string& cellName)
{
  std::unique_ptr<te::da::DataSetType> cellDst = source->getDataSetType(cellName);
  std::unique_ptr<te::da::DataSet> cellDs = source->getDataSet(cellName);

  te::dt::Property* cellOid = new te::dt::StringProperty("cell_oid", te::dt::VAR_STRING, 255);
  te::dt::Property* processId = new te::dt::SimpleProperty("process_id", te::dt::INT32_TYPE);

  cellDst->add(cellOid);
  cellDst->add(processId);

  source->addProperty(cellDst->getName(), cellOid);
  source->addProperty(cellDst->getName(), processId);

  cellDs->moveBeforeFirst();

  std::unique_ptr<te::da::DataSourceTransactor> t = source->getTransactor();

  try
  {
    t->begin();

    while (cellDs->moveNext())
    {
      std::string id = cellDs->getString("id");
      int col = cellDs->getInt32("col");
      int row = cellDs->getInt32("row");

      std::string result = boost::lexical_cast<std::string>(col)+":" + boost::lexical_cast<std::string>(row);

      //TODO: create class Update at dataaccess/query to do this operations
      std::string sql = "UPDATE " + cellDst->getName() + " SET cell_oid = '" + result + "' WHERE id = '" + id + "';";

      t->execute(sql);
    }

    t->commit();
  }
  catch (const std::exception& e)
  {
    t->rollBack();

    throw e;
  }
}

void ta::common::PrepareCatalogCellLayer(te::da::DataSourceInfoPtr targetDs, const std::string& cellLayerName, const std::string& source, ta::model::Catalog* catalog)
{
  //create default syle
  std::string strStyle = "";
  std::unique_ptr<te::se::Style> style(te::se::CreateFeatureTypeStyle(te::gm::PolygonType));

  if (style.get())
  {
    strStyle = ta::common::Write(style.get());
  }

  catalog->setName("public." + cellLayerName);
  catalog->setUri(targetDs->getConnInfoAsString());
  catalog->setType(targetDs->getType());
  catalog->setAuthor("unknown user");
  catalog->setSource(source);
  catalog->setPermission(1);
  catalog->setStyle(strStyle);
  catalog->setLocked(true);
  catalog->setCellColumn("cell_oid");

  boost::posix_time::ptime boostLocalTime = boost::posix_time::second_clock::local_time();
  te::dt::TimeInstant* localTime = new te::dt::TimeInstant(boostLocalTime);
  catalog->setLastEdition(localTime);

  catalog->setScaleMin(1);
  catalog->setScaleMax(1000000);
}

