/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

#ifndef __TERRAAMAZON_COMMON_QT_CREATECELLLAYERWIDGET_H
#define __TERRAAMAZON_COMMON_QT_CREATECELLLAYERWIDGET_H

// TerraAmazon
#include "Config.h"

// TerraLib
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/maptools/AbstractLayer.h>

// STL
#include <string>

// Qt
#include <QDialog>

namespace Ui { class CreateCellLayerWidgetForm; }

namespace ta
{
  namespace common
  {
    namespace qt
    {
      class CreateCellLayerWidget : public QWidget
      {
        Q_OBJECT

      public:

        CreateCellLayerWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~CreateCellLayerWidget();

        void setDataSources(std::vector<te::da::DataSourceInfoPtr> sources);

        void setLayers(std::list<te::map::AbstractLayerPtr> layers);

        te::gm::Envelope getEnvelope();

        void setEnvelope(const te::gm::Envelope& env);

        double getResolution();

        void setResolution(const double& res);

        int getSrid();

        void setSrid(int srid);

        te::da::DataSourceInfoPtr getTargetDataSource();

        bool validateParameters(QString& error);

        te::da::DataSourceInfoPtr getCurrentDataSourceInfoPtr();

        std::string getCurrentCellLayerName();

        double getCurrentResolution();

        void setActivationToDataSourceSelector(const bool& active);

        std::string getUserName();

      private:

        void loadCountries();

        void clearEnvelope();

      protected slots:

        void onDataSourceCreatorToolButtonClicked(bool);

        void onDataSourceSelectorToolButtonClicked(bool);

        void onProjectionPushButtonClicked();

        void onRefLayerBoxCheckBoxToggled(bool checked);

        void onCountryBoxCheckBoxToggled(bool checked);

        void onRefLayerBoxComboBoxActivated(int idx);

        void onCountryBoxComboBoxActivated(int idx);

        void onResolutionLineEditTextChanged(const QString & text);

      private:

        std::auto_ptr<Ui::CreateCellLayerWidgetForm> m_ui;

        int m_countriesSrid;

        int m_srid;

      };
    } // end namespace qt
  }   // end namespace common
}     // end namespace ta

#endif  // __TERRAAMAZON_COMMON_QT_CREATECELLLAYERWIDGET_H
