/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

//TerraAmazon Includes
#include "../../workspace/qt/ServerLoginDialog.h"
#include "../../model/facade/UserFacade.h"
#include "../Utils.h"
#include "WelcomeDialog.h"
#include "ui_WelcomeDialogForm.h"

#include <QLabel>
#include <QLineEdit>
#include <QMessageBox>
#include <QPixmap>

ta::common::qt::WelcomeDialog::WelcomeDialog(QWidget* parent)
  : QDialog(parent/*, Qt::CustomizeWindowHint | Qt::WindowTitleHint*/),
  m_ui(new Ui::WelcomeDialogForm)
{
  // add controls
  m_ui->setupUi(this);

  m_ui->m_souDevelToolButton->setIcon(QIcon::fromTheme("wand"));
  
  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_logoLabel->setPixmap(QIcon(icon.c_str()).pixmap(200, 100));

  icon = ta::common::FindInTerraAmazonPath("share/icons/svg/ta-gis-user.svg");
  m_ui->m_gisToolButton->setIcon(QIcon(icon.c_str()));
  m_ui->m_gisToolButton->setIconSize(QSize(120, 170));

  icon = ta::common::FindInTerraAmazonPath("share/icons/svg/ta-operator-user.svg");
  m_ui->m_operatorToolButton->setIcon(QIcon(icon.c_str()));
  m_ui->m_operatorToolButton->setIconSize(QSize(120, 170));

  icon = ta::common::FindInTerraAmazonPath("share/icons/svg/ta-admin-user.svg");
  m_ui->m_adminToolButton->setIcon(QIcon(icon.c_str()));
  m_ui->m_adminToolButton->setIconSize(QSize(60, 80));

  icon = ta::common::FindInTerraAmazonPath("share/icons/svg/ta-admin-creator.svg");
  m_ui->m_createProjectToolButton->setIcon(QIcon(icon.c_str()));
  m_ui->m_createProjectToolButton->setIconSize(QSize(60, 80));

  connect(m_ui->m_gisToolButton, SIGNAL(clicked(bool)), this, SLOT(onGisUserToolButtonClicked(bool)));
  connect(m_ui->m_operatorToolButton, SIGNAL(clicked(bool)), this, SLOT(onOperatorUserToolButtonClicked(bool)));
  connect(m_ui->m_adminToolButton, SIGNAL(clicked(bool)), this, SLOT(onAdminUserToolButtonClicked(bool)));
  connect(m_ui->m_createProjectToolButton, SIGNAL(clicked(bool)), this, SLOT(onCreateProjectToolButtonClicked(bool)));
  connect(m_ui->m_souDevelToolButton, SIGNAL(clicked(bool)), this, SLOT(onsouDevelToolButtonClicked(bool)));

  m_option = ta::common::TA_GIS_USER;
  m_userId = -1;
  m_loginDialog.reset(new ta::workspace::qt::ServerLoginDialog(this));

  m_ui->m_souDevelToolButton->setVisible(false);
}

ta::common::qt::WelcomeDialog::~WelcomeDialog()
{

}

ta::common::TaUserType ta::common::qt::WelcomeDialog::getOption()
{
  return m_option;
}

bool ta::common::qt::WelcomeDialog::showWelcomeDlg()
{
  return m_ui->m_rememberCheckBox->isChecked();
}

void ta::common::qt::WelcomeDialog::setServers(std::vector<te::da::DataSourceInfoPtr> servers)
{
  if (!servers.empty())
    m_loginDialog->setServers(servers);
}

te::da::DataSourceInfoPtr ta::common::qt::WelcomeDialog::getActiveServer()
{
  return m_activeServer;
}

int ta::common::qt::WelcomeDialog::getActiveUserId()
{
  return m_userId;
}

void ta::common::qt::WelcomeDialog::onGisUserToolButtonClicked(bool)
{
  m_option = ta::common::TA_GIS_USER;
  accept();
}

void ta::common::qt::WelcomeDialog::onOperatorUserToolButtonClicked(bool)
{
  m_option = ta::common::TA_OPERATOR_USER;

  //show connect
  if (m_loginDialog->exec() == QDialog::Accepted)
  {
    m_activeServer = m_loginDialog->getActiveServer();
    m_userId = m_loginDialog->getActiveUserId();

    te::da::DataSourcePtr dsPtr = te::da::DataSourceManager::getInstance().get(m_activeServer->getId(), m_activeServer->getType(), m_activeServer->getConnInfo());
    std::unique_ptr<ta::model::UserFacade> uf(new ta::model::UserFacade(dsPtr));
    int privilege = uf->getPrivilege(m_userId);
    if (privilege < 1)
    {
      QMessageBox::information(this, windowTitle(), tr("The current user does not have the requested permission."));
      return;
    }

    accept();
  }
}

void ta::common::qt::WelcomeDialog::onAdminUserToolButtonClicked(bool)
{
  m_option = ta::common::TA_ADMIN_USER;

  //show connect
  if (m_loginDialog->exec() == QDialog::Accepted)
  {
    m_activeServer = m_loginDialog->getActiveServer();
    m_userId = m_loginDialog->getActiveUserId();

    te::da::DataSourcePtr dsPtr = te::da::DataSourceManager::getInstance().get(m_activeServer->getId(), m_activeServer->getType(), m_activeServer->getConnInfo());
    std::unique_ptr<ta::model::UserFacade> uf(new ta::model::UserFacade(dsPtr));
    int privilege = uf->getPrivilege(m_userId);
    if (privilege < 7)
    {
      QMessageBox::information(this, windowTitle(), tr("The current user does not have the requested permission."));
      return;
    }

    accept();
  }
}

void ta::common::qt::WelcomeDialog::onCreateProjectToolButtonClicked(bool)
{
  m_option = ta::common::TA_ADMIN_USER;

  accept();
}

void ta::common::qt::WelcomeDialog::onsouDevelToolButtonClicked(bool)
{
  m_option = ta::common::TA_ADMIN_USER;

  accept();
}
