﻿#include <terralib/common/PlatformUtils.h>
#include <terralib/qt/widgets/layer/explorer/LayerItem.h>

// Terraamazon
#include "EditDelegate.h"
#include "../../common/Utils.h"
#include "Utils.h"

// QT
#include <QPainter>

QPainterPath GetStar()
{
  QPainterPath star, aux;

  aux.setFillRule(Qt::WindingFill);

  aux.moveTo(1.0, 0.5);
  for(int i = 1; i < 5; ++i)
  {
    QPointF pt(0.5 + 0.5 * cos(0.8 * i * 3.14),
                0.5 + 0.5 * sin(0.8 * i * 3.14));
    aux.lineTo(pt);
    star = aux.simplified();
  }

  return star;
}

QPainterPath RecalculateMark(const QPainterPath& markShape, const int& markSize)
{
  QMatrix m;
  m.scale(markSize*2, markSize*2);

  return m.map(markShape);
}

QImage GetStarImage(const QSize& size, const QColor& c)
{
  QImage img(size, QImage::Format_ARGB32_Premultiplied);

  img.fill(Qt::color0);

  QPainter p(&img);
  QPainterPath star = GetStar();
  star = RecalculateMark(star, size.height()/2);

  p.translate(img.rect().center()-star.boundingRect().center());

  p.fillPath(star, Qt::color1);
  p.setPen(QPen(Qt::color1, 1));
  p.drawPath(star);

  p.fillPath(star, c);
  p.setPen(QPen(Qt::black, 1));
  p.drawPath(star);

  return img;
}

QIcon GetIcon(const bool& stash)
{
  QPixmap p1 = QIcon::fromTheme("dataset-layer").pixmap(20, 20);
  QPixmap p2;

  if (stash)
    p2 = QPixmap(ta::common::FindInTerraAmazonPath("share/icons/png/lock.png").c_str()).scaled(17, 17, Qt::KeepAspectRatio);
  else
    p2 = QPixmap(te::common::FindInTerraLibPath("resources/images/png/mem_edit.png").c_str()).scaled(17, 17, Qt::KeepAspectRatio);

  QPainter pt(&p1);

  pt.drawPixmap(3, 3, p2);

  pt.end();

  return p1;
}


EditDelegate::EditDelegate(QStyledItemDelegate* decorated, QObject* parent) :
  te::common::Decorator<QStyledItemDelegate>(decorated)
{
  setParent(parent);

  m_stash = GetIcon(true);
  m_memory = GetIcon(false);
}

EditDelegate::~EditDelegate()
{}

void EditDelegate::paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
  std::string connectionlayer = "";
  te::qt::widgets::TreeItem* item = static_cast<te::qt::widgets::TreeItem*>(index.internalPointer());
  QStyleOptionViewItem opt(option);

  if(item->getType() == "LAYER")
  {
    te::map::AbstractLayerPtr l = ((te::qt::widgets::LayerItem*)item)->getLayer();

    te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(l->getDataSourceId());

    if (dsInfo.get())
      connectionlayer = ta::edition::qt::GetURICustomized(l->getTitle(), dsInfo);

    if (m_stashed.find(QString(connectionlayer.c_str())) != m_stashed.end())
    {
      opt.icon = m_stash; 
      opt.palette.setColor(QPalette::Text, QColor(255, 0, 0, 128));
      opt.font.setBold(true);
    }
    else if (m_edited.find(QString(connectionlayer.c_str())) != m_edited.end())
    {
      opt.palette.setColor(QPalette::Text, Qt::blue);
      opt.font.setBold(true);
    }
  }

  if(m_decorated != 0)
    m_decorated->paint(painter, opt, index);
  else
    QStyledItemDelegate::paint(painter, opt, index);
}

void EditDelegate::addStashed(const std::string& lName)
{
  m_stashed.insert(QString(lName.c_str()));
  removeEdited(lName);
}

void EditDelegate::removeStashed(const std::string& lName)
{
  m_stashed.erase(QString(lName.c_str()));
}

void EditDelegate::addEdited(const std::string& lName)
{
  m_edited.insert(QString(lName.c_str()));
  removeStashed(lName);
}

void EditDelegate::removeEdited(const std::string& lName)
{
  m_edited.erase(QString(lName.c_str()));
}

bool EditDelegate::isStached(const std::string& lName)
{
  return (m_stashed.find(QString(lName.c_str())) != m_stashed.end());
}
