﻿/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

#ifndef __TERRAAMAZON_EDITION_QT_EDITDELEGATE_H
#define __TERRAAMAZON_EDITION_QT_EDITDELEGATE_H

// TerraLib
#include "terralib/common/Decorator.h"

// Qt
#include <QIcon>
#include <QStyledItemDelegate>

// STL
#include <set>

class EditDelegate: public te::common::Decorator<QStyledItemDelegate>
{
  Q_OBJECT

public:

  EditDelegate(QStyledItemDelegate* decorated, QObject* parent = 0);

  ~EditDelegate();

  void paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const;

  void addStashed(const std::string& lName);

  void removeStashed(const std::string& lName);

  void addEdited(const std::string& lName);

  void removeEdited(const std::string& lName);

  bool isStached(const std::string& lName);


protected:

  struct InsensitiveCompare {
    bool operator() (const QString& a, const QString& b) const {
      return (a.compare(b, Qt::CaseInsensitive) < 0);
    }
  };
  
  std::set<QString, InsensitiveCompare> m_stashed;
  std::set<QString, InsensitiveCompare> m_edited;

  QIcon m_stash;
  QIcon m_memory;

};

#endif //__TERRAAMAZON_EDITION_QT_EDITDELEGATE_H