/*  This file is part of the TerraAmazon application.

  See the Copyright file for details about the license of
  this application.
*/

#ifndef __TERRAAMAZON_EDITION_QT_UTILS_H
#define __TERRAAMAZON_EDITION_QT_UTILS_H

// TerraLib
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/edit/Utils.h>
#include <terralib/geometry/Coord2D.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>
#include <terralib/qt/widgets/canvas/MapDisplay.h>
#include <terralib/qt/widgets/layer/explorer/LayerItem.h>
#include <terralib/qt/widgets/layer/explorer/LayerItemView.h>
#include <terralib/qt/widgets/layer/explorer/TreeItem.h>

// TerraAmazon
#include "Config.h"
#include "../../model/item/Process.h"

// STL
#include <set>
#include <string>
#include <vector>

namespace ta
{
  namespace edition
  {
    namespace qt
    {
      /*!
      \enum CellStyle

      \brief The cell style identification.
      */
      enum CellStyle
      {
        MYSELECTEDCELLS = 1,
        SELECTEDCELLSTOOL = 2,
        OTHERSELECTEDCELLS = 3
      };

      /*!
      \brief verify if the layer has geometries.
      */
      void HasGeometriesInMemory(const te::map::AbstractLayerPtr& layer, te::qt::widgets::MapDisplay* display);

      /*!
      \brief save the features.
      */
      void SaveGeometries(const te::map::AbstractLayerPtr& layer, te::qt::widgets::MapDisplay* display);

      /*!
      \brief remove the features.
      */
      void RemoveGeometries(const te::map::AbstractLayerPtr& layer, te::qt::widgets::MapDisplay* display);

      /*!
      \brief get selected layer.
      */
      te::map::AbstractLayer* GetSelectedLayerTreeView(QTreeView* view);

      /*!
      \brief get editable layers name from catalog.
      */
      std::set<std::string> GetLockedLayers(te::da::DataSourcePtr server);

      /*!
      \brief move mapdisplay to Interest Area.
      */
      void MoveToInterestArea(te::qt::widgets::MapDisplay* display, const  te::map::AbstractLayerPtr& layerAI,
                              te::da::DataSourcePtr server, int processID);

      /*!
      \brief get customizable uri from DataSourceInfoPtr
      */
      std::string GetURICustomized(std::string title, te::da::DataSourceInfoPtr dsInfo);

      /*!
      \brief setup cell color
      */
      void SetupCellStyle(QColor& m_fillColor, QColor& m_contourColor, std::size_t& m_contourWidth, CellStyle cellStyle);


    }
  }

}

#endif  // __TERRAAMAZON_EDITION_QT_UTILS_H
