/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

// TerraLib
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/edit/RepositoryManager.h>
#include <terralib/edit/Utils.h>
#include <terralib/edit/qt/Utils.h>
#include <terralib/geometry/Envelope.h>
#include <terralib/geometry/Geometry.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/qt/af/events/LayerEvents.h>
#include <terralib/qt/widgets/canvas/MapDisplay.h>

// TerraAmazon
#include "DeleteTool.h"
#include "edition/qt/Utils.h"

// Qt
#include <QMessageBox>

ta::edition::qt::tools::DeleteTool::DeleteTool(te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, QObject* parent)
: GeometriesUpdateTool(display, layer.get() , parent)
{}

ta::edition::qt::tools::DeleteTool::~DeleteTool()
{}

void ta::edition::qt::tools::DeleteTool::execute()
{
  const te::da::ObjectIdSet* objSet = m_layer->getSelected();

  if (!objSet)
    return;

  std::auto_ptr<te::da::DataSet> ds(m_layer->getData(objSet));

  std::auto_ptr<te::da::DataSetType> dt = m_layer->getSchema();

  te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(dt.get());

  std::vector<std::string> pnames;
  te::da::GetOIDPropertyNames(m_layer->getSchema().get(), pnames);

  while (ds->moveNext())
  {
    std::auto_ptr<te::gm::Geometry> geom(ds->getGeometry(geomProp->getName()));
    if (geom.get())
    {
      te::edit::Feature* feature = new te::edit::Feature(te::da::GenerateOID(ds.get(), pnames), geom.release(), te::edit::GEOMETRY_DELETE);

      te::edit::RepositoryManager::getInstance().addFeature(m_layer->getId(),feature->clone());
    }
  }

  m_layer->clearSelected();

  // remove the geometries in the database
  RemoveGeometries(m_layer, m_display);

}