/*  This file is part of the TerraAmazon application.

  See the Copyright file for details about the license of
  this application.

*/
#ifndef __TERRAAMAZON_EDITION_QT_TOOLS_DELETETOOL_H
#define __TERRAAMAZON_EDITION_QT_TOOLS_DELETETOOL_H

// TerraLib
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/edit/Feature.h>
#include <terralib/edit/qt/tools/GeometriesUpdateTool.h>

// TerraAmazon
#include "Config.h"

// STL
#include <vector>

namespace te
{
  namespace gm
  {
    class Geometry;
  }

  namespace qt
  {
    namespace widgets
    {
      class MapDisplay;
    }
  }
}

namespace ta
{
  namespace edition
  {
    namespace qt
    {
      namespace tools
      {

        /*!
          \class SelectCellTool

          \brief This class implements a concrete tool to select cells.
          */
        class DeleteTool : public te::edit::GeometriesUpdateTool
        {
          Q_OBJECT

          public:

            /** @name Initializer Methods
              *  Methods related to instantiation and destruction.
              */
            //@{

            /*!
              \brief It constructs a select cells tool associated with the given map display.

              \param display The map display associated with the tool.
              \param parent The tool's parent.

              \note The tool will NOT take the ownership of the given pointers.
              */
            DeleteTool(te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, QObject* parent = 0);

            /*! \brief Destructor. */
            ~DeleteTool();

            /*
            \brief Main function.
            */
            void execute();
        };
      }
    }
  }
}

#endif  // __TERRAAMAZON_EDITION_QT_TOOLS_DELETETOOL_HH

  