/*  This file is part of the TerraAmazon application.

  See the Copyright file for details about the license of
  this application.

*/
#ifndef __TERRAAMAZON_EDITION_QT_TOOLS_RECLASSIFY_H
#define __TERRAAMAZON_EDITION_QT_TOOLS_RECLASSIFY_H

// TerraLib
#include <terralib/geometry/Coord2D.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/edit/qt/tools/GeometriesUpdateTool.h>

#include "Config.h"

// STL
#include <vector>

namespace te
{
  namespace gm
  {
    class Geometry;
  }

  namespace qt
  {
    namespace widgets
    {
      class MapDisplay;
    }
  }
}

namespace ta
{
  namespace edition
  {
    namespace qt
    {
      namespace tools
      {
        /*!
          \class Reclassify

          \brief This class implements a concrete reclassification tool.
          */
        class Reclassify : public te::edit::GeometriesUpdateTool
        {
          Q_OBJECT

          public:

            /** @name Initializer Methods
              *  Methods related to instantiation and destruction.
              */
            //@{

            /*!
              \brief It constructs a reclassification tool associated with the given map display.

              \param display The map display associated with the tool.
              \param parent The tool's parent.

              \note The tool will NOT take the ownership of the given pointers.
              */
            Reclassify(te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, const QCursor& cursor, QObject* parent = 0);

            /*! \brief Destructor. */
            ~Reclassify();

            //@}

            /** @name AbstractTool Methods
              *  Methods related with tool behavior.
              */
            //@{

            bool mousePressEvent(QMouseEvent* e);

            bool mouseMoveEvent(QMouseEvent* e);

            bool mouseDoubleClickEvent(QMouseEvent* e);

            //@}

          private:

            void draw();

            void storeNewGeometry();

          private slots:

            void onExtentChanged();

          protected:

            std::vector<te::gm::Coord2D> m_coords;  //!< The coord list managed by this tool.
            te::gm::Coord2D m_lastPos;              //!< The last position captured on mouse move event.
            bool m_continuousMode;                  //!< A flag that indicates if the tool is working in 'continuous mode'. i.e. the coordinates will be acquired  from each mouseMove.
            bool m_isFinished;                      //!< A flag that indicates if the operations was finished.

            void clear();
            te::gm::Geometry* buildLine();
        };
      }
    }
  }
}

#endif  // __TERRAAMAZON_EDITION_QT_TOOLS_RECLASSIFY_H

  