/*  This file is part of the TerraAmazon application.

  See the Copyright file for details about the license of
  this application.

*/
#ifndef __TERRAAMAZON_EDITION_QT_TOOLS_SELECTCELLTOOL_H
#define __TERRAAMAZON_EDITION_QT_TOOLS_SELECTCELLTOOL_H

// TerraLib
#include <terralib/edit/Feature.h>
#include <terralib/edit/qt/tools/GeometriesUpdateTool.h>
#include <terralib/geometry/Coord2D.h>
#include <terralib/maptools/AbstractLayer.h>

// TerraAmazon
#include "Config.h"
#include "../Utils.h"
#include "../../../model/item/Cell.h"

// STL
#include <vector>

namespace te
{
  namespace gm
  {
    class Geometry;
  }

  namespace qt
  {
    namespace widgets
    {
      class MapDisplay;
    }
  }
}

namespace ta
{
  namespace edition
  {
    namespace qt
    {
      namespace tools
      {

        /*!
          \class SelectCellTool

          \brief This class implements a concrete tool to select cells.
          */
        class SelectCellTool : public te::edit::GeometriesUpdateTool
        {
          Q_OBJECT

          public:

            /** @name Initializer Methods
              *  Methods related to instantiation and destruction.
              */
            //@{

            /*!
              \brief It constructs a select cells tool associated with the given map display.

              \param display The map display associated with the tool.
              \param parent The tool's parent.

              \note The tool will NOT take the ownership of the given pointers.
              */
            SelectCellTool(te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, 
                           const te::map::AbstractLayerPtr& cellLayer, std::vector<ta::model::Cell*> cells, const QCursor& cursor, QObject* parent = 0);

            /*! \brief Destructor. */
            ~SelectCellTool();

            //@}

            /** @name AbstractTool Methods
              *  Methods related with tool behavior.
              */
            //@{

            bool mousePressEvent(QMouseEvent* e);

            bool mouseReleaseEvent(QMouseEvent* e);

            //@}

            void clearSelectedCells();

            void setSelectedCells(std::vector<ta::model::Cell*> cells);

          private:

            void locateCell();

            te::gm::Envelope buildEnvelope(const QPointF& pos);

            void pickFeature(const QPointF& pos);

          signals:

            void layerSelectedCellsChanged(std::vector<ta::model::Cell*>& lockedCells, CellStyle cellStyle);

          protected:

            std::vector<ta::model::Cell*> m_lockedCells;
            te::map::AbstractLayer* m_cellLayer;

        };
      }
    }
  }
}

#endif  // __TERRAAMAZON_EDITION_QT_TOOLS_SELECTCELLTOOL_HH

  