/*  This file is part of the TerraAmazon application.

  See the Copyright file for details about the license of
  this application.

*/
#ifndef __TERRAAMAZON_EDITION_QT_TOOLS_SELECTMULTILAYERTOOL_H
#define __TERRAAMAZON_EDITION_QT_TOOLS_SELECTMULTILAYERTOOL_H

// TerraLib
#include <terralib/geometry/Coord2D.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/edit/Feature.h>
#include <terralib/edit/qt/tools/GeometriesUpdateTool.h>

// TerraAmazon
#include "Config.h"

// Qt
#include <QPen>

// STL
#include <vector>

namespace te
{
  namespace gm
  {
    class Geometry;
  }

  namespace qt
  {
    namespace widgets
    {
      class MapDisplay;
    }
  }
}

namespace ta
{
  namespace edition
  {
    namespace qt
    {
      namespace tools
      {

        /*!
          \class SelectAreaTool
          \brief This class implements a concrete tool to select geometries by drawing an area (polygon).
          */
        class SelectMultiLayerTool : public te::edit::GeometriesUpdateTool
        {
          Q_OBJECT

          public:

            /** @name Initializer Methods
              *  Methods related to instantiation and destruction.
              */
            //@{

            /*!
              \brief It constructs a select cells tool associated with the given map display.

              \param display The map display associated with the tool.
              \param parent The tool's parent.

              \note The tool will NOT take the ownership of the given pointers.
              */
            SelectMultiLayerTool(te::qt::widgets::MapDisplay* display, const std::vector<te::map::AbstractLayerPtr>& layers, const QCursor& cursor, QObject* parent = 0);

            /*! \brief Destructor. */
            ~SelectMultiLayerTool();

            //@}

            /** @name AbstractTool Methods
              *  Methods related with tool behavior.
              */
            //@{

            bool mousePressEvent(QMouseEvent* e);

            bool mouseMoveEvent(QMouseEvent* e);

            bool mouseReleaseEvent(QMouseEvent* e);

          private:

            void draw();

            void drawPolygon();

            void clear();

            te::gm::Geometry* buildPolygon();

            void executeSelection(const te::map::AbstractLayerPtr& layer, const te::gm::Envelope& e);

            te::gm::Envelope buildEnvelope(const QPointF& pos);

          Q_SIGNALS:

            void triggered(te::qt::af::evt::Event* e);

          protected:

            std::vector<te::gm::Coord2D> m_coords;  //!< The coord list managed by this tool.
            te::gm::Coord2D m_lastPos;              //!< The last position captured on mouse move event.
            bool m_continuousMode;                  //!< A flag that indicates if the tool is working in 'continuous mode'. i.e. the coordinates will be acquired  from each mouseMove.
            bool m_keepPreviousSelection;           //!< Flag that indicates if the tool must keep the previous selection.
            bool m_selectionStarted;                //!< Flag that indicates if selection operation was started.
            bool m_selectionByPointing;
            te::gm::Geometry* m_geom;
            std::vector<te::map::AbstractLayerPtr> m_layers;


        };
      }
    }
  }
}

#endif  // __TERRAAMAZON_EDITION_QT_TOOLS_SELECTMULTILAYERTOOL_H

  