/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

#ifndef __TERRAAMAZON_EDITION_QT_TOOLS_UTILS_H
#define __TERRAAMAZON_EDITION_QT_TOOLS_UTILS_H

// TerraLib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/query_h.h>

// TerraAmazon
#include "Config.h"

// STL
#include <string>
#include <vector>

namespace ta
{
  namespace model
  {
    class Rule;
    class RulesLayerColumn;
  }

  namespace edition
  {
    namespace qt
    {
      namespace tools
      {
        
        struct RulesInfo
        {
          int                                       m_id;
          ta::model::Rule*                          m_rule;
          std::vector<ta::model::RulesLayerColumn*> m_columns;
        };

        te::da::Select GetCellQuery(const std::string& catalogName,
                                    const std::string& columnName,
                                    const std::vector<te::gm::Geometry*>& cellVec);

        ta::edition::qt::tools::RulesInfo CreateImplicitRule(const int& Id,
                                                            const std::string& name,
                                                            const int& catalogId,
                                                            const int& operationId,
                                                            const std::string columnIn = "",
                                                            const std::string columnOut = "");

        std::vector<ta::edition::qt::tools::RulesInfo> GetRules(te::da::DataSourcePtr server,
                                                                const ta::model::RuleSet& ruleSet,
                                                                const bool& clean = false);

        /*!
        \brief Get all rules of a project

        \param server    The server
        \param projectId The current project id
        \param processId The current process id
        \param processId The current ruleset id
        \param clean     Define if is a clear operation. If true, only cell, aoi and clear rules are used

        return Vector of RulesInfo struct

        */
        std::vector<ta::edition::qt::tools::RulesInfo> GetRules(te::da::DataSourcePtr server, 
                                                                const int& projectId, 
                                                                const int& processId, 
                                                                const int& ruleSetId,
                                                                const bool& clean = false);

        te::da::Select GetRestrictionQuery(const std::string& catalogName,
                                           const std::string& columnName,
                                           const std::vector<std::string>& aiObjectName);

        te::gm::Polygon* GetValidPolygon(te::gm::Polygon* gm);
      }
    }
  }

}

#endif  // __TERRAAMAZON_EDITION_QT_TOOLS_UTILS_H
