/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file WorkspaceEvents.h

  \brief Contains the list of the workspace events.
*/
#ifndef __TERRAAMAZON_EVENTS_WORKSPACEEVENTS_H
#define __TERRAAMAZON_EVENTS_WORKSPACEEVENTS_H

#include <terralib/qt/af/events/Event.h>
#include <terralib/qt/af/events/Enums.h>

namespace ta
{

  // Forward declarations
  class Workspace;

  namespace evt
  {
    /*!
      \struct WorkspaceAdded

      \brief It indicates that a new workspace was created.
    */
    struct WorkspaceAdded : public te::qt::af::evt::Event
    {
      /*!
        \brief Constructor.

        \param ws The workspace to be added.
      */
      WorkspaceAdded(Workspace* ws)
        : te::qt::af::evt::Event(te::qt::af::evt::PROJECT_ADDED),
          m_ws(ws)
      {
      }

      Workspace* m_ws;    //!< Pointer to the new workspace.
    };

    /*!
      \struct WorkspaceUnsaved

      \brief It indicates that the workspace has unsaved changes.
    */
    struct WorkspaceUnsaved : public te::qt::af::evt::Event
    {
      /*!
        \brief Constructor.
      */
      WorkspaceUnsaved()
        : te::qt::af::evt::Event(te::qt::af::evt::PROJECT_UNSAVED)
      {
      }
    };
  } // end namespace evt
}   // end namespace ta

#endif //__TERRAAMAZON_EVENTS_WORKSPACEEVENTS_H
