/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/main.cpp

  \brief It contains the main routine of TerraAmazon.
*/

// TerraAmazon
#include "common/TerraAmazon.h"
#include "Config.h"

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/qt/af/Utils.h>
#include <terralib/qt/af/SplashScreenManager.h>

// STL
#include <cstdlib>
#include <exception>
#include <locale>

// Qt
#include <QApplication>
#include <QDir>
#include <QFileInfo>
#include <QLibraryInfo>
#include <QLocale>
#include <QMessageBox>
#include <QSplashScreen>
#include <QTextCodec>
#include <QTranslator>

#if (defined(__APPLE__) && defined(__MACH__)) || defined(macintosh) || defined(Macintosh)
#include <CoreFoundation/CoreFoundation.h>
#endif


int main(int argc, char** argv)
{
  QApplication app(argc, argv);

  QDir dir(QLibraryInfo::location(QLibraryInfo::TranslationsPath));

  QStringList filters;
  filters << "*" + QLocale::system().name().toLower() + ".qm";

  QFileInfoList lst = dir.entryInfoList(filters, QDir::Files);

  for (int i = 0; i<lst.size(); ++i)
  {
    QTranslator* trans = new QTranslator;
    trans->load(lst.at(i).baseName(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    app.installTranslator(trans);
  }

  setlocale(LC_ALL, "C"); // This force to use "." as decimal separator.

#if QT_VERSION >= 0x050000
  QTextCodec::setCodecForLocale(QTextCodec::codecForLocale());
#else
  QTextCodec::setCodecForCStrings(QTextCodec::codecForLocale());
#endif

  int waitVal = EXIT_FAILURE;

  const int RESTART_CODE = 1000;

  try
  {
    do
    {
      std::string splash_pix = TAMZ_SPLASH_SCREEN_PIXMAP;

      QPixmap pixmap(splash_pix.c_str());

      QSplashScreen splash(pixmap/*, Qt::WindowStaysOnTopHint*/);

      splash.setStyleSheet("QWidget { font-size: 12px; font-weight: bold }");

      te::qt::af::SplashScreenManager::getInstance().set(&splash, Qt::AlignBottom | Qt::AlignHCenter, Qt::white);

      splash.show();

      ta::common::TerraAmazon ta;

#if (defined(__APPLE__) && defined(__MACH__)) || defined(macintosh) || defined(Macintosh)
      CFBundleRef mainBundle = CFBundleGetMainBundle();
      CFURLRef execPath = CFBundleCopyBundleURL(mainBundle);

      char path[PATH_MAX];

      if (!CFURLGetFileSystemRepresentation(execPath, TRUE, (UInt8 *)path, PATH_MAX))
        throw; // error!

      CFRelease(execPath);

      QDir dPath(path);

      dPath.cd("Contents");

      chdir(dPath.path().toStdString().c_str());
#endif

      ta.init();

      splash.finish(&ta);

      ta.showMaximized();

      bool res = ta.showWelcome();

      if (res)
      {
        // Start TerraAmazon from project file(.tmz)
        if (argc > 1)
        {
          std::string wsPath = argv[1];

          if (!wsPath.empty())
            ta.startWorkspace(wsPath.c_str());
        }

        waitVal = app.exec();
      }
      else
      {
        waitVal = EXIT_SUCCESS;
      }

    } while (waitVal == RESTART_CODE);
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(0,"TerraAmazon", e.what());
    return EXIT_FAILURE;
  }
  catch (...)
  {
    return EXIT_FAILURE;
  }

  return waitVal;
}
