/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/CatalogFacade.h

\brief
*/

#ifndef __TERRAAMAZON_MODEL_FACADE_CATALOGFACADE_H
#define __TERRAAMAZON_MODEL_FACADE_CATALOGFACADE_H

// TerraAmazon
#include "AbstractFacade.h"
#include "../item/CellLayer.h"

// TerraLib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/dataset/DataSetType.h>
#include <terralib/dataaccess/dataset/DataSet.h>

namespace ta
{
  namespace model
  {
    class Catalog;

    class CatalogFacade : public AbstractFacade
    {
    public:

      static const std::string sm_rulesLayerParams_catalogId;         //!< ...
      static const std::string sm_rulesLayerParams_classColumn;       //!< ...

      static const std::string sm_cellLayer_catalogId;                //!< ...
      static const std::string sm_cellLayer_resolution;               //!< ...
      static const std::string sm_cellLayer_active;                   //!< ...

      static const std::string sm_aoiLayer_catalogId;                 //!< ...
      static const std::string sm_aoiLayer_aoiIdColumn;               //!< ...

    public:

      /*! Default Constructor */
      CatalogFacade(te::da::DataSourcePtr ds);

      /*! \brief Virtual destructor. */
      ~CatalogFacade();

    public:

      bool saveAsCellLayer(ta::model::Catalog* catalog, const double& resolution, const bool& active);

      bool remove(const int& id);

      te::da::DataSourcePtr getDataSource(ta::model::Catalog* catalog);

      std::auto_ptr<te::da::DataSetType> getDataSetType(ta::model::Catalog* catalog);

      std::auto_ptr<te::da::DataSet> getDataSet(ta::model::Catalog* catalog);

      std::auto_ptr<te::da::DataSet> getDataSet(ta::model::Catalog* catalog, std::string oid, std::string propName);

      bool addClassColumn(const int& catalogId, const std::string& className);

      bool removeClassColumn(const int& catalogId, const std::string& className);

      bool getClassColumn(const int& catalogId, std::string& className);

      std::vector<std::string> getClassColumns(const int& catalogId);

      std::vector<std::string> getAoiIdColumns(const int& catalogId);

      ta::model::Catalog* getByUriAndName(const std::string& uri, const std::string& name);

      bool isCellLayer(const int& catalogId);

      int getActiveCellLayer();

      bool setActiveCellLayer(const int& catalogId);

      std::vector<ta::model::CellLayer*> getCellLayers();

      ta::model::CellLayer* getCellLayerData(const int& catalogId);

      void unlockCellLayerItems(const int& catalogId, const std::vector<std::string>& cellOids);

      bool createAmazonLayer(const std::string& layerName, const std::string& userName, int& newLayerId);

      bool isCatalogLockedForEdition(const std::string& layerName);

      std::vector<ta::model::Catalog*> getCatalogs();

    protected:

      bool checkProperties(ta::model::AbstractItem* item);

      bool hasDependecy(const int& id);
    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_FACADE_CATALOGFACADE_H
