/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/CellFacade.h

\brief
*/

#ifndef __TERRAAMAZON_MODEL_FACADE_CELLFACADE_H
#define __TERRAAMAZON_MODEL_FACADE_CELLFACADE_H

// Terralib
#include <terralib/geometry/Geometry.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/qt/af/ApplicationController.h>
#include <terralib/qt/af/events/LayerEvents.h>

// TerraAmazon
#include "../Enums.h"
#include "AbstractFacade.h"

// Qt
#include <QObject>

namespace ta
{
  namespace model
  {
    class Cell;
    class Process;

    class CellFacade : public QObject, public AbstractFacade
    {
      Q_OBJECT //for slots/signals

      public:

        /*! Default Constructor */
        CellFacade(te::da::DataSourcePtr ds);

        /*! \brief Virtual destructor. */
        ~CellFacade();

      public:

        bool loadCells(std::vector< ta::model::Cell*>& vecCell,ta::model::Cell& cell, const bool& getCellInfo);

        ta::model::Cell* get(const std::string& id);

        bool isIntersectIA(ta::model::Cell& cell, int cellSRID, ta::model::Process* process, int aiSRID, te::gm::Geometry* aiGeometry);

        bool setLockState(ta::model::Cell& cell, const bool& lockState);

        bool getCellState(ta::model::Cell& cell, const bool& lockForConcurrency);

        bool getSelectedCells(const unsigned int& processId, std::vector<te::gm::Geometry*>& cs);

        bool queryCellInformation(const te::gm::Envelope& env, std::vector<ta::model::Cell*>& cellSet, const te::map::AbstractLayerPtr& layer);

        bool getLayerCell(std::string& layerCell, std::string& uri);

        bool loadLayer();

        std::string getLayerName();

        std::string getLayerURI();

        te::map::AbstractLayerPtr getLayer();

        bool unlockCells(const unsigned int& processId);

      protected slots:

        void onApplicationTriggered(te::qt::af::evt::Event* e);

      Q_SIGNALS:

        void triggered(te::qt::af::evt::Event* e);

      protected:

        bool checkProperties(ta::model::AbstractItem* item);

        bool hasDependecy(const int& id);

        std::string m_layerName;

        std::string m_layerURI;

        te::map::AbstractLayerPtr m_layer;

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_FACADE_CELLFACADE_H
