/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/ClassFacade.h

\brief
*/

#ifndef __TERRAAMAZON_MODEL_FACADE_CLASSFACADE_H
#define __TERRAAMAZON_MODEL_FACADE_CLASSFACADE_H

// TerraAmazon
#include "../Enums.h"
#include "AbstractFacade.h"

// TerralLib
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>

namespace ta
{
  namespace model
  {
    class Class;

    class ClassFacade : public AbstractFacade
    {
      public:

        /*! Default Constructor */
        ClassFacade(te::da::DataSourcePtr ds);

        /*! \brief Virtual destructor. */
        ~ClassFacade();

      public:

        std::vector<ta::model::Class*> getNotAssociatedClasses();

        std::vector<ta::model::Class*> getNotAssociatedClasses(const int& catalogId);

        bool existsByName(const std::string& className);

        int getByName(const std::string& className, te::da::DataSourceTransactor* transactor);

        bool existsByName(const std::string& className, te::da::DataSourceTransactor* transactor);

        int getByName(const std::string& className);

        std::vector<int> getByRuleSetId(const int& ruleSetId);

        bool hasAssociation(const int& classId);

        bool getClassificationCatalogs(const int& projectId, std::vector<ta::model::Catalog*>& vecCatalogs);

      protected:

        bool checkProperties(ta::model::AbstractItem* item);

        bool hasDependecy(const int& id);
    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_FACADE_CLASSFACADE_H
