/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/PhaseFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../../common/Utils.h"
#include "../factory/AbstractItemFactory.h"
#include "../item/Phase.h"
#include "PhaseFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/IsNull.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/dataaccess/query/Where.h>


ta::model::PhaseFacade::PhaseFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_phase;
}

ta::model::PhaseFacade::~PhaseFacade()
{

}

bool ta::model::PhaseFacade::existsByDescription(const std::string& phaseDescription)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_phase);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string name = dataSet->getString(ta::model::Phase::sm_description);

    std::string nameConvert = te::common::Convert2UCase(name);
    std::string phaseDescriptionConvert = te::common::Convert2UCase(phaseDescription);

    if (Trim(nameConvert) == Trim(phaseDescriptionConvert))
      return true;
  }

  return false;
}

bool ta::model::PhaseFacade::isAssociated(const int& phaseId)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_groupPhase);
  from->push_back(fItem);

  te::da::PropertyName* pName = new te::da::PropertyName("phase_id");
  te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(phaseId));
  te::da::Literal* literal = new te::da::Literal(id);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
    return true;

  return false;
}

std::vector<ta::model::Phase*> ta::model::PhaseFacade::getNotAssociatedPhases()
{
  std::vector<ta::model::Phase*> vec;

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field1 = new te::da::Field(te::da::PropertyName(ta::model::Phase::sm_id));
  te::da::Field* field2 = new te::da::Field(te::da::PropertyName(ta::model::Phase::sm_description));
  fields->push_back(field1);
  fields->push_back(field2);

  te::da::FromItem* fItem1 = new te::da::DataSetName(m_datasetName);
  te::da::FromItem* fItem2 = new te::da::DataSetName(ta::model::Globals::sm_groupPhase);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Phase::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName("phase_id");
  te::da::Expression* expression = new te::da::BinaryFunction("=", exp1, exp2);

  te::da::Join* join = new te::da::Join(fItem1, fItem2, te::da::LEFT_JOIN, new te::da::JoinConditionOn(expression));

  te::da::From* from = new te::da::From();
  from->push_back(join);

  te::dt::AbstractData* function = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>("phase_id " + te::da::FunctionNames::sm_IsNull));
  te::da::Expression* isNull = new te::da::Literal(function);
  te::da::Where* where = new te::da::Where(isNull);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());

    vec.push_back(dynamic_cast<ta::model::Phase*>(ai));
  }

  return vec;
}

bool ta::model::PhaseFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::PhaseFacade::hasDependecy(const int& /*id*/)
{
  return false;
}