/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/ProcessFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../item/Phase.h"
#include "../item/Process.h"
#include "../item/Project.h"
#include "../item/ProjectScene.h"
#include "../item/Scene.h"
#include "../item/Task.h"
#include "../item/TaskLog.h"
#include "../item/User.h"
#include "ProcessFacade.h"
#include "SceneFacade.h"
#include "TaskFacade.h"
#include "TaskLogFacade.h"
#include "UserFacade.h"
#include "../../common/Utils.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/GroupBy.h>
#include <terralib/dataaccess/query/GroupByItem.h>
#include <terralib/dataaccess/query/In.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralInt32.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/Max.h>
#include <terralib/dataaccess/query/NotEqualTo.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/query/OrderBy.h>
#include <terralib/dataaccess/query/OrderByItem.h>
#include <terralib/dataaccess/utils/Utils.h>

ta::model::ProcessFacade::ProcessFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_process;
}

ta::model::ProcessFacade::~ProcessFacade()
{}

bool ta::model::ProcessFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::ProcessFacade::hasDependecy(const int& /*id*/)
{
  return false;
}

std::auto_ptr<te::da::DataSet> ta::model::ProcessFacade::getProcessControl(const int& projId, const int& userId)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* fieldA = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id));
  te::da::Field* fieldA1 = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_sceneId));
  te::da::Field* fieldC = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_viewDate));
  te::da::Field* fieldC1 = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_aiobjectId));
  te::da::Field* fieldD = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_phase + "." + ta::model::Phase::sm_description));

  te::da::Field* fieldE = new te::da::Field(te::da::PropertyName("STATUS.STATUS"));

  fields->push_back(fieldA);
  fields->push_back(fieldA1);
  fields->push_back(fieldC);
  fields->push_back(fieldC1);
  fields->push_back(fieldD);
  fields->push_back(fieldE);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_task);
  te::da::FromItem* fItemSub = new te::da::SubSelect(subSelectLastTask());

  te::da::Expression* exp1 = new te::da::PropertyName("STATUS.TASK_ID");
  te::da::Expression* exp2 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id);
  te::da::Expression* expression = new te::da::BinaryFunction("=", exp1, exp2);
  te::da::JoinConditionOn* on = new te::da::JoinConditionOn(expression);
  te::da::Join* join = new te::da::Join(*fItemA, *fItemSub, te::da::LEFT_JOIN, *on);

  te::da::FromItem* fItemB = new te::da::DataSetName(ta::model::Globals::sm_user);
  te::da::FromItem* fItemC = new te::da::DataSetName(ta::model::Globals::sm_scene);
  te::da::FromItem* fItemD = new te::da::DataSetName(ta::model::Globals::sm_phase);
  te::da::FromItem* fItemE = new te::da::DataSetName(ta::model::Globals::sm_projectScene);

  from->push_back(join);
  from->push_back(fItemB);
  from->push_back(fItemC);
  from->push_back(fItemD);
  from->push_back(fItemE);

  // U.ID = T.USER_ID
  te::da::PropertyName* pNameB = new te::da::PropertyName(ta::model::Globals::sm_user + "." + ta::model::User::sm_id);
  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_userId);

  te::da::EqualTo* equalAB = new te::da::EqualTo(pNameA, pNameB);

  // S.ID = T.SCENE_ID
  te::da::PropertyName* pNameC = new te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_id);
  te::da::PropertyName* pNameA1 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_sceneId);

  te::da::EqualTo* equalCA1 = new te::da::EqualTo(pNameC, pNameA1);

  te::da::And* and1 = new te::da::And(equalAB, equalCA1);

  // F.ID = T.PHASE_ID
  te::da::PropertyName* pNameD = new te::da::PropertyName(ta::model::Globals::sm_phase + "." + ta::model::Phase::sm_id);
  te::da::PropertyName* pNameA2 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_phaseId);

  te::da::EqualTo* equalDA2 = new te::da::EqualTo(pNameD, pNameA2);

  te::da::And* and2 = new te::da::And(and1, equalDA2);

  // PS.SCENE_ID = S.ID
  te::da::PropertyName* pNameE = new te::da::PropertyName(ta::model::Globals::sm_projectScene + "." + ta::model::ProjectScene::sm_sceneId);

  te::da::EqualTo* equalEC = new te::da::EqualTo(pNameE, pNameC);

  te::da::And* and3 = new te::da::And(and2, equalEC);

  // T.PROJECT_ID = PS.PROJECT_ID
  te::da::PropertyName* pNameA3 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_projectId);
  te::da::PropertyName* pNameE1 = new te::da::PropertyName(ta::model::Globals::sm_projectScene + "." + ta::model::ProjectScene::sm_projectId);

  te::da::EqualTo* equalA3E1 = new te::da::EqualTo(pNameA3, pNameE1);

  te::da::And* and4 = new te::da::And(and3, equalA3E1);

  // U.ID = userId
  te::dt::AbstractData* idB = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(userId));
  te::da::Literal* literalB = new te::da::Literal(idB);

  te::da::EqualTo* equalB = new te::da::EqualTo(pNameB, literalB);

  te::da::And* and5 = new te::da::And(and4, equalB);

  // T.PROJECT_ID = projId
  te::dt::AbstractData* idA3 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(projId));
  te::da::Literal* literalA3 = new te::da::Literal(idA3);

  te::da::EqualTo* equalA3 = new te::da::EqualTo(pNameA3, literalA3);

  te::da::And* and6 = new te::da::And(and5, equalA3);

  // BUILDS THE FINAL QUERY

  te::da::Where* where = new te::da::Where(and6);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return std::auto_ptr<te::da::DataSet>();
  }

  return dataSet;
  /*
  SELECT
  T.ID, T.SCENE_ID, S.VIEW_DATE, S.AI_OBJECT_ID, F.DESCRIPTION, STATUS.STATUS
  FROM
  TERRAAMAZON.TA_TASK T
  LEFT JOIN
  (SELECT
  TERRAAMAZON.TA_TASKLOG.STATUS, TERRAAMAZON.TA_TASKLOG.TASK_ID
  FROM
  TERRAAMAZON.TA_TASKLOG,
  (SELECT
  MAX(TERRAAMAZON.TA_TASKLOG.ID) AS LAST_LOG_ID
  FROM
  TERRAAMAZON.TA_TASKLOG
  GROUP BY
  TERRAAMAZON.TA_TASKLOG.TASK_ID) AS SUB1
  WHERE
  (TERRAAMAZON.TA_TASKLOG.ID = SUB1.LAST_LOG_ID)) AS STATUS
  ON
  STATUS.TASK_ID = T.ID,
  TERRAAMAZON.TA_USER U,
  TERRAAMAZON.TA_SCENE S,
  TERRAAMAZON.TA_PHASE F,
  TERRAAMAZON.TA_PROJECT_SCENE PS
  WHERE
  U.ID = T.USER_ID AND
  S.ID = T.SCENE_ID AND
  F.ID = T.PHASE_ID AND
  PS.SCENE_ID = S.ID AND
  T.PROJECT_ID = PS.PROJECT_ID AND
  U.ID = userId AND
  T.PROJECT_ID = projId
  */
}

bool ta::model::ProcessFacade::loadProcess(const int& projId, const int& userId, std::vector<Process*>& vecProcess)
{
  vecProcess.clear();

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_process + "." + ta::model::Process::sm_id));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_process);
  te::da::FromItem* fItemB = new te::da::DataSetName(ta::model::Globals::sm_task);
  from->push_back(fItemA);
  from->push_back(fItemB);

  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_process + "." + ta::model::Process::sm_taskId);
  te::da::PropertyName* pNameB = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id);
  te::da::PropertyName* pNameB1 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_projectId);
  te::da::PropertyName* pNameB2 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_userId);

  te::da::EqualTo* equalAB = new te::da::EqualTo(pNameA, pNameB);

  te::dt::AbstractData* idB1 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(projId));
  te::da::Literal* literalB1 = new te::da::Literal(idB1);

  te::da::EqualTo* equalB1 = new te::da::EqualTo(pNameB1, literalB1);

  te::dt::AbstractData* idB2 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(userId));
  te::da::Literal* literalB2 = new te::da::Literal(idB2);

  te::da::EqualTo* equalB2 = new te::da::EqualTo(pNameB2, literalB2);

  te::da::And* and1 = new te::da::And(equalAB, equalB1);
  te::da::And* and2 = new te::da::And(and1, equalB2);

  te::da::Where* where = new te::da::Where(and2);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);

    dataSet->moveBeforeFirst();

    ta::model::ProcessFacade pf(m_dataSource);

    while (dataSet->moveNext())
    {
      ta::model::Process* p = dynamic_cast<ta::model::Process*>(pf.get(dataSet->getInt32("ID")));

      vecProcess.push_back(p);
    }
  }
  catch (...)
  {
    return false;
  }

  if (dataSet->size() < 1)
  {
    return false;
  }

  if (vecProcess.size() < 1)
  {
    return false;
  }

  return true;
}

ta::model::Process* ta::model::ProcessFacade::getProcessByProject(const int& projId)
{
  std::unique_ptr<ta::model::Process> process;

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_process + "." + ta::model::Process::sm_id));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_process);
  te::da::FromItem* fItemB = new te::da::DataSetName(ta::model::Globals::sm_task);
  from->push_back(fItemA);
  from->push_back(fItemB);

  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_process + "." + ta::model::Process::sm_taskId);
  te::da::PropertyName* pNameB = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id);
  te::da::PropertyName* pNameB1 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_projectId);

  te::da::EqualTo* equalAB = new te::da::EqualTo(pNameA, pNameB);

  te::dt::AbstractData* idB1 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(projId));
  te::da::Literal* literalB1 = new te::da::Literal(idB1);

  te::da::EqualTo* equalB1 = new te::da::EqualTo(pNameB1, literalB1);

  te::da::And* andClause = new te::da::And(equalAB, equalB1);

  te::da::Where* where = new te::da::Where(andClause);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() < 1)
  {
    return 0;
  }

  dataSet->moveFirst();
  return dynamic_cast<ta::model::Process*>(get(dataSet->getInt32("id")));
}

ta::model::Scene* ta::model::ProcessFacade::getScene(ta::model::Process* process)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_id));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemP = new te::da::DataSetName(ta::model::Globals::sm_process);
  te::da::FromItem* fItemT = new te::da::DataSetName(ta::model::Globals::sm_task);
  te::da::FromItem* fItemS = new te::da::DataSetName(ta::model::Globals::sm_scene);
  from->push_back(fItemP);
  from->push_back(fItemT);
  from->push_back(fItemS);

  te::da::PropertyName* pNameP = new te::da::PropertyName(ta::model::Globals::sm_process + "." + ta::model::Process::sm_id);
  te::da::PropertyName* pNameP1 = new te::da::PropertyName(ta::model::Globals::sm_process + "." + ta::model::Process::sm_taskId);
  te::da::PropertyName* pNameT = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id);
  te::da::PropertyName* pNameT1 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_sceneId);
  te::da::PropertyName* pNameS = new te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_id);

  te::da::EqualTo* equalPT = new te::da::EqualTo(pNameP1, pNameT);
  te::da::EqualTo* equalTS = new te::da::EqualTo(pNameT1, pNameS);

  te::dt::AbstractData* idP = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(process->getId()));
  te::da::Literal* literalP = new te::da::Literal(idP);

  te::da::EqualTo* equalP = new te::da::EqualTo(pNameP, literalP);

  te::da::And* and1 = new te::da::And(equalPT, equalTS);
  te::da::And* and2 = new te::da::And(and1, equalP);
  
  te::da::Where* where = new te::da::Where(and2);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() == 0)
  {
    return 0;
  }

  SceneFacade sceneFacade(m_dataSource);

  dataSet->moveFirst();
  return dynamic_cast<ta::model::Scene*>(sceneFacade.get(dataSet->getInt32("id")));
}

ta::model::Task* ta::model::ProcessFacade::getTask(ta::model::Process* process)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_process);
  te::da::FromItem* fItemB = new te::da::DataSetName(ta::model::Globals::sm_task);
  from->push_back(fItemA);
  from->push_back(fItemB);

  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_process + "." + ta::model::Process::sm_taskId);
  te::da::PropertyName* pNameA1 = new te::da::PropertyName(ta::model::Globals::sm_process + "." + ta::model::Process::sm_id);
  te::da::PropertyName* pNameB = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id);

  te::da::EqualTo* equalAB = new te::da::EqualTo(pNameA, pNameB);

  te::dt::AbstractData* idA1 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(process->getId()));
  te::da::Literal* literalA1 = new te::da::Literal(idA1);

  te::da::EqualTo* equalA1 = new te::da::EqualTo(pNameA1, literalA1);

  te::dt::AbstractData* idB = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(process->getTaskId()));
  te::da::Literal* literalB = new te::da::Literal(idB);

  te::da::EqualTo* equalB = new te::da::EqualTo(pNameB, literalB);

  te::da::And* and1 = new te::da::And(equalAB, equalA1);
  te::da::And* and2 = new te::da::And(and1, equalB);

  te::da::Where* where = new te::da::Where(and2);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() != 1)
  {
    return 0;
  }

  TaskFacade taskFacade(m_dataSource);

  dataSet->moveFirst();
  return dynamic_cast<ta::model::Task*>(taskFacade.get(dataSet->getInt32("id")));

}

std::vector<ta::model::User*> ta::model::ProcessFacade::getAssociatedUsersByScene(const int& scnId)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_user + "." + ta::model::User::sm_id));
  te::da::Field* field2 = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_user + "." + ta::model::User::sm_name));
  fields->push_back(field);
  fields->push_back(field2);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_task);
  te::da::FromItem* fItemB = new te::da::DataSetName(ta::model::Globals::sm_user);
  from->push_back(fItemA);
  from->push_back(fItemB);

  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_user + "." + ta::model::User::sm_id);
  te::da::PropertyName* pNameB = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_userId);
  te::da::PropertyName* pNameB1 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_sceneId);

  te::da::EqualTo* equalAB = new te::da::EqualTo(pNameA, pNameB);

  te::dt::AbstractData* idB1 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(scnId));
  te::da::Literal* literalB1 = new te::da::Literal(idB1);

  te::da::EqualTo* equalB1 = new te::da::EqualTo(pNameB1, literalB1);

  te::da::And* andClause = new te::da::And(equalAB, equalB1);

  te::da::Where* where = new te::da::Where(andClause);

  te::da::OrderByItem* obItem = new te::da::OrderByItem(new te::da::PropertyName(ta::model::Globals::sm_user + "." + ta::model::User::sm_name));
  te::da::OrderBy* ob = new te::da::OrderBy();
  ob->push_back(obItem);

  te::da::Select select = new te::da::Select(fields, from, where, ob);

  std::auto_ptr<te::da::DataSet> dataSet;

  std::vector<ta::model::User*> users;

  try
  {

    dataSet = m_dataSource->query(select);

    dataSet->moveBeforeFirst();

    ta::model::UserFacade uf(m_dataSource);

    while (dataSet->moveNext())
    {
      ta::model::User* u = dynamic_cast<ta::model::User*>(uf.get(dataSet->getInt32("ID")));

      users.push_back(u);
    }

  }
  catch (...)
  {
    return std::vector<ta::model::User*>();
  }

  return users;
}

te::da::SubSelect ta::model::ProcessFacade::subSelectLastTask()
{
  te::da::Fields* fieldsSub1 = new te::da::Fields();
  te::da::Expression* e_max = new te::da::Max(te::da::PropertyName(ta::model::Globals::sm_tasklog + "." + ta::model::TaskLog::sm_id));
  te::da::Field* fieldSub1 = new te::da::Field(*e_max, "LAST_LOG_ID");

  fieldsSub1->push_back(fieldSub1);

  te::da::From* fromSub1 = new te::da::From();
  te::da::FromItem* fItemASub1 = new te::da::DataSetName(ta::model::Globals::sm_tasklog);
  fromSub1->push_back(fItemASub1);

  te::da::GroupBy* groupBySub1 = new te::da::GroupBy;
  te::da::GroupByItem* giSub1 = new te::da::GroupByItem(new te::da::PropertyName(ta::model::Globals::sm_tasklog + "." + ta::model::TaskLog::sm_taskId));
  groupBySub1->push_back(giSub1);

  te::da::Select selectSub1 = new te::da::Select(fieldsSub1, fromSub1, groupBySub1);
  te::da::SubSelect subSelectSub1(selectSub1, "SUB1");
  //
  te::da::Fields* fieldsSub = new te::da::Fields();
  te::da::Field* fieldASub = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_tasklog + "." + ta::model::TaskLog::sm_status));
  te::da::Field* fieldA1Sub = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_tasklog + "." + ta::model::TaskLog::sm_taskId));
  fieldsSub->push_back(fieldASub);
  fieldsSub->push_back(fieldA1Sub);

  te::da::From* fromSub = new te::da::From();
  te::da::FromItem* fItemASub = new te::da::DataSetName(ta::model::Globals::sm_tasklog);
  te::da::FromItem* fItemBSub1 = new te::da::SubSelect(subSelectSub1);
  fromSub->push_back(fItemASub);
  fromSub->push_back(fItemBSub1);

  te::da::PropertyName* pNameASub = new te::da::PropertyName(ta::model::Globals::sm_tasklog + "." + ta::model::TaskLog::sm_id);
  te::da::PropertyName* pNameBSub1 = new te::da::PropertyName("SUB1.LAST_LOG_ID");

  te::da::EqualTo* equalASub = new te::da::EqualTo(pNameASub, pNameBSub1);

  te::da::Where* whereSub = new te::da::Where(equalASub);

  te::da::Select selectSub = new te::da::Select(fieldsSub, fromSub, whereSub);
  te::da::SubSelect subSelectSub(selectSub, "STATUS");

  return subSelectSub;

  /*
  (SELECT
  TERRAAMAZON.TA_TASKLOG.STATUS, TERRAAMAZON.TA_TASKLOG.TASK_ID
  FROM
  TERRAAMAZON.TA_TASKLOG,
  (SELECT MAX(TERRAAMAZON.TA_TASKLOG.ID) AS LAST_LOG_ID FROM TERRAAMAZON.TA_TASKLOG GROUP BY TERRAAMAZON.TA_TASKLOG.TASK_ID) AS SUB1
  WHERE
  (TERRAAMAZON.TA_TASKLOG.ID = SUB1.LAST_LOG_ID)) AS STATUS
  */
}

std::vector<ta::model::CellLock*> ta::model::ProcessFacade::getLockedCells(const std::string& cellLayerName, const std::string& cellOid, const int& userId)
{
  /*SELECT u.name, cust_1.cell_oid, tl.initial_time, pj.name FROM(((((public.[CELL_LAYER] AS custom_cell
    INNER JOIN terraamazon.ta_process AS pc ON((custom_cell.process_id = pc.id)))
    INNER JOIN terraamazon.ta_task AS tk ON((pc.task_id = tk.id)))
    INNER JOIN terraamazon.ta_tasklog AS tl ON((pc.tasklog_id = tl.id)))
    INNER JOIN terraamazon.ta_user AS u ON((tk.user_id = u.id)))
    INNER JOIN terraamazon.ta_project AS pj ON((tk.project_id = pj.id)))
    WHERE not custom_cell.process_id isnull;*/

  std::string cellOidSearch = "";
  int userIdSearch = -1;

  if (!cellOid.empty())
  {
    std::string cellOidUp = te::common::Convert2UCase(cellOid);
    cellOidSearch = Trim(cellOidUp);
  }
  if (userId > 0)
  {
    userIdSearch = userId;
  }

  std::string cellLayerAlias = "custom_cell";
  std::string cellLayerFieldCellOid = "cell_oid";
  std::string cellLayerFieldProcessId = "process_id";
  std::string projectFieldAliasName = "project_field_alias_" + ta::model::Project::sm_name;

  te::da::Field* field1 = new te::da::Field(ta::model::Globals::sm_userAlias + "." + ta::model::User::sm_name);
  te::da::Field* field2 = new te::da::Field(cellLayerAlias + "." + cellLayerFieldCellOid);
  te::da::Field* field3 = new te::da::Field(ta::model::Globals::sm_taskLogAlias + "." + ta::model::TaskLog::sm_initialTime);
  te::da::Field* field4 = new te::da::Field(ta::model::Globals::sm_projectAlias + "." + ta::model::Project::sm_name, projectFieldAliasName);

  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field1);
  fields->push_back(field2);
  fields->push_back(field3);
  fields->push_back(field4);

  te::da::FromItem* fromItem1 = new te::da::DataSetName(cellLayerName, cellLayerAlias);
  te::da::FromItem* fromItem2 = new te::da::DataSetName(ta::model::Globals::sm_process, ta::model::Globals::sm_processAlias);
  te::da::FromItem* fromItem3 = new te::da::DataSetName(ta::model::Globals::sm_task, ta::model::Globals::sm_taskAlias);
  te::da::FromItem* fromItem4 = new te::da::DataSetName(ta::model::Globals::sm_tasklog, ta::model::Globals::sm_taskLogAlias);
  te::da::FromItem* fromItem5 = new te::da::DataSetName(ta::model::Globals::sm_user, ta::model::Globals::sm_userAlias);
  te::da::FromItem* fromItem6 = new te::da::DataSetName(ta::model::Globals::sm_project, ta::model::Globals::sm_projectAlias);

  te::da::Expression* exp1 = new te::da::PropertyName(cellLayerAlias + "." + cellLayerFieldProcessId);
  te::da::Expression* exp2 = new te::da::PropertyName(ta::model::Globals::sm_processAlias + "." + ta::model::Process::sm_id);
  te::da::Expression* exp3 = new te::da::PropertyName(ta::model::Globals::sm_processAlias + "." + ta::model::Process::sm_taskId);
  te::da::Expression* exp4 = new te::da::PropertyName(ta::model::Globals::sm_taskAlias + "." + ta::model::Task::sm_id);
  te::da::Expression* exp5 = new te::da::PropertyName(ta::model::Globals::sm_processAlias + "." + ta::model::Process::sm_tasklogId);
  te::da::Expression* exp6 = new te::da::PropertyName(ta::model::Globals::sm_taskLogAlias + "." + ta::model::TaskLog::sm_id);
  te::da::Expression* exp7 = new te::da::PropertyName(ta::model::Globals::sm_taskAlias + "." + ta::model::Task::sm_userId);
  te::da::Expression* exp8 = new te::da::PropertyName(ta::model::Globals::sm_userAlias + "." + ta::model::User::sm_id);
  te::da::Expression* exp9 = new te::da::PropertyName(ta::model::Globals::sm_taskAlias + "." + ta::model::Task::sm_projectId);
  te::da::Expression* exp10 = new te::da::PropertyName(ta::model::Globals::sm_projectAlias + "." + ta::model::Project::sm_id);

  te::da::Expression* expression1 = new te::da::BinaryFunction("=", exp1, exp2);
  te::da::Expression* expression2 = new te::da::BinaryFunction("=", exp3, exp4);
  te::da::Expression* expression3 = new te::da::BinaryFunction("=", exp5, exp6);
  te::da::Expression* expression4 = new te::da::BinaryFunction("=", exp7, exp8);
  te::da::Expression* expression5 = new te::da::BinaryFunction("=", exp9, exp10);

  te::da::Join* join1 = new te::da::Join(fromItem1, fromItem2, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression1));
  te::da::Join* join2 = new te::da::Join(join1, fromItem3, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression2));
  te::da::Join* join3 = new te::da::Join(join2, fromItem4, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression3));
  te::da::Join* join4 = new te::da::Join(join3, fromItem5, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression4));
  te::da::Join* join5 = new te::da::Join(join4, fromItem6, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression5));

  te::da::From* from = new te::da::From();
  from->push_back(join5);

  te::da::Expression* condition;

  // condition 1: process id is not null
  te::dt::AbstractData* function = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>
    (te::da::FunctionNames::sm_Not + " " + cellLayerAlias + "." + cellLayerFieldProcessId + " " + te::da::FunctionNames::sm_IsNull));
  te::da::Expression* processIdIsNotNull = new te::da::Literal(function);
  condition = processIdIsNotNull;

  // condition 2: filter cell oid
  te::da::And* cellOidFilter = NULL;
  if (!cellOidSearch.empty())
  {
    te::da::PropertyName* pCellOid = new te::da::PropertyName(cellLayerAlias + "." + cellLayerFieldCellOid);
    te::da::LiteralString* lCellOid = new te::da::LiteralString(cellOidSearch);
    te::da::EqualTo* cellOidExp = new te::da::EqualTo(pCellOid, lCellOid);
    cellOidFilter = new te::da::And(processIdIsNotNull, cellOidExp);
    condition = cellOidFilter;
  }

  // condition 3: filter user id
  te::da::And* userIdFilter;
  if (userIdSearch > 0)
  {
    te::da::PropertyName* pUserId = new te::da::PropertyName(ta::model::Globals::sm_userAlias + "." + ta::model::User::sm_id);
    te::da::Literal* lUserId = new te::da::LiteralInt32(userIdSearch);
    te::da::EqualTo* userIdExp = new te::da::EqualTo(pUserId, lUserId);
    if (cellOidFilter != NULL)
    {
      userIdFilter = new te::da::And(cellOidFilter, userIdExp);
    }
    else
    {
      userIdFilter = new te::da::And(processIdIsNotNull, userIdExp);
    }
    condition = userIdFilter;
  }

  te::da::Where* where = new te::da::Where(condition);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  std::vector<ta::model::CellLock*> result;

  while (dataSet->moveNext())
  {
    std::string userName = dataSet->getString(ta::model::User::sm_name);
    std::string cellOid = dataSet->getString(cellLayerFieldCellOid);
    std::auto_ptr<te::dt::DateTime> dt(dataSet->getDateTime(ta::model::TaskLog::sm_initialTime));
    te::dt::TimeInstant* dateTime = dynamic_cast<te::dt::TimeInstant*>(dt.get()->clone());
    std::string projectName = dataSet->getString(projectFieldAliasName);
    ta::model::CellLock* cellLock = new CellLock();
    cellLock->setUserName(userName);
    cellLock->setCellOid(cellOid);
    cellLock->setDateTime(dateTime);
    cellLock->setProjetName(projectName);

    result.push_back(cellLock);
  }

  return result;
}
