/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/ProcessFacade.h

\brief
*/

#ifndef __TERRAAMAZON_MODEL_FACADE_PROCESSFACADE_H
#define __TERRAAMAZON_MODEL_FACADE_PROCESSFACADE_H

// TerraLib
#include <terralib/dataaccess/query/SubSelect.h>

// TerraAmazon
#include "AbstractFacade.h"
#include "../item/CellLock.h"

namespace ta
{
  namespace model
  {
    class Process;
    class Scene;
    class Task;
    class TaskLog;
    class User;

    class ProcessFacade: public AbstractFacade
    {
      public:

        /*! Default Constructor */
        ProcessFacade(te::da::DataSourcePtr ds);

        /*! \brief Virtual destructor. */
        ~ProcessFacade();

        std::auto_ptr<te::da::DataSet> getProcessControl(const int& projId, const int& userId);

        bool loadProcess(const int& projId, const int& userId, std::vector<Process*>& vecProcess);//const int& taskId);

        ta::model::Process* getProcessByProject(const int& projId);

        ta::model::Scene* getScene(ta::model::Process* process);

        ta::model::Task* getTask(ta::model::Process* process);

        std::vector<ta::model::User*> getAssociatedUsersByScene(const int& scnId);

        std::vector<ta::model::CellLock*> getLockedCells(const std::string& cellLayerName, const std::string& cellOid, const int& userId);

      protected:

        bool checkProperties(ta::model::AbstractItem* item);

        bool hasDependecy(const int& id);

      private:

        te::da::SubSelect subSelectLastTask();

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_FACADE_PROCESSFACADE_H
