/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/ProjectFacade.h

\brief
*/

#ifndef __TERRAAMAZON_MODEL_FACADE_PROJECTFACADE_H
#define __TERRAAMAZON_MODEL_FACADE_PROJECTFACADE_H

// TerraAmazon
#include "../Enums.h"
#include "AbstractFacade.h"
#include "../item_mem/ProjectMem.h"

namespace ta
{
  namespace model
  {
    class Project;

    class ProjectFacade : public AbstractFacade
    {
    public:

      static const std::string sm_ruleSetProject_projectId;  //!< ...
      static const std::string sm_ruleSetProject_ruleSetId;  //!< ...

      static const std::string sm_projectScene_projectId;    //!< ...
      static const std::string sm_projectScene_sceneId;      //!< ...

    public:

      /*! Default Constructor */
      ProjectFacade(te::da::DataSourcePtr ds);

      /*! \brief Virtual destructor. */
      ~ProjectFacade();

    public:

      bool hasScaleDefined(Project* proj);

      void setNoScale(Project* proj);

      bool existsByName(const std::string& projName);


      bool addAssociateRuleSet(const int &projectId, const int &ruleSetId);

      bool addAssociateRuleSet(const int &projectId, const int &ruleSetId, te::da::DataSourceTransactor* transactor);

      bool removeAssociateRuleSet(const int &projectId, const int &ruleSetId);

      std::vector<int> getRuleSets(const int& projectId);


      std::vector<int> getRules(const int& projectId);

      std::vector<int> getRules(const int& projectId, const ta::model::SpatialOperation& soType);


      bool addAssocietedScene(const int &projectId, const int &sceneId);


      std::vector<int> getOutputCatalogs(const int& projectId);

      bool hasCatalog(const int& projectId, const int& catalogId);

      bool hasCatalogRule(const int& projectId, const int& catalogId, const ta::model::SpatialOperation& soType);


      bool importToDatabase(ta::model::ProjectMem* projectMem, std::string& errorMessage, std::string& warningMessage);

    protected:

      bool checkProperties(ta::model::AbstractItem* item);

      bool hasDependecy(const int& id);
    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_FACADE_PROJECTFACADE_H
