/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/RuleRuleSetFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../item/Rule.h"
#include "../item/RuleRuleSet.h"
#include "../factory/AbstractItemFactory.h"
#include "RuleRuleSetFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralInt32.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>


ta::model::RuleRuleSetFacade::RuleRuleSetFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_ruleRuleset;
}

ta::model::RuleRuleSetFacade::~RuleRuleSetFacade()
{
}

bool ta::model::RuleRuleSetFacade::save(ta::model::AbstractItem* item)
{
  if (!checkProperties(item))
  {
    return false;
  }

  ta::model::ItemPropertyVector props = item->toProperties();

  ta::model::ItemPropertyVector::iterator it;

  std::auto_ptr<te::da::DataSetType> newDst(new te::da::DataSetType(m_datasetName));

  for (it = props.begin(); it != props.end(); ++it)
    newDst->add((*it)->getProperty()->clone());

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  for (it = props.begin(); it != props.end(); ++it)
    dsItem->setValue((*it)->getProperty()->getName(), (*it)->getData()->clone());

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::auto_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(m_datasetName, newData, op);

  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

bool ta::model::RuleRuleSetFacade::save(ta::model::AbstractItem* item, te::da::DataSourceTransactor* transactor)
{
  if (!checkProperties(item))
  {
    return false;
  }

  ta::model::ItemPropertyVector props = item->toProperties();

  ta::model::ItemPropertyVector::iterator it;

  std::auto_ptr<te::da::DataSetType> newDst(new te::da::DataSetType(m_datasetName));

  for (it = props.begin(); it != props.end(); ++it)
    newDst->add((*it)->getProperty()->clone());

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  for (it = props.begin(); it != props.end(); ++it)
    dsItem->setValue((*it)->getProperty()->getName(), (*it)->getData()->clone());

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  try
  {
    transactor->add(m_datasetName, newData, op);

  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";
    return false;
  }

  return true;
}

bool ta::model::RuleRuleSetFacade::remove(const int& ruleId, const int& ruleSetId)
{
  //TODO: refactoring...

  std::string sql = "DELETE FROM ";
  sql += m_datasetName;
  sql += " WHERE ";
  sql += ta::model::RuleRuleSet::sm_ruleset_id;
  sql += " = ";
  sql += te::common::Convert2String(ruleSetId);
  sql += " AND ";
  sql += ta::model::RuleRuleSet::sm_rule_id;
  sql += " = ";
  sql += te::common::Convert2String(ruleId);

  //remove element
  try
  {
    m_dataSource->execute(sql);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    return false;
  }

  return true;
}

bool ta::model::RuleRuleSetFacade::hasRuleDependecy(const int& ruleId)
{
  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_ruleRuleset);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::RuleRuleSet::sm_rule_id);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ruleId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  return !dataSet->isEmpty();
}

std::vector<int> ta::model::RuleRuleSetFacade::getRules(const int& ruleSetId)
{
  std::vector<int> rules;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_ruleRuleset);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::RuleRuleSet::sm_ruleset_id);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ruleSetId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    rules.push_back(dataSet->getInt32(ta::model::RuleRuleSet::sm_rule_id));
  }

  return rules;
}

std::vector<ta::model::RuleRuleSet*> ta::model::RuleRuleSetFacade::getByRuleSetId(const int& ruleSetId)
{
  std::vector<ta::model::RuleRuleSet*> result;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_ruleRuleset);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::RuleRuleSet::sm_ruleset_id);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ruleSetId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::RuleRuleSet* rrs = dynamic_cast<ta::model::RuleRuleSet*>(ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get()));
    result.push_back(rrs);
  }

  return result;
}

bool ta::model::RuleRuleSetFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::RuleRuleSetFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
