/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/RuleSetFacade.h

\brief
*/

#ifndef __TERRAAMAZON_MODEL_FACADE_RULESETFACADE_H
#define __TERRAAMAZON_MODEL_FACADE_RULESETFACADE_H

// TerraAmazon
#include "../Enums.h"
#include "AbstractFacade.h"
#include "../item/Catalog.h"

namespace ta
{
  namespace model
  {
    class RuleSet;

    class RuleSetFacade: public AbstractFacade
    {
      public:

        static const std::string sm_ruleSetClass_ruleSetId;        //!< ...
        static const std::string sm_ruleSetClass_classId;         //!< ...

      public:

        /*! Default Constructor */
        RuleSetFacade(te::da::DataSourcePtr ds);

        /*! \brief Virtual destructor. */
        ~RuleSetFacade();

      public:

        std::string createRuleSetName(const int& catalogId);

        bool existsByName(const std::string& ruleSetName);

        bool existsByCatalogId(const int& catalogId);

        ta::model::RuleSet* getByCatalogId(const int& catalogId);

        bool createOutputRule(const int& ruleSetId, const int& catalogId, const ta::model::SpatialOperation& so);

        bool createAssociation(const int& ruleId, const int& ruleSetId, const int& priority = 0);

        bool isOutputWithIntersection(const int& ruleSetId, ta::model::RuleSet* &output);

        bool removeAssociation(const int& ruleId, const int& ruleSetId);

        std::vector<int> getRulesRuleSet(const int& ruleSetId);

        bool getRuleSetId(const int& catalogId, int& ruleSetId);

        bool hasCatalog(const int& catalogId);

        std::vector<int> getRuleSetClasses(const int& catalogId);

        bool addRuleSetClass(const int& catalogId, const int& classId);
        
        bool addRuleSetClass(const int& ruleSetId, const int& classId, te::da::DataSourceTransactor* transactor);

        bool removeRuleSetClass(const int& catalogId, const int& classId);

      protected:

        bool checkProperties(ta::model::AbstractItem* item);

        bool hasDependecy(const int& id);
    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_FACADE_RULESETFACADE_H
