/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/RulesLayerColumnFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../Enums.h"
#include "../item/Rule.h"
#include "../factory/AbstractItemFactory.h"
#include "../item/RulesLayerColumn.h"
#include "RulesLayerColumnFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralInt32.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>


ta::model::RulesLayerColumnFacade::RulesLayerColumnFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_rulesLayerColumn;
}

ta::model::RulesLayerColumnFacade::~RulesLayerColumnFacade()
{

}

ta::model::RulesLayerColumn* ta::model::RulesLayerColumnFacade::getByRuleSetIdAndColumnOut(const int& ruleSetId, const std::string& columnOut)
{
  ta::model::RulesLayerColumn* result = new ta::model::RulesLayerColumn();

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_rulesLayerColumnAlias + "." + "*"));
  fields->push_back(field);

  te::da::FromItem* fItem1 = new te::da::DataSetName(ta::model::Globals::sm_rulesLayerColumn, ta::model::Globals::sm_rulesLayerColumnAlias);
  te::da::FromItem* fItem2 = new te::da::DataSetName(ta::model::Globals::sm_rule, ta::model::Globals::sm_ruleAlias);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Globals::sm_ruleAlias + "." + ta::model::Rule::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName(ta::model::Globals::sm_rulesLayerColumnAlias + "." + ta::model::RulesLayerColumn::sm_rule_id);
  te::da::Expression* expression = new te::da::BinaryFunction("=", exp1, exp2);

  te::da::Join* join = new te::da::Join(fItem1, fItem2, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression));

  te::da::From* from = new te::da::From();
  from->push_back(join);

  te::da::PropertyName* prop1 = new te::da::PropertyName(ta::model::Globals::sm_rulesLayerColumnAlias + "." + ta::model::RulesLayerColumn::sm_ruleset_id);
  te::da::Literal* lit1 = new te::da::LiteralInt32(ruleSetId);
  te::da::EqualTo* equal1 = new te::da::EqualTo(prop1, lit1);

  te::da::PropertyName* prop2 = new te::da::PropertyName(ta::model::Globals::sm_rulesLayerColumnAlias + "." + ta::model::RulesLayerColumn::sm_column_out);
  te::da::LiteralString* lit2 = new te::da::LiteralString(columnOut);
  te::da::EqualTo* equal2 = new te::da::EqualTo(prop2, lit2);

  te::da::PropertyName* prop3 = new te::da::PropertyName(ta::model::Globals::sm_ruleAlias + "." + ta::model::Rule::sm_operation_id);
  te::dt::AbstractData* dat = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ta::model::SO_INTERSECTION));
  te::da::Literal* lit3 = new te::da::Literal(dat);
  te::da::EqualTo* equal3 = new te::da::EqualTo(prop3, lit3);

  te::da::And* and1 = new te::da::And(equal1, equal2);

  te::da::And* and2 = new te::da::And(and1, equal3);

  te::da::Where* where = new te::da::Where(and2);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());
    result = dynamic_cast<ta::model::RulesLayerColumn*>(ai);
  }

  return result;
}

std::vector<ta::model::RulesLayerColumn*> ta::model::RulesLayerColumnFacade::getByRuleSetIdAndRuleId(const int& ruleSetId, const int& ruleId)
{
  std::vector<ta::model::RulesLayerColumn*> result;

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_rulesLayerColumn);

  te::da::From* from = new te::da::From();
  from->push_back(fItem);

  te::da::PropertyName* prop1 = new te::da::PropertyName(ta::model::RulesLayerColumn::sm_ruleset_id);
  te::da::Literal* lit1 = new te::da::LiteralInt32(ruleSetId);
  te::da::EqualTo* equal1 = new te::da::EqualTo(prop1, lit1);

  te::da::PropertyName* prop2 = new te::da::PropertyName(ta::model::RulesLayerColumn::sm_rule_id);
  te::da::Literal* lit2 = new te::da::LiteralInt32(ruleId);
  te::da::EqualTo* equal2 = new te::da::EqualTo(prop2, lit2);

  te::da::And* and1 = new te::da::And(equal1, equal2);

  te::da::Where* where = new te::da::Where(and1);

  te::da::Select select1 = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select1);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());
    result.push_back(dynamic_cast<ta::model::RulesLayerColumn*>(ai));
  }

  return result;
}

bool ta::model::RulesLayerColumnFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::RulesLayerColumnFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
