/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/SceneFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../item/Catalog.h"
#include "../item/Process.h"
#include "../item/ProjectScene.h"
#include "../item/Scene.h"
#include "../item/Task.h"
#include "AoiLayerFacade.h"
#include "CatalogFacade.h"
#include "ProcessFacade.h"
#include "SceneFacade.h"
#include "TaskFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/geometry/Geometry.h>
#include <terralib/geometry/GeometryProperty.h>

ta::model::SceneFacade::SceneFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_scene;
}

ta::model::SceneFacade::~SceneFacade()
{}

bool ta::model::SceneFacade::isSceneLocked(const int& sceneId, const int& projectId, bool& lockState)
{
  lockState = false;

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_projectScene);
  from->push_back(fItemA);

  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_projectScene + "." + ta::model::ProjectScene::sm_sceneId);
  te::da::PropertyName* pNameA1 = new te::da::PropertyName(ta::model::Globals::sm_projectScene + "." + ta::model::ProjectScene::sm_projectId);

  te::dt::AbstractData* idA = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(sceneId));
  te::da::Literal* literalA = new te::da::Literal(idA);

  te::da::EqualTo* equalA = new te::da::EqualTo(pNameA, literalA);

  te::dt::AbstractData* idA1 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(projectId));
  te::da::Literal* literalA1 = new te::da::Literal(idA1);

  te::da::EqualTo* equalA1 = new te::da::EqualTo(pNameA1, literalA1);

  te::da::And* andClause = new te::da::And(equalA, equalA1);

  te::da::Where* where = new te::da::Where(andClause);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return false;
  }

  dataSet->moveFirst();
  lockState = dataSet->getBool(ta::model::ProjectScene::sm_locked);

  return true;
}

const std::string ta::model::SceneFacade::getAoiIdColumn(const int& sceneId)
{
  //Acquiring the  the scene
  std::unique_ptr<ta::model::Scene> scene(dynamic_cast<ta::model::Scene*>(get(sceneId)));
  if (scene.get())
  {
    //Acquiring the aoi column, if it has been defined laready

    te::da::Fields* fields = new te::da::Fields();
    te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::AoiLayer::sm_aoiIdColumn));
    fields->push_back(field);

    te::da::From* from = new te::da::From();
    te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_aoiLayer);
    from->push_back(fItem);

    te::da::PropertyName* pName = new te::da::PropertyName(ta::model::AoiLayer::sm_catalogId);
    te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(scene->getAoiLayerId())); //TODO change
    te::da::Literal* literal = new te::da::Literal(id);

    te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);

    te::da::Where* where = new te::da::Where(equal);

    te::da::Select select = new te::da::Select(fields, from, where);

    std::unique_ptr<te::da::DataSet> dataSet;

    try
    {
      dataSet = m_dataSource->query(select);
    }
    catch (...)
    {
      return "";
    }

    if(dataSet->moveFirst())
      return dataSet->getString(ta::model::AoiLayer::sm_aoiIdColumn);
    else
      return "";
  }
  else
    return "";
}

std::auto_ptr<te::gm::Geometry> ta::model::SceneFacade::getAiGeometries(const int& processId)
{
  std::auto_ptr<te::gm::Geometry> geom;

  ta::model::ProcessFacade pf(m_dataSource);
  std::auto_ptr<ta::model::Process> process(dynamic_cast<ta::model::Process*>(pf.get(processId)));

  ta::model::TaskFacade tf(m_dataSource);
  std::auto_ptr<ta::model::Task> task(pf.getTask(process.get()));

  ta::model::SceneFacade scnf(m_dataSource);
  std::unique_ptr<ta::model::Scene> scene(dynamic_cast<ta::model::Scene*>(scnf.get(task->getSceneId())));

  ta::model::AoiLayerFacade aoif(m_dataSource);
  std::unique_ptr<ta::model::AoiLayer> aoilayer(dynamic_cast<ta::model::AoiLayer*>(aoif.get(scene->getAoiLayerId())));

  ta::model::CatalogFacade ctf(m_dataSource);
  std::unique_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(ctf.get(aoilayer->getCatalogId())));

  std::unique_ptr<te::da::DataSet> ds = ctf.getDataSet(catalog.get(), scene->getAiObjectId(), aoilayer->getAoiIdColumn());

  std::size_t gpos = te::da::GetFirstSpatialPropertyPos(ds.get());

  ds.get()->moveBeforeFirst();

  if (ds.get()->moveNext())
    geom = ds.get()->getGeometry(gpos);

  return std::move(geom);
}

std::auto_ptr<ta::model::AoiLayer> ta::model::SceneFacade::getAoiLayer(int aoiLayerId)
{
  ta::model::AoiLayerFacade aoiLayerFacade(m_dataSource);
  std::auto_ptr<ta::model::AoiLayer> aoiLayer(dynamic_cast<ta::model::AoiLayer*>(aoiLayerFacade.get(aoiLayerId)));

  return aoiLayer;
}

bool ta::model::SceneFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::SceneFacade::hasDependecy(const int& /*id*/)
{
  return false;
}