/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/TaskFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../item/Catalog.h"
#include "../item/Phase.h"
#include "../item/ProjectScene.h"
#include "../item/Scene.h"
#include "../item/Task.h"
#include "../item/User.h"
#include "SceneFacade.h"
#include "TaskFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/query/OrderBy.h>
#include <terralib/dataaccess/query/OrderByItem.h>

ta::model::TaskFacade::TaskFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_task;
}

ta::model::TaskFacade::~TaskFacade()
{}

bool ta::model::TaskFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::TaskFacade::hasDependecy(const int& /*id*/)
{
  return false;
}

std::auto_ptr<te::da::DataSet> ta::model::TaskFacade::getTaskControl(const int& projId)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* fieldA = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_id));
  te::da::Field* fieldA1 = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_aiobjectId));
  te::da::Field* fieldA2 = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_viewDate));
  te::da::Field* fieldA3 = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_julDay));
  te::da::Field* fieldA4 = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_inclusion));
  te::da::Field* fieldB = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_projectScene + "." + ta::model::ProjectScene::sm_locked));
  te::da::Field* fieldA5 = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_aoilayerid));
  te::da::Field* fieldC = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_catalog + "." + ta::model::Catalog::sm_name));
  fields->push_back(fieldA);
  fields->push_back(fieldA1);
  fields->push_back(fieldA2);
  fields->push_back(fieldA3);
  fields->push_back(fieldA4);
  fields->push_back(fieldB);
  fields->push_back(fieldA5);
  fields->push_back(fieldC);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_scene);
  te::da::FromItem* fItemB = new te::da::DataSetName(ta::model::Globals::sm_projectScene);
  te::da::FromItem* fItemC = new te::da::DataSetName(ta::model::Globals::sm_catalog);
  from->push_back(fItemA);
  from->push_back(fItemB);
  from->push_back(fItemC);

  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_id);
  te::da::PropertyName* pNameB = new te::da::PropertyName(ta::model::Globals::sm_projectScene + "." + ta::model::ProjectScene::sm_sceneId);
  te::da::PropertyName* pNameC = new te::da::PropertyName(ta::model::Globals::sm_catalog + "." + ta::model::Catalog::sm_id);
  te::da::PropertyName* pNameA1 = new te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_aoilayerid);
  te::da::PropertyName* pNameB1 = new te::da::PropertyName(ta::model::Globals::sm_projectScene + "." + ta::model::ProjectScene::sm_projectId);

  te::da::EqualTo* equalAB = new te::da::EqualTo(pNameA, pNameB);
  te::da::EqualTo* equalCA1 = new te::da::EqualTo(pNameC, pNameA1);

  te::dt::AbstractData* idB1 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(projId));
  te::da::Literal* literalB1 = new te::da::Literal(idB1);

  te::da::EqualTo* equalB1 = new te::da::EqualTo(pNameB1, literalB1);

  te::da::And* and1 = new te::da::And(equalAB, equalCA1);
  te::da::And* and2 = new te::da::And(and1, equalB1);

  te::da::Where* where = new te::da::Where(and2);

  te::da::OrderByItem* obItem = new te::da::OrderByItem(new te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_id));
  te::da::OrderBy* ob = new te::da::OrderBy();
  ob->push_back(obItem);

  te::da::Select select = new te::da::Select(fields, from, where, ob);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return std::auto_ptr<te::da::DataSet>();
  }

  return dataSet;
}

bool ta::model::TaskFacade::loadTask(const int& projId, const int& phaseId, const int& userId, const int& sceneId)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_task);
  te::da::FromItem* fItemB = new te::da::DataSetName(ta::model::Globals::sm_phase);
  te::da::FromItem* fItemC = new te::da::DataSetName(ta::model::Globals::sm_user);
  from->push_back(fItemA);
  from->push_back(fItemB);
  from->push_back(fItemC);

  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_phaseId);
  te::da::PropertyName* pNameB = new te::da::PropertyName(ta::model::Globals::sm_phase + "." + ta::model::Phase::sm_id);
  te::da::PropertyName* pNameA1 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_userId);
  te::da::PropertyName* pNameC = new te::da::PropertyName(ta::model::Globals::sm_user + "." + ta::model::User::sm_id);
  te::da::PropertyName* pNameA2 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_projectId);
  te::da::PropertyName* pNameA3 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::ProjectScene::sm_sceneId);

  te::da::EqualTo* equalAB = new te::da::EqualTo(pNameA, pNameB);
  te::da::EqualTo* equalBC = new te::da::EqualTo(pNameA1, pNameC);

  te::da::And* and1 = new te::da::And(equalAB, equalBC);

  te::dt::AbstractData* idA2 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(projId));
  te::da::Literal* literalA2 = new te::da::Literal(idA2);
  te::da::EqualTo* equalA2 = new te::da::EqualTo(pNameA2, literalA2);

  te::da::And* and2 = new te::da::And(and1, equalA2);

  te::dt::AbstractData* idB = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(phaseId));
  te::da::Literal* literalB = new te::da::Literal(idB);
  te::da::EqualTo* equalB = new te::da::EqualTo(pNameB, literalB);

  te::da::And* and3 = new te::da::And(and2, equalB);

  te::dt::AbstractData* idC = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(userId));
  te::da::Literal* literalC = new te::da::Literal(idC);
  te::da::EqualTo* equalC = new te::da::EqualTo(pNameC, literalC);

  te::da::And* and4 = new te::da::And(and3, equalC);

  te::dt::AbstractData* idA3 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(sceneId));
  te::da::Literal* literalA3 = new te::da::Literal(idA3);
  te::da::EqualTo* equalA3 = new te::da::EqualTo(pNameA3, literalA3);

  te::da::And* and5 = new te::da::And(and4, equalA3);

  te::da::Where* where = new te::da::Where(and5);

  te::da::OrderByItem* obItem = new te::da::OrderByItem(new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id));
  te::da::OrderBy* ob = new te::da::OrderBy();
  ob->push_back(obItem);

  te::da::Select select = new te::da::Select(fields, from, where, ob);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return false;
  }

  if (dataSet->size() < 1)
  {
    return false;
  }

  return true;
}

ta::model::Scene* ta::model::TaskFacade::getScene(Task* task)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_id));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_task);
  te::da::FromItem* fItemB = new te::da::DataSetName(ta::model::Globals::sm_scene);
  from->push_back(fItemA);
  from->push_back(fItemB);

  te::da::PropertyName* pNameA = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_sceneId);
  te::da::PropertyName* pNameA1 = new te::da::PropertyName(ta::model::Globals::sm_task + "." + ta::model::Task::sm_id);
  te::da::PropertyName* pNameB = new te::da::PropertyName(ta::model::Globals::sm_scene + "." + ta::model::Scene::sm_id);

  te::da::EqualTo* equalAB = new te::da::EqualTo(pNameA, pNameB);

  te::dt::AbstractData* idA1 = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(task->getId()));
  te::da::Literal* literalA1 = new te::da::Literal(idA1);

  te::da::EqualTo* equalA1 = new te::da::EqualTo(pNameA1, literalA1);

  te::dt::AbstractData* idB = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(task->getSceneId()));
  te::da::Literal* literalB = new te::da::Literal(idB);

  te::da::EqualTo* equalB = new te::da::EqualTo(pNameB, literalB);

  te::da::And* and1 = new te::da::And(equalAB, equalA1);
  te::da::And* and2 = new te::da::And(and1, equalB);

  te::da::Where* where = new te::da::Where(and2);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() != 1)
  {
    return 0;
  }

  SceneFacade sceneFacade(m_dataSource);

  dataSet->moveFirst();
  return dynamic_cast<ta::model::Scene*>(sceneFacade.get(dataSet->getInt32("id")));

}