/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/TaskLogFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../item/TaskLog.h"
#include "TaskLogFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/Max.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>

ta::model::TaskLogFacade::TaskLogFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_tasklog;
}

ta::model::TaskLogFacade::~TaskLogFacade()
{}

ta::model::TaskLog* ta::model::TaskLogFacade::getLastTaskLog(const int& taskId)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Expression* e_max = new te::da::Max(te::da::PropertyName(ta::model::Globals::sm_tasklog + "." + ta::model::TaskLog::sm_id));
  te::da::Field* field = new te::da::Field(*e_max, "ID");

  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(ta::model::Globals::sm_tasklog);
  from->push_back(fItemA);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Globals::sm_tasklog + "." + ta::model::TaskLog::sm_taskId);

  te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(taskId));
  te::da::Literal* literal = new te::da::Literal(id);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() < 1)
  {
    return 0;
  }

  dataSet->moveFirst();
  return dynamic_cast<TaskLog*>(get(dataSet->getInt32("id")));

}

bool ta::model::TaskLogFacade::updateLogTimer(const int& taskLogId, unsigned int& valueToIncreaseInSec)
{
  if (taskLogId <= 0)
  {
    return false;
  }
  if (valueToIncreaseInSec <= 0)
  {
    return false;
  }

  //TODO: use query
  std::string update = "UPDATE " + ta::model::Globals::sm_tasklog;
  update += " SET timer = timer + " + te::common::Convert2String(valueToIncreaseInSec);
  update += " WHERE id = " + te::common::Convert2String(taskLogId);

  std::auto_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();
  try
  {
    transactor->begin();
    transactor->execute(update);
    transactor->commit();
    return true;
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }

}

bool ta::model::TaskLogFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::TaskLogFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
