/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/UserFacade.h

\brief
*/

#ifndef __TERRAAMAZON_MODEL_FACADE_USERFACADE_H
#define __TERRAAMAZON_MODEL_FACADE_USERFACADE_H

// TerraAmazon
#include "AbstractFacade.h"

namespace ta
{
  namespace model
  {
    class Group;
    class User;
    
    class UserFacade: public AbstractFacade
    {

      static const std::string sm_groupUser_groupId;  //!< ...
      static const std::string sm_groupUser_userId;  //!< ...

      public:

        /*! Default Constructor */
        UserFacade(te::da::DataSourcePtr ds);

        /*! \brief Virtual destructor. */
        ~UserFacade();

      public:

        bool addAssociatedGroup(const int &groupId, const int &userId);

        bool isAssociated(const int& groupId, const int &userId);

        bool existsByName(const std::string& userName);

        std::vector<ta::model::User*> getAllowedUsers();

        std::unique_ptr<ta::model::User> getByName(const std::string& userName);

        std::vector<std::unique_ptr<ta::model::Group> > getAssociatedGroups(int userId);

        int getPrivilege(int userId);

        bool removeAssociatedGroup(const int &groupId, const int &userId);

        bool setUserProject(const int& user, const int& projId);

      protected:

        bool checkProperties(ta::model::AbstractItem* item);

        bool hasDependecy(const int& id);
    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_FACADE_USERFACADE_H
