/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/AbstractItemFactory.h

\brief ...
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_ABSTRACTITEMFACTORY_H
#define __TERRAAMAZON_MODEL_ITEM_ABSTRACTITEMFACTORY_H

// TerraAmazon
#include "../item/AbstractItem.h"

// TerraLib
#include <terralib/common/AbstractFactory.h>
#include <terralib/dataaccess/dataset/DataSet.h>

namespace ta
{
  namespace model
  {
    class AbstractItemFactory : public te::common::AbstractFactory<ta::model::AbstractItem, std::string>
    {
    
      public:

        static ta::model::AbstractItem* make(const std::string& type);

        static ta::model::AbstractItem* make(const std::string& type, te::da::DataSet* ds = 0);

        /*!
        \brief Virtual destructor.

        \note It will automatically unregister the factory from the dictionary.
        */
        virtual ~AbstractItemFactory();

        /*!
        \brief It returns the type of item that the factory knows how to create.

        \return The type of item that the factory knows how to create.
        */
        virtual std::string getType() const = 0;

      protected:

        /*!
        \brief It creates the factory.

        The key of a item is a string.

        \param factoryKey The key that identifies the factory.
        */
        AbstractItemFactory(const std::string& factoryKey);

        virtual ta::model::AbstractItem* create(te::da::DataSet* ds = 0) = 0;
    };
  }
}

#endif // __TERRAAMAZON_MODEL_ITEM_ABSTRACTITEMFACTORY_H
