/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/AoiLayerFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "AoiLayerFactory.h"
#include "../item/AoiLayer.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::AoiLayerFactory* ta::model::AoiLayerFactory::sm_factory(0);

void ta::model::AoiLayerFactory::initialize()
{
  finalize();
  sm_factory = new AoiLayerFactory;
}

void ta::model::AoiLayerFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::AoiLayerFactory::getType() const
{
  return ta::model::Globals::sm_aoiLayer;
}

ta::model::AoiLayerFactory::~AoiLayerFactory()
{

}

ta::model::AbstractItem* ta::model::AoiLayerFactory::build()
{
  return new AoiLayer();
}

ta::model::AbstractItem* ta::model::AoiLayerFactory::create(te::da::DataSet* ds)
{
  AoiLayer * aoi = new AoiLayer();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::unique_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::AoiLayer::sm_catalogId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      aoi->setCatalogId(sd->getValue());
    }
    else if (propName == ta::model::AoiLayer::sm_aoiIdColumn)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      aoi->setAoiIdColumn(sd->getValue());
    }
  }

  return aoi;
}

ta::model::AoiLayerFactory::AoiLayerFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_aoiLayer)
{

}
