/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/CatalogFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "CatalogFactory.h"
#include "../item/Catalog.h"

// TerraLib
#include <terralib/datatype/TimeInstant.h>
#include <terralib/datatype/SimpleData.h>

ta::model::CatalogFactory* ta::model::CatalogFactory::sm_factory(0);

void ta::model::CatalogFactory::initialize()
{
  finalize();
  sm_factory = new CatalogFactory;
}

void ta::model::CatalogFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::CatalogFactory::getType() const
{
  return ta::model::Globals::sm_catalog;
}

ta::model::CatalogFactory::~CatalogFactory()
{

}

ta::model::AbstractItem* ta::model::CatalogFactory::build()
{
  return new Catalog();
}

ta::model::AbstractItem* ta::model::CatalogFactory::create(te::da::DataSet* ds)
{
  Catalog * catalog = new Catalog();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Catalog::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      catalog->setId(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_name)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      catalog->setName(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_uri)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      catalog->setUri(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_type)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      catalog->setType(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_permission)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      catalog->setPermission(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_lastEdition)
    {
      catalog->setLastEdition(dynamic_cast<te::dt::TimeInstant*>(ad.get()->clone()));
    }
    else if (propName == ta::model::Catalog::sm_author)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      catalog->setAuthor(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_source)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      catalog->setSource(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_description)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      catalog->setDescription(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_scaleMin)
    {
      te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>* sd = dynamic_cast< te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>* >(ad.get());
      catalog->setScaleMin(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_scaleMax)
    {
      te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>* sd = dynamic_cast< te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>* >(ad.get());
      catalog->setScaleMax(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_style)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      catalog->setStyle(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_cellColumn)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      catalog->setCellColumn(sd->getValue());
    }
    else if (propName == ta::model::Catalog::sm_locked)
    {
      te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>* sd = dynamic_cast< te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>* >(ad.get());
      catalog->setLocked(sd->getValue());
    }
  }

  return catalog;
}

ta::model::CatalogFactory::CatalogFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_catalog)
{

}
