/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/CellFactory.cpp

\brief ...
*/

// TerraAmazon
#include "../Globals.h"
#include "CellFactory.h"
#include "../item/Cell.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>

ta::model::CellFactory* ta::model::CellFactory::sm_factory(0);

void ta::model::CellFactory::initialize()
{
  finalize();
  sm_factory = new CellFactory;
}

void ta::model::CellFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::CellFactory::getType() const
{
  return ta::model::Globals::sm_cellLayer;
}

ta::model::CellFactory::~CellFactory()
{

}

ta::model::AbstractItem* ta::model::CellFactory::build()
{
  return new Cell();
}

ta::model::AbstractItem* ta::model::CellFactory::create(te::da::DataSet* ds)
{
  Cell * c = new Cell();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Cell::sm_oid)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      c->setOid(sd->getValue());
    }
    else if (propName == ta::model::Cell::sm_processId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      c->setProcessId(sd->getValue());
    }
    else if (propName == ta::model::Cell::sm_cell)
    {
      te::gm::Geometry* sd = dynamic_cast<te::gm::Geometry*>(ad.get());

      c->setCell(*sd);
    }
  }

  return c;
}

ta::model::CellFactory::CellFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_cellLayer)
{

}
