/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/ClassFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "ClassFactory.h"
#include "../item/Class.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>

ta::model::ClassFactory* ta::model::ClassFactory::sm_factory(0);

void ta::model::ClassFactory::initialize()
{
  finalize();
  sm_factory = new ClassFactory;
}

void ta::model::ClassFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::ClassFactory::getType() const
{
  return ta::model::Globals::sm_class;
}

ta::model::ClassFactory::~ClassFactory()
{

}

ta::model::AbstractItem* ta::model::ClassFactory::build()
{
  return new Class();
}

ta::model::AbstractItem* ta::model::ClassFactory::create(te::da::DataSet* ds)
{
  Class * c = new Class();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Class::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      c->setId(sd->getValue());
    }
    else if (propName == ta::model::Class::sm_name)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      c->setName(sd->getValue());
    }
    else if (propName == ta::model::Class::sm_description)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      c->setDescription(sd->getValue());
    }
    else if (propName == ta::model::Class::sm_style)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      c->setStyle(sd->getValue());
    }
  }

  return c;
}

ta::model::ClassFactory::ClassFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_class)
{

}
