/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/GroupFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "GroupFactory.h"
#include "../item/Group.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::GroupFactory* ta::model::GroupFactory::sm_factory(0);

void ta::model::GroupFactory::initialize()
{
  finalize();
  sm_factory = new GroupFactory;
}

void ta::model::GroupFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::GroupFactory::getType() const
{
  return ta::model::Globals::sm_group;
}

ta::model::GroupFactory::~GroupFactory()
{

}

ta::model::AbstractItem* ta::model::GroupFactory::build()
{
  return new Group();
}

ta::model::AbstractItem* ta::model::GroupFactory::create(te::da::DataSet* ds)
{
  Group * group = new Group();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    std::string propName = ds->getPropertyName(t);
    std::unique_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Group::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      group->setId(sd->getValue());
    }
    else if (propName == ta::model::Group::sm_name)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      group->setName(sd->getValue());
    }
    else if (propName == ta::model::Group::sm_privileges)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      group->setPrivileges(sd->getValue());
    }
  }

  return group;
}

ta::model::GroupFactory::GroupFactory()
: ta::model::AbstractItemFactory(ta::model::Globals::sm_group)
{
}
