/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/PhaseFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "PhaseFactory.h"
#include "../item/Phase.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::PhaseFactory* ta::model::PhaseFactory::sm_factory(0);

void ta::model::PhaseFactory::initialize()
{
  finalize();
  sm_factory = new PhaseFactory;
}

void ta::model::PhaseFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::PhaseFactory::getType() const
{
  return ta::model::Globals::sm_phase;
}

ta::model::PhaseFactory::~PhaseFactory()
{

}

ta::model::AbstractItem* ta::model::PhaseFactory::build()
{
  return new Phase();
}

ta::model::AbstractItem* ta::model::PhaseFactory::create(te::da::DataSet* ds)
{
  Phase * proj = new Phase();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Phase::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      proj->setId(sd->getValue());
    }
    else if (propName == ta::model::Phase::sm_description)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      proj->setDescription(sd->getValue());
    }
  }

  return proj;
}

ta::model::PhaseFactory::PhaseFactory()
: ta::model::AbstractItemFactory(ta::model::Globals::sm_phase)
{

}
