/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/ProcessFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "ProcessFactory.h"
#include "../item/Process.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::ProcessFactory* ta::model::ProcessFactory::sm_factory(0);

void ta::model::ProcessFactory::initialize()
{
  finalize();
  sm_factory = new ProcessFactory;
}

void ta::model::ProcessFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::ProcessFactory::getType() const
{
  return ta::model::Globals::sm_process;
}

ta::model::ProcessFactory::~ProcessFactory()
{}

ta::model::AbstractItem* ta::model::ProcessFactory::build()
{
  return new Process();
}

ta::model::AbstractItem* ta::model::ProcessFactory::create(te::da::DataSet* ds)
{
  Process* process = new Process();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Process::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      process->setId(sd->getValue());
    }
    else if (propName == ta::model::Process::sm_tasklogId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      process->setTaskLogId(sd->getValue());
    }
    else if (propName == ta::model::Process::sm_taskId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      process->setTaskId(sd->getValue());
    }
  }

  return process;
}

ta::model::ProcessFactory::ProcessFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_process)
{

}
