/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/ProjectFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "ProjectFactory.h"
#include "../item/Project.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::ProjectFactory* ta::model::ProjectFactory::sm_factory(0);

void ta::model::ProjectFactory::initialize()
{
  finalize();
  sm_factory = new ProjectFactory;
}

void ta::model::ProjectFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::ProjectFactory::getType() const
{
  return ta::model::Globals::sm_project;
}

ta::model::ProjectFactory::~ProjectFactory()
{

}

ta::model::AbstractItem* ta::model::ProjectFactory::build()
{
  return new Project();
}

ta::model::AbstractItem* ta::model::ProjectFactory::create(te::da::DataSet* ds)
{
  Project * proj = new Project();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Project::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      proj->setId(sd->getValue());
    }
    else if (propName == ta::model::Project::sm_name)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      proj->setName(sd->getValue());
    }
    else if (propName == ta::model::Project::sm_maxCells)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      proj->setMaxCells(sd->getValue());
    }
    else if (propName == ta::model::Project::sm_minScale)
    {
      te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>* sd = dynamic_cast<te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>*>(ad.get());
      proj->setMinScale(sd->getValue());
    }
    else if (propName == ta::model::Project::sm_maxScale)
    {
      te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>* sd = dynamic_cast<te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>*>(ad.get());
      proj->setMaxScale(sd->getValue());
    }
    else if (propName == ta::model::Project::sm_created)
    {
      proj->setCreated(dynamic_cast<te::dt::Date*>(ad->clone()));
    }
    else if (propName == ta::model::Project::sm_updated)
    {
      proj->setUpdated(dynamic_cast<te::dt::Date*>(ad->clone()));
    }
    else if (propName == ta::model::Project::sm_active)
    {
      te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>* sd = dynamic_cast<te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>*>(ad.get());
      proj->setActive(sd->getValue());
    }
  }

  return proj;
}

ta::model::ProjectFactory::ProjectFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_project)
{

}
