/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/ProjectSceneFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "ProjectSceneFactory.h"
#include "../item/ProjectScene.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::ProjectSceneFactory* ta::model::ProjectSceneFactory::sm_factory(0);

void ta::model::ProjectSceneFactory::initialize()
{
  finalize();
  sm_factory = new ProjectSceneFactory;
}

void ta::model::ProjectSceneFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::ProjectSceneFactory::getType() const
{
  return ta::model::Globals::sm_projectScene;
}

ta::model::ProjectSceneFactory::~ProjectSceneFactory()
{

}

ta::model::AbstractItem* ta::model::ProjectSceneFactory::build()
{
  return new ProjectScene();
}

ta::model::AbstractItem* ta::model::ProjectSceneFactory::create(te::da::DataSet* ds)
{
  ProjectScene * ps = new ProjectScene();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::ProjectScene::sm_projectId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      ps->setProjectId(sd->getValue());
    }
    else if (propName == ta::model::ProjectScene::sm_sceneId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      ps->setSceneId(sd->getValue());
    }
    else if (propName == ta::model::ProjectScene::sm_locked)
    {
      te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>* sd = dynamic_cast<te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>*>(ad.get());
      ps->setLocked(sd->getValue());
    }
  }

  return ps;
}

ta::model::ProjectSceneFactory::ProjectSceneFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_projectScene)
{

}
