/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/RuleFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "RuleFactory.h"
#include "../item/Rule.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::RuleFactory* ta::model::RuleFactory::sm_factory(0);

void ta::model::RuleFactory::initialize()
{
  finalize();
  sm_factory = new RuleFactory;
}

void ta::model::RuleFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::RuleFactory::getType() const
{
  return ta::model::Globals::sm_rule;
}

ta::model::RuleFactory::~RuleFactory()
{

}

ta::model::AbstractItem* ta::model::RuleFactory::build()
{
  return new Rule();
}

ta::model::AbstractItem* ta::model::RuleFactory::create(te::da::DataSet* ds)
{
  Rule * r = new Rule();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Rule::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setId(sd->getValue());
    }
    else if (propName == ta::model::Rule::sm_name)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      r->setName(sd->getValue());
    }
    else if (propName == ta::model::Rule::sm_catalog_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setCatalogId(sd->getValue());
    }
    else if (propName == ta::model::Rule::sm_operation_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setOperationId(sd->getValue());
    }
  }

  return r;
}

ta::model::RuleFactory::RuleFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_rule)
{

}
