/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/RuleRuleSetFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "RuleRuleSetFactory.h"
#include "../item/RuleRuleSet.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::RuleRuleSetFactory* ta::model::RuleRuleSetFactory::sm_factory(0);

void ta::model::RuleRuleSetFactory::initialize()
{
  finalize();
  sm_factory = new RuleRuleSetFactory;
}

void ta::model::RuleRuleSetFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::RuleRuleSetFactory::getType() const
{
  return ta::model::Globals::sm_ruleRuleset;
}

ta::model::RuleRuleSetFactory::~RuleRuleSetFactory()
{

}

ta::model::AbstractItem* ta::model::RuleRuleSetFactory::build()
{
  return new RuleRuleSet();
}

ta::model::AbstractItem* ta::model::RuleRuleSetFactory::create(te::da::DataSet* ds)
{
  RuleRuleSet * r = new RuleRuleSet();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::RuleRuleSet::sm_ruleset_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setRuleSetId(sd->getValue());
    }
    else if (propName == ta::model::RuleRuleSet::sm_rule_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setRuleId(sd->getValue());
    }
    else if (propName == ta::model::RuleRuleSet::sm_priority)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setPriority(sd->getValue());
    }
  }

  return r;
}

ta::model::RuleRuleSetFactory::RuleRuleSetFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_ruleRuleset)
{

}
