/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/RuleSetFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "RuleSetFactory.h"
#include "../item/RuleSet.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::RuleSetFactory* ta::model::RuleSetFactory::sm_factory(0);

void ta::model::RuleSetFactory::initialize()
{
  finalize();
  sm_factory = new RuleSetFactory;
}

void ta::model::RuleSetFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::RuleSetFactory::getType() const
{
  return ta::model::Globals::sm_ruleset;
}

ta::model::RuleSetFactory::~RuleSetFactory()
{

}

ta::model::AbstractItem* ta::model::RuleSetFactory::build()
{
  return new RuleSet();
}

ta::model::AbstractItem* ta::model::RuleSetFactory::create(te::da::DataSet* ds)
{
  RuleSet * r = new RuleSet();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::RuleSet::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setId(sd->getValue());
    }
    else if (propName == ta::model::RuleSet::sm_name)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      r->setName(sd->getValue());
    }
    else if (propName == ta::model::RuleSet::sm_catalog_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setCatalogId(sd->getValue());
    }
  }

  return r;
}

ta::model::RuleSetFactory::RuleSetFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_ruleset)
{

}
