/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/RulesLayerColumnFactory.cpp

\brief ...
*/


// TerraAmazon
#include "../Globals.h"
#include "RulesLayerColumnFactory.h"
#include "../item/RulesLayerColumn.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::RulesLayerColumnFactory* ta::model::RulesLayerColumnFactory::sm_factory(0);

void ta::model::RulesLayerColumnFactory::initialize()
{
  finalize();
  sm_factory = new RulesLayerColumnFactory;
}

void ta::model::RulesLayerColumnFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::RulesLayerColumnFactory::getType() const
{
  return ta::model::Globals::sm_rulesLayerColumn;
}

ta::model::RulesLayerColumnFactory::~RulesLayerColumnFactory()
{

}

ta::model::AbstractItem* ta::model::RulesLayerColumnFactory::build()
{
  return new RulesLayerColumn();
}

ta::model::AbstractItem* ta::model::RulesLayerColumnFactory::create(te::da::DataSet* ds)
{
  RulesLayerColumn * r = new RulesLayerColumn();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t))
      continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::RulesLayerColumn::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setId(sd->getValue());
    }
    else if (propName == ta::model::RulesLayerColumn::sm_column_in)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      r->setColumnIn(sd->getValue());
    }
    else if (propName == ta::model::RulesLayerColumn::sm_column_out)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
      r->setColumnOut(sd->getValue());
    }
    else if (propName == ta::model::RulesLayerColumn::sm_rule_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setRuleId(sd->getValue());
    }
    else if (propName == ta::model::RulesLayerColumn::sm_ruleset_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
      r->setRulesetId(sd->getValue());
    }
  }

  return r;
}

ta::model::RulesLayerColumnFactory::RulesLayerColumnFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_rulesLayerColumn)
{

}
