/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/SceneFactory.cpp

\brief ...
*/

// TerraAmazon
#include "../Globals.h"
#include "SceneFactory.h"
#include "../item/Scene.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/TimeInstant.h>
#include <terralib/datatype/SimpleData.h>

ta::model::SceneFactory* ta::model::SceneFactory::sm_factory(0);

void ta::model::SceneFactory::initialize()
{
  finalize();
  sm_factory = new SceneFactory;
}

void ta::model::SceneFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::SceneFactory::getType() const
{
  return ta::model::Globals::sm_scene;
}

ta::model::SceneFactory::~SceneFactory()
{}

ta::model::AbstractItem* ta::model::SceneFactory::build()
{
  return new Scene();
}

ta::model::AbstractItem* ta::model::SceneFactory::create(te::da::DataSet* ds)
{
  Scene* scene = new Scene();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    std::string propName = ds->getPropertyName(t);
    std::unique_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Scene::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      scene->setId(sd->getValue());
    }
    else if (propName == ta::model::Scene::sm_viewDate)
    {
      std::unique_ptr<te::dt::Date> viewDate(dynamic_cast<te::dt::Date*>(ad.release()));
      scene->setViewDate(std::move(viewDate));
    }
    else if (propName == ta::model::Scene::sm_julDay)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      scene->setJulDay(sd->getValue());
    }
    else if (propName == ta::model::Scene::sm_inclusion)
    {
      std::unique_ptr<te::dt::TimeInstant> viewInstant(dynamic_cast<te::dt::TimeInstant*>(ad.release()));
      scene->setInclusion(std::move(viewInstant));
    }
    else if (propName == ta::model::Scene::sm_aiobjectId)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      scene->setAiObjectId(sd->getValue());
    }
    else if (propName == ta::model::Scene::sm_aoilayerid)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      scene->setAoiLayerId(sd->getValue());
    }
  }

  return scene;
}

ta::model::SceneFactory::SceneFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_scene)
{

}
