/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/TagTaskFactory.cpp

\brief ...
*/

// TerraAmazon
#include "../Globals.h"
#include "TagTaskFactory.h"
#include "../item/TagTask.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::TagTaskFactory* ta::model::TagTaskFactory::sm_factory(0);

void ta::model::TagTaskFactory::initialize()
{
  finalize();
  sm_factory = new TagTaskFactory;
}

void ta::model::TagTaskFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::TagTaskFactory::getType() const
{
  return ta::model::Globals::sm_tagtask;
}

ta::model::TagTaskFactory::~TagTaskFactory()
{}

ta::model::AbstractItem* ta::model::TagTaskFactory::build()
{
  return new TagTask();
}

ta::model::AbstractItem* ta::model::TagTaskFactory::create(te::da::DataSet* ds)
{
  TagTask* tagTask = new TagTask();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::TagTask::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      tagTask->setId(sd->getValue());
    }
    else if (propName == ta::model::TagTask::sm_taskId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      tagTask->setTaskId(sd->getValue());
    }
    else if (propName == ta::model::TagTask::sm_projectId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      tagTask->setProjectId(sd->getValue());
    }
    else if (propName == ta::model::TagTask::sm_label)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      tagTask->setLabel(sd->getValue());
    }
    else if (propName == ta::model::TagTask::sm_initialDate)
    {
      tagTask->setInitialDate(dynamic_cast<te::dt::Date*>(ad->clone()));
    }
    else if (propName == ta::model::TagTask::sm_finalDate)
    {
      tagTask->setFinalDate(dynamic_cast<te::dt::Date*>(ad->clone()));
    }
  }

  return tagTask;
}

ta::model::TagTaskFactory::TagTaskFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_tagtask)
{

}
