/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/TaskFactory.cpp

\brief ...
*/

// TerraAmazon
#include "../Globals.h"
#include "TaskFactory.h"
#include "../item/Task.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::TaskFactory* ta::model::TaskFactory::sm_factory(0);

void ta::model::TaskFactory::initialize()
{
  finalize();
  sm_factory = new TaskFactory;
}

void ta::model::TaskFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::TaskFactory::getType() const
{
  return ta::model::Globals::sm_task;
}

ta::model::TaskFactory::~TaskFactory()
{

}

ta::model::AbstractItem* ta::model::TaskFactory::build()
{
  return new Task();
}

ta::model::AbstractItem* ta::model::TaskFactory::create(te::da::DataSet* ds)
{
  Task* task = new Task();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::Task::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      task->setId(sd->getValue());
    }
    else if (propName == ta::model::Task::sm_sceneId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      task->setSceneId(sd->getValue());
    }
    else if (propName == ta::model::Task::sm_projectId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      task->setProjectId(sd->getValue());
    }
    else if (propName == ta::model::Task::sm_phaseId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      task->setPhaseId(sd->getValue());
    }
    else if (propName == ta::model::Task::sm_userId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      task->setUserId(sd->getValue());
    }
  }

  return task;
}

ta::model::TaskFactory::TaskFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_task)
{

}
