/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/TaskLogFactory.cpp

\brief ...
*/

// TerraAmazon
#include "../Globals.h"
#include "TaskLogFactory.h"
#include "../item/TaskLog.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/SimpleData.h>

ta::model::TaskLogFactory* ta::model::TaskLogFactory::sm_factory(0);

void ta::model::TaskLogFactory::initialize()
{
  finalize();
  sm_factory = new TaskLogFactory;
}

void ta::model::TaskLogFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::TaskLogFactory::getType() const
{
  return ta::model::Globals::sm_tasklog;
}

ta::model::TaskLogFactory::~TaskLogFactory()
{}

ta::model::AbstractItem* ta::model::TaskLogFactory::build()
{
  return new TaskLog();
}

ta::model::AbstractItem* ta::model::TaskLogFactory::create(te::da::DataSet* ds)
{
  TaskLog* tasklog = new TaskLog();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    if (ds->isNull(t)) continue;

    std::string propName = ds->getPropertyName(t);
    std::auto_ptr<te::dt::AbstractData> ad = ds->getValue(t);

    if (propName == ta::model::TaskLog::sm_id)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      tasklog->setId(sd->getValue());
    }
    else if (propName == ta::model::TaskLog::sm_taskId)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      tasklog->setTaskId(sd->getValue());
    }
    else if (propName == ta::model::TaskLog::sm_timer)
    {
      te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
      tasklog->setTimer(sd->getValue());
    }
    else if (propName == ta::model::TaskLog::sm_obs)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      tasklog->setObs(sd->getValue());
    }
    else if (propName == ta::model::TaskLog::sm_status)
    {
      te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
      tasklog->setStatus(sd->getValue());
    }
    else if (propName == ta::model::TaskLog::sm_initialTime)
    {
      tasklog->setInitialTime(dynamic_cast<te::dt::DateTime*>(ad->clone()));
    }
    else if (propName == ta::model::TaskLog::sm_finalTime)
    {
      tasklog->setFinalTime(dynamic_cast<te::dt::DateTime*>(ad->clone()));
    }
  }

  return tasklog;
}

ta::model::TaskLogFactory::TaskLogFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_tasklog)
{

}
