/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/UserFactory.cpp

\brief ...
*/

// TerraAmazon
#include "../Globals.h"
#include "UserFactory.h"
#include "../item/User.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/DateTimeProperty.h>
#include <terralib/datatype/SimpleData.h>

ta::model::UserFactory* ta::model::UserFactory::sm_factory(0);

void ta::model::UserFactory::initialize()
{
  finalize();
  sm_factory = new UserFactory;
}

void ta::model::UserFactory::finalize()
{
  delete sm_factory;
  sm_factory = 0;
}

std::string ta::model::UserFactory::getType() const
{
  return ta::model::Globals::sm_user;
}

ta::model::UserFactory::~UserFactory()
{

}

ta::model::AbstractItem* ta::model::UserFactory::build()
{
  return new User();
}

ta::model::AbstractItem* ta::model::UserFactory::create(te::da::DataSet* ds)
{
  User * user = new User();

  for (std::size_t t = 0; t < ds->getNumProperties(); ++t)
  {
    std::string propName = ds->getPropertyName(t);
    std::unique_ptr<te::dt::AbstractData> ad;
    if (!ds->isNull(t))
    {
      ad = ds->getValue(t);

      if (propName == ta::model::User::sm_id)
      {
        te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
        user->setId(sd->getValue());
      }
      else if (propName == ta::model::User::sm_projectId)
      {
        te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast< te::dt::SimpleData<int, te::dt::INT32_TYPE>* >(ad.get());
        user->setProjectId(sd->getValue());
      }
      else if (propName == ta::model::User::sm_name)
      {
        te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
        user->setName(sd->getValue());
      }
      else if (propName == ta::model::User::sm_password)
      {
        te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast<te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(ad.get());
        user->setPassword(sd->getValue());
      }
      else if (propName == ta::model::User::sm_eMail)
      {
        te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* sd = dynamic_cast< te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* >(ad.get());
        user->setEMail(sd->getValue());
      }
      else if (propName == ta::model::User::sm_initialDate)
      {
        user->setInitialDate(dynamic_cast<te::dt::Date*>(ad.release()));
      }
      else if (propName == ta::model::User::sm_finalDate)
      {
        user->setFinalDate(dynamic_cast<te::dt::Date*>(ad.release()));
      }
      else if (propName == ta::model::User::sm_active)
      {
        te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>* sd = dynamic_cast<te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>*>(ad.get());
        user->setIsActive(sd->getValue());
      }
    }
  }

  return user;
}

ta::model::UserFactory::UserFactory()
  : ta::model::AbstractItemFactory(ta::model::Globals::sm_user)
{

}
