/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/AoiLayer.h

\brief This file defines the Project Scene model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "AoiLayer.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::AoiLayer::sm_catalogId("catalog_id");
const std::string ta::model::AoiLayer::sm_aoiIdColumn("aoi_id_column");

ta::model::AoiLayer::AoiLayer()
  : AbstractItem(),
  m_catalogId(-1),
  m_aoiIdColumn("")
{}

ta::model::AoiLayer::~AoiLayer()
{}

ta::model::ItemPropertyVector ta::model::AoiLayer::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::AoiLayer::sm_catalogId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_catalogId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::AoiLayer::sm_catalogId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::AoiLayer::sm_catalogId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::AoiLayer::sm_aoiIdColumn, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_aoiIdColumn);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::AoiLayer::sm_aoiIdColumn);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::AoiLayer::sm_aoiIdColumn, mep, ipv);
  }

  return ipv;
}

int ta::model::AoiLayer::getCatalogId()
{
  return m_catalogId;
}

std::string ta::model::AoiLayer::getAoiIdColumn()
{
  return m_aoiIdColumn;
}

void ta::model::AoiLayer::setCatalogId(const int& newCatalogId)
{
  m_catalogId = newCatalogId;
}

void ta::model::AoiLayer::setAoiIdColumn(const std::string& newAoiIdColumn)
{
  m_aoiIdColumn = newAoiIdColumn;
}
