/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/AoiLayer.h

\brief This file defines the area of interest layer model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_AOILAYER_H
#define __TERRAAMAZON_MODEL_ITEM_AOILAYER_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace ta
{
  namespace model
  {
    /*!
    \class AoiLayer

    \brief This class defines the area of interest layer model.

    \ingroup model

    \sa AbstractItem
    */
    class AoiLayer : public AbstractItem
    {
      public:

        static const std::string sm_catalogId;      //!< ...
        static const std::string sm_aoiIdColumn;  //!< ...

      public:

        /*! \brief Default constructor  */
        AoiLayer();

        /*! \brief Default destructor  */
        ~AoiLayer();

      public:

        ItemPropertyVector toProperties();

      public:
        /** @name Getters
        */
        //@{

        int getCatalogId();
        std::string getAoiIdColumn();

        //@}

        /** @name Setters
        */
        //@{

        void setCatalogId(const int& newCatalogId);
        void setAoiIdColumn(const std::string& newAoiIdColumn);

        //@}

      private:

        int          m_catalogId;    //!< ...
        std::string  m_aoiIdColumn;  //!< ...

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_AOILAYER_H

