/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Catalog.h

\brief This file defines the Catalog model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "Catalog.h"

// TerraLib
#include <terralib/datatype/DateTimeProperty.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::Catalog::sm_id("id");
const std::string ta::model::Catalog::sm_name("name");
const std::string ta::model::Catalog::sm_uri("uri");
const std::string ta::model::Catalog::sm_type("source_type");
const std::string ta::model::Catalog::sm_permission("permission");
const std::string ta::model::Catalog::sm_lastEdition("last_edition");
const std::string ta::model::Catalog::sm_author("author");
const std::string ta::model::Catalog::sm_source("source");
const std::string ta::model::Catalog::sm_description("description");
const std::string ta::model::Catalog::sm_scaleMin("scale_min");
const std::string ta::model::Catalog::sm_scaleMax("scale_max");
const std::string ta::model::Catalog::sm_style("style");
const std::string ta::model::Catalog::sm_cellColumn("cell_column");
const std::string ta::model::Catalog::sm_locked("locked");

ta::model::Catalog::Catalog()
  : AbstractItem(),
  m_permission(0),
  m_lastEdition(0),
  m_scaleMin(std::numeric_limits<double>::max()),
  m_scaleMax(-std::numeric_limits<double>::max()),
  m_style(""),
  m_cellColumn(""),
  m_locked(false)
{

}

ta::model::Catalog::~Catalog()
{
  delete m_lastEdition;
}

ta::model::ItemPropertyVector ta::model::Catalog::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Catalog::sm_name, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_name);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_name);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_name, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Catalog::sm_uri, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_uri);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_uri);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_uri, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Catalog::sm_type, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_type);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_type);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_type, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Catalog::sm_permission, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_permission);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_permission);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_permission, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::Catalog::sm_lastEdition, te::dt::DateTimeType::TIME_INSTANT);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_lastEdition);
    mep->setProperty(p);
    mep->setData(m_lastEdition->clone());

    addProperty(ta::model::Catalog::sm_lastEdition, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Catalog::sm_author, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_author);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_author);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_author, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Catalog::sm_source, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_source);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_source);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_source, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Catalog::sm_description, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_description);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_description);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_description, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Catalog::sm_scaleMin, te::dt::DOUBLE_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>(m_scaleMin);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_scaleMin);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_scaleMin, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Catalog::sm_scaleMax, te::dt::DOUBLE_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>(m_scaleMax);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_scaleMax);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_scaleMax, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Catalog::sm_style, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_style);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_style);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_style, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Catalog::sm_cellColumn, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_cellColumn);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_cellColumn);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_cellColumn, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Catalog::sm_locked, te::dt::BOOLEAN_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>(m_locked);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Catalog::sm_locked);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Catalog::sm_locked, mep, ipv);
  }

  return ipv;
}

std::string ta::model::Catalog::getName()
{
  return m_name;
}

std::string ta::model::Catalog::getUri()
{
  return m_uri;
}

std::string ta::model::Catalog::getType()
{
  return m_type;
}

unsigned int ta::model::Catalog::getPermission()
{
  return m_permission;
}

te::dt::TimeInstant* ta::model::Catalog::getLastEdition()
{
  return m_lastEdition;
}

std::string ta::model::Catalog::getAuthor()
{
  return m_author;
}

std::string ta::model::Catalog::getSource()
{
  return m_source;
}

std::string ta::model::Catalog::getDescription()
{
  return m_description;
}

double ta::model::Catalog::getScaleMin()
{
  return m_scaleMin;
}

double ta::model::Catalog::getScaleMax()
{
  return m_scaleMax;
}

std::string ta::model::Catalog::getStyle()
{
  return m_style;
}

std::string ta::model::Catalog::getCellColumn()
{
  return m_cellColumn;
}

bool ta::model::Catalog::isLocked()
{
  return m_locked;
}

void ta::model::Catalog::setName(const std::string& name)
{
  m_name = name;
}

void ta::model::Catalog::setUri(const std::string& uri)
{
  m_uri = uri;
}

void ta::model::Catalog::setType(const std::string& type)
{
  m_type = type;
}

void ta::model::Catalog::setPermission(unsigned int permission)
{
  m_permission = permission;
}

void ta::model::Catalog::setLastEdition(te::dt::TimeInstant* lastEdition)
{
  m_lastEdition = lastEdition;
}

void ta::model::Catalog::setAuthor(const std::string& author)
{
  m_author = author;
}

void ta::model::Catalog::setSource(const std::string& source)
{
  m_source = source;
}

void ta::model::Catalog::setDescription(const std::string& description)
{
  m_description = description;
}

void ta::model::Catalog::setScaleMin(double min)
{
  m_scaleMin = min;
}

void ta::model::Catalog::setScaleMax(double max)
{
  m_scaleMax = max;
}

void ta::model::Catalog::setStyle(const std::string& style)
{
  m_style = style;
}

void ta::model::Catalog::setCellColumn(const std::string& name)
{
  m_cellColumn = name;
}

void ta::model::Catalog::setLocked(bool locked)
{
  m_locked = locked;
}
