/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Catalog.h

\brief This file defines the Catalog model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_CATALOG_H
#define __TERRAAMAZON_MODEL_ITEM_CATALOG_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace te
{
  namespace dt
  {
    class TimeInstant;
  }
}

namespace ta
{
  namespace model
  {
    /*!
    \class Catalog

    \brief This class defines the project model.

    \ingroup model

    \sa AbstractItem
    */
    class Catalog : public AbstractItem
    {
      public:

        static const std::string sm_id;           //!< ...
        static const std::string sm_name;         //!< ...
        static const std::string sm_uri;          //!< ...
        static const std::string sm_type;         //!< ...
        static const std::string sm_permission;   //!< ...
        static const std::string sm_lastEdition;  //!< ...
        static const std::string sm_author;       //!< ...
        static const std::string sm_source;       //!< ...
        static const std::string sm_description;  //!< ...
        static const std::string sm_scaleMin;     //!< ...
        static const std::string sm_scaleMax;     //!< ...
        static const std::string sm_style;        //!< ...
        static const std::string sm_cellColumn;   //!< ...
        static const std::string sm_locked;       //!< ...

      public:

        /*! \brief Default constructor  */
        Catalog();

        /*! \brief Default destructor  */
        ~Catalog();

      public:

        ItemPropertyVector toProperties();

      public:

        /** @name Getters
        */
        //@{

        /*!
        \brief ...

        \return ...
        */
        std::string getName();

        /*!
        \brief ...

        \return ...
        */
        std::string getUri();

        /*!
        \brief ...

        \return ...
        */
        std::string getType();

        /*!
        \brief ...

        \return ...
        */
        unsigned int getPermission();

        /*!
        \brief ...

        \return ...
        */
        te::dt::TimeInstant* getLastEdition();

        /*!
        \brief ...

        \return ...
        */
        std::string getAuthor();

        /*!
        \brief ...

        \return ...
        */
        std::string  getSource();

        /*!
        \brief ...

        \return ...
        */
        std::string getDescription();

        /*!
        \brief ...

        \return ...
        */
        double getScaleMin();

        /*!
        \brief ...

        \return ...
        */
        double getScaleMax();

        /*!
        \brief ...

        \return ...
        */
        std::string getStyle();

        /*!
        \brief ...

        \return ...
        */
        std::string getCellColumn();

        /*!
        \brief ...

        \return ...
        */
        bool isLocked();

        //@}

        /** @name Setters
        */
        //@{

        /*!
        \brief ...

        \param ...
        */
        void setName(const std::string& name);

        /*!
        \brief ...

        \param ...
        */
        void setUri(const std::string& uri);

        /*!
        \brief ...

        \param ...
        */
        void setType(const std::string& type);

        /*!
        \brief ...

        \param ...
        */
        void setPermission(unsigned int permission);

        /*!
        \brief ...

        \param ...
        */
        void setLastEdition(te::dt::TimeInstant* lastEdition);

        /*!
        \brief ...

        \param ...
        */
        void setAuthor(const std::string& author);

        /*!
        \brief ...

        \param ...
        */
        void setSource(const std::string& source);

        /*!
        \brief ...

        \param ...
        */
        void setDescription(const std::string& description);

        /*!
        \brief ...

        \param ...
        */
        void setScaleMin(double min);

        /*!
        \brief ...

        \param ...
        */
        void setScaleMax(double max);

        /*!
        \brief ...

        \param ...
        */
        void setStyle(const std::string& style);

        /*!
        \brief ...

        \param ...
        */
        void setCellColumn(const std::string& name);

        /*!
        \brief ...

        \param ...
        */
        void setLocked(bool locked);

      //@}

      private:

        std::string m_name;                       //!< ...
        std::string m_uri;                        //!< ...
        std::string m_type;                       //!< ...
        unsigned int m_permission;                //!< ...
        te::dt::TimeInstant* m_lastEdition;       //!< ...
        std::string m_author;                     //!< ...
        std::string m_source;                     //!< ...
        std::string m_description;                //!< ...
        double m_scaleMin;                        //!< ...
        double m_scaleMax;                        //!< ...
        std::string m_style;                      //!< ...
        std::string m_cellColumn;                 //!< ...
        bool m_locked;                            //!< ...
    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_CATALOG_H

