/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Cell.cpp

\brief This file defines the Cell model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "Cell.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::Cell::sm_oid("id");
const std::string ta::model::Cell::sm_processId("process_id");
const std::string ta::model::Cell::sm_cell("geom");

ta::model::Cell::Cell()
: AbstractItem(),
  m_oid(""),
  m_processId(-1)
{}

ta::model::Cell::~Cell()
{
}

ta::model::ItemPropertyVector ta::model::Cell::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    if (m_oid != "")
    {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Cell::sm_oid, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_oid);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Cell::sm_oid);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::Cell::sm_oid, mep, ipv);

    }
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Cell::sm_processId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_processId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Cell::sm_processId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Cell::sm_processId, mep, ipv);
  }

  return ipv;
}

std::string ta::model::Cell::getOid()
{
  return m_oid;
}

int ta::model::Cell::getProcessId()
{
  return m_processId;
}

te::gm::Geometry* ta::model::Cell::getCell()
{
  return m_cell.get();
}

void ta::model::Cell::setOid(const std::string& oid)
{
  m_oid = oid;
}

void ta::model::Cell::setProcessId(const int& processId)
{
  m_processId = processId;
}

void ta::model::Cell::setCell(te::gm::Geometry& cell)
{
  m_cell.reset(static_cast<te::gm::Geometry*>(cell.clone()));
}
