/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Cell.h

\brief This file defines the Cell model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_CELL_H
#define __TERRAAMAZON_MODEL_ITEM_CELL_H

// TerraLib
#include <terralib/geometry/Geometry.h>

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace ta
{
  namespace model
  {
    /*!
    \class Cell

    \brief This class defines the cell model.

    \ingroup model

    \sa AbstractItem
    */
    class Cell : public AbstractItem
    {
      public:

        static const std::string sm_oid;           //!< ...
        static const std::string sm_processId;    //!< ...
        static const std::string sm_cell;         //!< ...

      public:

        /*! \brief Default constructor  */
        Cell();

        /*! \brief Default destructor  */
        ~Cell();

      public:

        ItemPropertyVector toProperties();

      public:

        /** @name Getters
        */
        //@{

        /*!
        \brief ...

        \return ...
        */
        std::string getOid();

        /*!
        \brief ...

        \return ...
        */
        int getProcessId();

        /*!
        \brief ...

        \return ...
        */
        te::gm::Geometry* getCell();

        //@}

        /** @name Setters
        */
        //@{

        /*!
        \brief ...

        \param ...
        */
        void setOid(const std::string& oid);

        /*!
        \brief ...

        \param ...
        */
        void setProcessId(const int& processId);

        /*!
        \brief ...

        \param ...
        */
        void setCell(te::gm::Geometry& cell);

      //@}

      private:

        std::string                       m_oid;        //!< ...
        int                               m_processId; //!< ...
        std::unique_ptr<te::gm::Geometry> m_cell;      //!< ...

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_CELL_H

