/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Class.cpp

\brief This file defines the Class model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "Class.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::Class::sm_id("id");
const std::string ta::model::Class::sm_name("name");
const std::string ta::model::Class::sm_description("description");
const std::string ta::model::Class::sm_style("style");


ta::model::Class::Class()
  : AbstractItem(),
  m_name(""),
  m_desc(""),
  m_style("")
{

}

ta::model::Class::~Class()
{
}

ta::model::ItemPropertyVector ta::model::Class::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    if (m_id != -1)
    {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Class::sm_id, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_id);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Class::sm_id);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::Class::sm_id, mep, ipv);
    }
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Class::sm_name, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_name);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Class::sm_name);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Class::sm_name, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Class::sm_description, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_desc);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Class::sm_description);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Class::sm_description, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Class::sm_style, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_style);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Class::sm_style);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Class::sm_style, mep, ipv);
  }

  return ipv;
}

std::string ta::model::Class::getName()
{
  return m_name;
}

std::string ta::model::Class::getDescription()
{
  return m_desc;
}

std::string ta::model::Class::getStyle()
{
  return m_style;
}


void ta::model::Class::setName(const std::string& name)
{
  m_name = name;
}

void ta::model::Class::setDescription(const std::string& desc)
{
  m_desc = desc;
}

void ta::model::Class::setStyle(const std::string& style)
{
  m_style = style;
}
