/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Group.h

\brief This file defines the Group model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "Group.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::Group::sm_id("id");
const std::string ta::model::Group::sm_name("name");
const std::string ta::model::Group::sm_privileges("privileges");

ta::model::Group::Group()
: AbstractItem(),
m_name(""),
m_privileges(1)
{

}

ta::model::Group::~Group()
{
}

ta::model::ItemPropertyVector ta::model::Group::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    if (m_id != -1)
    {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Group::sm_id, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_id);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Group::sm_id);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::Group::sm_id, mep, ipv);
    }
  }
  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Group::sm_name, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_name);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Group::sm_name);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Group::sm_name, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Group::sm_privileges, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_privileges);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Group::sm_privileges);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Group::sm_privileges, mep, ipv);
  }

  return ipv;
}

std::string ta::model::Group::getName()
{
  return m_name;
}

int ta::model::Group::getPrivileges()
{
  return m_privileges;
}

void ta::model::Group::setName(const std::string& name)
{
  m_name = name;
}

void ta::model::Group::setPrivileges(const int& privileges)
{
  m_privileges = privileges;
}

