/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Project.h

\brief This file defines the Project model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_PROJECT_H
#define __TERRAAMAZON_MODEL_ITEM_PROJECT_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace te
{
  namespace dt
  {
    class Date;
  }
}

namespace ta
{
  namespace model
  {
    /*!
    \class Project

    \brief This class defines the project model.

    \ingroup model

    \sa AbstractItem
    */
    class Project : public AbstractItem
    {
      public:

        static const std::string sm_id;        //!< ...
        static const std::string sm_name;      //!< ...
        static const std::string sm_active;    //!< ...
        static const std::string sm_minScale;  //!< ...
        static const std::string sm_maxScale;  //!< ...
        static const std::string sm_created;   //!< ...
        static const std::string sm_updated;   //!< ...
        static const std::string sm_maxCells;  //!< ...

      public:

        /*! \brief Default constructor  */
        Project();

        /*! \brief Default destructor  */
        ~Project();

      public:

        ItemPropertyVector toProperties();

      public:

        /** @name Getters
        */
        //@{

        /*!
        \brief ...

        \return ...
        */
        std::string getName();

        /*!
        \brief ...

        \return ...
        */
        double getMinScale();

        /*!
        \brief ...

        \return ...
        */
        double getMaxScale();

        /*!
        \brief ...

        \return ...
        */
        te::dt::Date* getCreated();

        /*!
        \brief ...

        \return ...
        */
        te::dt::Date* getUpdated();

        /*!
        \brief ...

        \return ...
        */
        std::size_t getMaxCells();

        /*!
        \brief ...

        \return ...
        */
        bool isActive();

        //@}

        /** @name Setters
        */
        //@{

        /*!
        \brief ...

        \param ...
        */
        void setName(const std::string& name);

        /*!
        \brief ...

        \param ...
        */
        void setMinScale(const double& scale);

        /*!
        \brief ...

        \param ...
        */
        void setMaxScale(const double& scale);

        /*!
        \brief ...

        \param ...
        */
        void setCreated(te::dt::Date* date);

        /*!
        \brief ...

        \param ...
        */
        void setUpdated(te::dt::Date* date);

        /*!
        \brief ...

        \param ...
        */
        void setMaxCells(const std::size_t& maxCells);

        /*!
        \brief ...

        \param ...
        */
        void setActive(const bool& active);

      //@}

      private:

        std::string               m_name;     //!< ...
        bool                      m_active;   //!< ...
        double                    m_minScale; //!< ...
        double                    m_maxScale; //!< ...
        te::dt::Date*             m_created;  //!< ...
        te::dt::Date*             m_updated;  //!< ...
        std::size_t               m_maxCells; //!< ...
    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_PROJECT_H

