/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/ProjectScene.h

\brief This file defines the Project Scene model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "ProjectScene.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::ProjectScene::sm_projectId("project_id");
const std::string ta::model::ProjectScene::sm_sceneId("scene_id");
const std::string ta::model::ProjectScene::sm_locked("locked");

ta::model::ProjectScene::ProjectScene()
  : AbstractItem(),
  m_sceneId(-1),
  m_projectId(-1),
  m_locked(false)
{}

ta::model::ProjectScene::~ProjectScene()
{}

ta::model::ItemPropertyVector ta::model::ProjectScene::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::ProjectScene::sm_projectId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_projectId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::ProjectScene::sm_projectId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::ProjectScene::sm_projectId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::ProjectScene::sm_sceneId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_sceneId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::ProjectScene::sm_sceneId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::ProjectScene::sm_sceneId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::ProjectScene::sm_locked, te::dt::BOOLEAN_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>(m_locked);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::ProjectScene::sm_locked);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::ProjectScene::sm_locked, mep, ipv);
  }

  return ipv;
}

int ta::model::ProjectScene::getSceneId()
{
  return m_sceneId;
}

int ta::model::ProjectScene::getProjectId()
{
  return m_projectId;
}

bool ta::model::ProjectScene::isLocked()
{
  return m_locked;
}

void ta::model::ProjectScene::setSceneId(const int& newsceneId)
{
  m_sceneId = newsceneId;
}

void ta::model::ProjectScene::setProjectId(const int& newprojId)
{
  m_projectId = newprojId;
}

void ta::model::ProjectScene::setLocked(const bool& locked)
{
  m_locked = locked;
}
