/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/ProjectScene.h

\brief This file defines the Project Scene model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_PROJECTSCENE_H
#define __TERRAAMAZON_MODEL_ITEM_PROJECTSCENE_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace ta
{
  namespace model
  {
    /*!
    \class ProjectScene

    \brief This class defines the project scene model.

    \ingroup model

    \sa AbstractItem
    */
    class ProjectScene : public AbstractItem
    {
      public:

        static const std::string sm_projectId; //!< ...
        static const std::string sm_sceneId;   //!< ...
        static const std::string sm_locked;    //!< ...

      public:

        /*! \brief Default constructor  */
        ProjectScene();

        /*! \brief Default destructor  */
        ~ProjectScene();

      public:

        ItemPropertyVector toProperties();

      public:
        /** @name Getters
        */
        //@{

        int getProjectId();
        int getSceneId();

        bool isLocked();

        //@}

        /** @name Setters
        */
        //@{

        void setProjectId(const int& newprojId);
        void setSceneId(const int& newsceneId);

        void setLocked(const bool& locked);

        //@}

      private:

        int                       m_sceneId;    //!< ...
        int                       m_projectId;  //!< ...
        bool                      m_locked;     //!< ...

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_PROJECTSCENE_H

