/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/RuleRuleSet.h

\brief This file defines the RuleRuleSet model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "RuleRuleSet.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::RuleRuleSet::sm_ruleset_id("ruleset_id");
const std::string ta::model::RuleRuleSet::sm_rule_id("rule_id");
const std::string ta::model::RuleRuleSet::sm_priority("priority");

ta::model::RuleRuleSet::RuleRuleSet()
  : AbstractItem(),
    m_ruleSetId(-1),
    m_ruleId(-1),
    m_priority(-1)
{

}

ta::model::RuleRuleSet::~RuleRuleSet()
{

}

ta::model::ItemPropertyVector ta::model::RuleRuleSet::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RuleRuleSet::sm_ruleset_id, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_ruleSetId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RuleRuleSet::sm_ruleset_id);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RuleRuleSet::sm_ruleset_id, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RuleRuleSet::sm_rule_id, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_ruleId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RuleRuleSet::sm_rule_id);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RuleRuleSet::sm_rule_id, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RuleRuleSet::sm_priority, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_priority);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RuleRuleSet::sm_priority);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RuleRuleSet::sm_priority, mep, ipv);
  }

  return ipv;
}

int  ta::model::RuleRuleSet::getRuleSetId()
{
  return m_ruleSetId;
}

int  ta::model::RuleRuleSet::getRuleId()
{
  return m_ruleId;
}

int  ta::model::RuleRuleSet::getPriority()
{
  return m_priority;
}

void  ta::model::RuleRuleSet::setRuleSetId(const int& id)
{
  m_ruleSetId = id;
}

void  ta::model::RuleRuleSet::setRuleId(const int& id)
{
  m_ruleId = id;
}

void  ta::model::RuleRuleSet::setPriority(const int& priority)
{
  m_priority = priority;
}
