/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/RuleSet.h

\brief This file defines the RuleSet model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "RuleSet.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::RuleSet::sm_id("id");
const std::string ta::model::RuleSet::sm_name("name");
const std::string ta::model::RuleSet::sm_catalog_id("catalog_id");

ta::model::RuleSet::RuleSet()
  : AbstractItem(),
    m_catalogId(-1)
{

}

ta::model::RuleSet::~RuleSet()
{

}

ta::model::ItemPropertyVector ta::model::RuleSet::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  if (m_id != -1)
  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RuleSet::sm_id, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_id);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RuleSet::sm_id);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RuleSet::sm_id, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::RuleSet::sm_name, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_name);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RuleSet::sm_name);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RuleSet::sm_name, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RuleSet::sm_catalog_id, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_catalogId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RuleSet::sm_catalog_id);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RuleSet::sm_catalog_id, mep, ipv);
  }

  return ipv;
}

std::string ta::model::RuleSet::getName()
{
  return m_name;
}

int ta::model::RuleSet::getCatalogId()
{
  return m_catalogId;
}

void ta::model::RuleSet::setName(const std::string& name)
{
  m_name = name;
}

void ta::model::RuleSet::setCatalogId(const int& id)
{
  m_catalogId = id;
}
