/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/RulesLayerColumn.h

\brief This file defines the RulesLayerColumn model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "RulesLayerColumn.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::RulesLayerColumn::sm_id("id");
const std::string ta::model::RulesLayerColumn::sm_column_in("column_in");
const std::string ta::model::RulesLayerColumn::sm_column_out("column_out");
const std::string ta::model::RulesLayerColumn::sm_rule_id("rule_id");
const std::string ta::model::RulesLayerColumn::sm_ruleset_id("ruleset_id");

ta::model::RulesLayerColumn::RulesLayerColumn()
  : AbstractItem(),
  m_columnIn(""),
  m_columnOut(""),
  m_ruleId(-1),
  m_rulesetId(-1)
{

}

ta::model::RulesLayerColumn::~RulesLayerColumn()
{

}

ta::model::ItemPropertyVector ta::model::RulesLayerColumn::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  if (m_id != -1)
  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RulesLayerColumn::sm_id, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_id);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RulesLayerColumn::sm_id);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RulesLayerColumn::sm_id, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::RulesLayerColumn::sm_column_in, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_columnIn);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RulesLayerColumn::sm_column_in);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RulesLayerColumn::sm_column_in, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::RulesLayerColumn::sm_column_out, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_columnOut);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RulesLayerColumn::sm_column_out);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RulesLayerColumn::sm_column_out, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RulesLayerColumn::sm_rule_id, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_ruleId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RulesLayerColumn::sm_rule_id);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RulesLayerColumn::sm_rule_id, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RulesLayerColumn::sm_ruleset_id, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_rulesetId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RulesLayerColumn::sm_ruleset_id);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RulesLayerColumn::sm_ruleset_id, mep, ipv);
  }

  return ipv;
}

std::string ta::model::RulesLayerColumn::getColumnIn()
{
  return m_columnIn;
}

std::string ta::model::RulesLayerColumn::getColumnOut()
{
  return m_columnOut;
}

int ta::model::RulesLayerColumn::getRuleId()
{
  return m_ruleId;
}

int ta::model::RulesLayerColumn::getRulesetId()
{
  return m_rulesetId;
}

void ta::model::RulesLayerColumn::setColumnIn(const std::string& name)
{
  m_columnIn = name;
}

void ta::model::RulesLayerColumn::setColumnOut(const std::string& name)
{
  m_columnOut = name;
}

void ta::model::RulesLayerColumn::setRuleId(const int& id)
{
  m_ruleId = id;
}

void ta::model::RulesLayerColumn::setRulesetId(const int& id)
{
  m_rulesetId = id;
}
