/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/RulesLayerParams.h

\brief
*/

// TerraAmazon
#include "ItemProperties.h"
#include "RulesLayerParams.h"

// TerraLib
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::RulesLayerParams::sm_catalogId("catalog_id");
const std::string ta::model::RulesLayerParams::sm_classColumn("class_column");

ta::model::RulesLayerParams::RulesLayerParams()
  : AbstractItem(),
  m_catalogId(-1),
  m_classColumn("")
{}

ta::model::RulesLayerParams::~RulesLayerParams()
{}

ta::model::ItemPropertyVector ta::model::RulesLayerParams::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::RulesLayerParams::sm_catalogId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_catalogId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RulesLayerParams::sm_catalogId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RulesLayerParams::sm_catalogId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::RulesLayerParams::sm_classColumn, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_classColumn);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::RulesLayerParams::sm_classColumn);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::RulesLayerParams::sm_classColumn, mep, ipv);
  }

  return ipv;
}

int ta::model::RulesLayerParams::getCatalogId()
{
  return m_catalogId;
}

std::string ta::model::RulesLayerParams::getClassColumn()
{
  return m_classColumn;
}

void ta::model::RulesLayerParams::setCatalogId(const int& newCatalogId)
{
  m_catalogId = newCatalogId;
}

void ta::model::RulesLayerParams::setClassColumn(const std::string& newClassColumn)
{
  m_classColumn = newClassColumn;
}
