/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Scene.h

\brief This file defines the Scene model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "Scene.h"

// TerraLib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/DateTimeProperty.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>
#include <terralib/datatype/TimeInstant.h>

const std::string ta::model::Scene::sm_id("id");
const std::string ta::model::Scene::sm_viewDate("view_date");
const std::string ta::model::Scene::sm_julDay("julday");
const std::string ta::model::Scene::sm_inclusion("inclusion");
const std::string ta::model::Scene::sm_aiobjectId("ai_object_id");
const std::string ta::model::Scene::sm_aoilayerid("aoi_layer_id");

ta::model::Scene::Scene()
  : AbstractItem(),
  m_julday(-1),
  m_aoilayerId(-1),
  m_aiobjectId("")
{}

ta::model::Scene::~Scene()
{}

ta::model::ItemPropertyVector ta::model::Scene::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    if (m_id != -1)
    {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Scene::sm_id, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_id);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Scene::sm_id);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::Scene::sm_id, mep, ipv);
    }
  }
  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::Scene::sm_viewDate, te::dt::DateTimeType::DATE);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Scene::sm_viewDate);
    mep->setProperty(p);
    mep->setData(m_viewDate->clone());

    addProperty(ta::model::Scene::sm_viewDate, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Scene::sm_julDay, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_julday);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Scene::sm_julDay);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Scene::sm_julDay, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::Scene::sm_inclusion, te::dt::DateTimeType::TIME_INSTANT);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Scene::sm_inclusion);
    mep->setProperty(p);
    mep->setData(m_inclusion->clone());

    addProperty(ta::model::Scene::sm_inclusion, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Scene::sm_aiobjectId, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_aiobjectId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Scene::sm_aiobjectId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Scene::sm_aiobjectId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Scene::sm_aoilayerid, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_aoilayerId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Scene::sm_aoilayerid);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Scene::sm_aoilayerid, mep, ipv);
  }

  return ipv;
}


te::dt::Date* ta::model::Scene::getViewDate()
{
  return m_viewDate.get();
}

int ta::model::Scene::getJulDay()
{
  return m_julday;
}

te::dt::TimeInstant* ta::model::Scene::getInclusion()
{
  return m_inclusion.get();
}

std::string ta::model::Scene::getAiObjectId()
{
  return m_aiobjectId;
}

int ta::model::Scene::getAoiLayerId()
{
  return m_aoilayerId;
}

void ta::model::Scene::setJulDay(const int& julDay)
{
  m_julday = julDay;
}

void ta::model::Scene::setViewDate(std::unique_ptr<te::dt::Date> date)
{
  m_viewDate = std::move(date);
}

void ta::model::Scene::setInclusion(std::unique_ptr<te::dt::TimeInstant> instant)
{
  m_inclusion = std::move(instant);
}

void ta::model::Scene::setAiObjectId(const std::string& newaiobjectId)
{
  m_aiobjectId = newaiobjectId;
}

void ta::model::Scene::setAoiLayerId(const int& newaoilayerId)
{
  m_aoilayerId = newaoilayerId;
}