/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Scene.h

\brief This file defines the Scene model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_SCENE_H
#define __TERRAAMAZON_MODEL_ITEM_SCENE_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace te
{
  namespace dt
  {
    class Date;
    class TimeInstant;
  }
}

namespace ta
{
  namespace model
  {
    /*!
    \class Scene

    \brief This class defines the Scene model.

    \ingroup model

    \sa AbstractItem
    */
    class Scene : public AbstractItem
    {
      public:

        static const std::string sm_id;           //!< ...
        static const std::string sm_viewDate;     //!< ...
        static const std::string sm_julDay;       //!< ...
        static const std::string sm_inclusion;    //!< ...
        static const std::string sm_aiobjectId;   //!< ...
        static const std::string sm_aoilayerid;    //!< ...

        /*! \brief Default constructor  */
        Scene();

        /*! \brief Default destructor  */
        ~Scene();

      public:

        ItemPropertyVector toProperties();

      public:

        /** @name Getters
        */
        //@{

        int getJulDay();
        int getAoiLayerId();

        te::dt::Date* getViewDate();
        te::dt::TimeInstant* getInclusion();

        std::string getAiObjectId();

        //@}

        /** @name Setters
        */
        //@{

        void setJulDay(const int& juldayId);
        void setAoiLayerId(const int& aoilayerId);

        void setViewDate(std::unique_ptr<te::dt::Date> date);
        void setInclusion(std::unique_ptr<te::dt::TimeInstant> instant);

        void setAiObjectId(const std::string& aiobjectId);

      //@}

      private:

        int                                   m_julday;
        int                                   m_aoilayerId;
        std::unique_ptr<te::dt::Date>         m_viewDate;
        std::unique_ptr<te::dt::TimeInstant>  m_inclusion;
        std::string                           m_aiobjectId;

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_SCENE_H

