/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/TagTask.h

\brief This file defines the TagTask model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "TagTask.h"

// TerraLib
#include <terralib/datatype/DateTimeInstant.h>
#include <terralib/datatype/DateTimeProperty.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::TagTask::sm_id("id");
const std::string ta::model::TagTask::sm_taskId("task_Id");
const std::string ta::model::TagTask::sm_projectId("project_Id");
const std::string ta::model::TagTask::sm_label("label");
const std::string ta::model::TagTask::sm_initialDate("initial_date");
const std::string ta::model::TagTask::sm_finalDate("final_date");

ta::model::TagTask::TagTask()
  : AbstractItem(),
  m_taskId(-1),
  m_projectId(-1),
  m_initialDate(0),
  m_finalDate(0)
{}

ta::model::TagTask::~TagTask()
{}

ta::model::ItemPropertyVector ta::model::TagTask::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::TagTask::sm_taskId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_taskId);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TagTask::sm_taskId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::TagTask::sm_taskId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::TagTask::sm_projectId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_projectId);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TagTask::sm_projectId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::TagTask::sm_projectId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::TagTask::sm_label, te::dt::STRING_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_label);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TagTask::sm_label);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::TagTask::m_label, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::TagTask::sm_initialDate, te::dt::DateTimeType::TIME_INSTANT);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TagTask::sm_initialDate);
    mep->setProperty(p);
    mep->setData(m_initialDate->clone());

    addProperty(ta::model::TagTask::sm_initialDate, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::TagTask::sm_finalDate, te::dt::DateTimeType::TIME_INSTANT);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TagTask::sm_finalDate);
    mep->setProperty(p);
    mep->setData(m_finalDate->clone());

    addProperty(ta::model::TagTask::sm_finalDate, mep, ipv);
  }


  return ipv;
}

int ta::model::TagTask::getId()
{
  return m_id;
}

int ta::model::TagTask::getTaskId()
{
  return m_taskId;
}

int ta::model::TagTask::getProjectId()
{
  return m_projectId;
}

std::string ta::model::TagTask::getLabel()
{
  return m_label;
}

te::dt::Date* ta::model::TagTask::getInicialDate()
{
  return m_initialDate;
}

te::dt::Date* ta::model::TagTask::getFinalDate()
{
  return m_finalDate;
}

void ta::model::TagTask::setId(const int& newId)
{
  m_id = newId;
}

void ta::model::TagTask::setTaskId(const int& newtaskId)
{
  m_taskId = newtaskId;
}

void ta::model::TagTask::setProjectId(const int& newprojId)
{
  m_projectId = newprojId;
}

void ta::model::TagTask::setLabel(const std::string& newLabel)
{
  m_label = newLabel;
}

void ta::model::TagTask::setInitialDate(te::dt::Date* date)
{
  m_initialDate = date;
}

void ta::model::TagTask::setFinalDate(te::dt::Date* date)
{
  m_finalDate = date;
}