/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/TagTask.h

\brief This file defines the Tag Task model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_TAGTASK_H
#define __TERRAAMAZON_MODEL_ITEM_TAGTASK_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace te
{
  namespace dt
  {
    class Date;
  }
}

namespace ta
{
  namespace model
  {
    /*!
    \class TagTask

    \brief This class defines the tag task model.

    \ingroup model

    \sa AbstractItem
    */
    class TagTask : public AbstractItem
    {
      public:

        static const std::string sm_id;          //!< ...
        static const std::string sm_taskId;      //!< ...
        static const std::string sm_projectId;   //!< ...
        static const std::string sm_label;       //!< ...
        static const std::string sm_initialDate; //!< ...
        static const std::string sm_finalDate;   //!< ...

      public:

        /*! \brief Default constructor  */
        TagTask();

        /*! \brief Default destructor  */
        ~TagTask();

      public:

        ItemPropertyVector toProperties();

      public:

        /** @name Getters
        */
        //@{

        int getId();
        int getTaskId();
        int getProjectId();
        std::string getLabel();
        te::dt::Date* getInicialDate();
        te::dt::Date* getFinalDate();

        //@}

        /** @name Setters
        */
        //@{
        void setId(const int& newId);
        void setTaskId(const int& newtaskId);
        void setProjectId(const int& newprojId);
        void setLabel(const std::string& newLabel);
        void setInitialDate(te::dt::Date*);
        void setFinalDate(te::dt::Date*);

      //@}

      private:

        int                                       m_taskId;
        int                                       m_projectId;
        std::string                               m_label;
        te::dt::Date*                             m_initialDate;
        te::dt::Date*                             m_finalDate;

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_TAGTASK_H

