/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Task.h

\brief This file defines the Task model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "Task.h"

// TerraLib
#include <terralib/datatype/DateTimeInstant.h>
#include <terralib/datatype/DateTimeProperty.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::Task::sm_id("id");
const std::string ta::model::Task::sm_sceneId("scene_id");
const std::string ta::model::Task::sm_projectId("project_id");
const std::string ta::model::Task::sm_phaseId("phase_id");
const std::string ta::model::Task::sm_userId("user_id");

ta::model::Task::Task()
  : AbstractItem(),
  m_sceneId(-1),
  m_projectId(-1),
  m_phaseId(-1),
  m_userId(-1)
{}

ta::model::Task::~Task()
{}

ta::model::ItemPropertyVector ta::model::Task::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Task::sm_sceneId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_sceneId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Task::sm_sceneId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Task::sm_sceneId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Task::sm_projectId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_projectId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Task::sm_projectId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Task::sm_projectId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Task::sm_phaseId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_phaseId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Task::sm_phaseId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Task::sm_phaseId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Task::sm_userId, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_userId);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Task::sm_userId);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Task::sm_userId, mep, ipv);
  }

  return ipv;
}

int ta::model::Task::getId()
{
  return m_id;
}

int ta::model::Task::getSceneId()
{
  return m_sceneId;
}

int ta::model::Task::getProjectId()
{
  return m_projectId;
}

int ta::model::Task::getPhaseId()
{
  return m_phaseId;
}

int ta::model::Task::getUserId()
{
  return m_userId;
}

void ta::model::Task::setId(const int& newId)
{
  m_id = newId;
}

void ta::model::Task::setSceneId(const int& newsceneId)
{
  m_sceneId = newsceneId;
}

void ta::model::Task::setProjectId(const int& newprojId)
{
  m_projectId = newprojId;
}

void ta::model::Task::setPhaseId(const int& newphaseId)
{
  m_phaseId = newphaseId;
}

void ta::model::Task::setUserId(const int& newuserId)
{
  m_userId = newuserId;
}