/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Task.h

\brief This file defines the Task model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_TASK_H
#define __TERRAAMAZON_MODEL_ITEM_TASK_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace te
{
  namespace dt
  {
    class Date;
  }
}

namespace ta
{
  namespace model
  {
    /*!
    \class Task

    \brief This class defines the task model.

    \ingroup model

    \sa AbstractItem
    */
    class Task : public AbstractItem
    {
      public:

        static const std::string sm_id;           //!< ...
        static const std::string sm_sceneId;      //!< ...
        static const std::string sm_projectId;    //!< ...
        static const std::string sm_phaseId;      //!< ...
        static const std::string sm_userId;       //!< ...

      public:

        /*! \brief Default constructor  */
        Task();

        /*! \brief Default destructor  */
        ~Task();

      public:

        ItemPropertyVector toProperties();

      public:

        /** @name Getters
        */
        //@{

        int getId();
        int getSceneId();
        int getProjectId();
        int getPhaseId();
        int getUserId();

        //@}

        /** @name Setters
        */
        //@{

        void setId(const int& newId);
        void setSceneId(const int& newsceneId);
        void setProjectId(const int& newprojId);
        void setPhaseId(const int& newphaseId);
        void setUserId(const int& newuserId);

      //@}

      private:

        int                       m_sceneId;
        int                       m_projectId;
        int                       m_phaseId;
        int                       m_userId;

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_TASK_H

