/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/TaskLog.h

\brief This file defines the Task Log model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "TaskLog.h"

// TerraLib
#include <terralib/datatype/DateTimeInstant.h>
#include <terralib/datatype/DateTimeProperty.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::TaskLog::sm_id("id");
const std::string ta::model::TaskLog::sm_taskId("task_id");
const std::string ta::model::TaskLog::sm_timer("timer");
const std::string ta::model::TaskLog::sm_obs("obs");
const std::string ta::model::TaskLog::sm_status("status");
const std::string ta::model::TaskLog::sm_initialTime("initial_time");
const std::string ta::model::TaskLog::sm_finalTime("final_time");

ta::model::TaskLog::TaskLog()
  : AbstractItem(),
  m_taskId(-1),
  m_timer(0),
  m_obs(""),
  m_status(""),
  m_initialTime(0),
  m_finalTime(0)
{}

ta::model::TaskLog::~TaskLog()
{}

ta::model::ItemPropertyVector ta::model::TaskLog::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    if (m_id != -1)
    {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::TaskLog::sm_id, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_id);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::TaskLog::sm_id);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::TaskLog::sm_id, mep, ipv);
    }
  }

  {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::TaskLog::sm_taskId, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_taskId);
      ta::model::ItemProperties* mep = new ItemProperties(ta::model::TaskLog::sm_taskId);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::TaskLog::sm_taskId, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::TaskLog::sm_timer, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_timer);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TaskLog::sm_timer);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::TaskLog::sm_timer, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::TaskLog::sm_obs, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_obs);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TaskLog::sm_obs);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::TaskLog::sm_obs, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::TaskLog::sm_status, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_status);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TaskLog::sm_status);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::TaskLog::sm_status, mep, ipv);
  }
  if (m_initialTime != 0)
  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::TaskLog::sm_initialTime, te::dt::DateTimeType::TIME_INSTANT);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TaskLog::sm_initialTime);
    mep->setProperty(p);
    mep->setData(m_initialTime->clone());

    addProperty(ta::model::TaskLog::sm_initialTime, mep, ipv);
  }
  if (m_finalTime != 0)
  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::TaskLog::sm_finalTime, te::dt::DateTimeType::TIME_INSTANT);
    ta::model::ItemProperties* mep = new ItemProperties(ta::model::TaskLog::sm_finalTime);
    mep->setProperty(p);
    mep->setData(m_finalTime->clone());

    addProperty(ta::model::TaskLog::sm_finalTime, mep, ipv);
  }

  return ipv;
}

int ta::model::TaskLog::getId()
{
  return m_id;
}

int ta::model::TaskLog::getTaskId()
{
  return m_taskId;
}

int ta::model::TaskLog::getTimer()
{
  return m_timer;
}

std::string ta::model::TaskLog::getObs()
{
  return m_obs;
}

std::string ta::model::TaskLog::getStatus()
{
  return m_status;
}

te::dt::DateTime* ta::model::TaskLog::getInicialTime()
{
  return m_initialTime;
}

te::dt::DateTime* ta::model::TaskLog::getFinalTime()
{
  return m_finalTime;
}

void ta::model::TaskLog::setId(const int& newId)
{
  m_id = newId;
}

void ta::model::TaskLog::setTaskId(const int& newtaskId)
{
  m_taskId = newtaskId;
}

void ta::model::TaskLog::setTimer(const int& newTimer)
{
  m_timer = newTimer;
}

void ta::model::TaskLog::setObs(const std::string& newObs)
{
  m_obs = newObs;
}

void ta::model::TaskLog::setStatus(const std::string& newStatus)
{
  m_status = newStatus;
}

void ta::model::TaskLog::setInitialTime(te::dt::DateTime* newInitialTime)
{
  m_initialTime = newInitialTime;
}

void ta::model::TaskLog::setFinalTime(te::dt::DateTime* newFinalTime)
{
  m_finalTime = newFinalTime;
}