/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/TaskLog.h

\brief This file defines the Task Log model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_TASKLOG_H
#define __TERRAAMAZON_MODEL_ITEM_TASKLOG_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace te
{
  namespace dt
  {
    class Date;
    class DateTime;
  }
}

namespace ta
{
  namespace model
  {
    /*!
    \class TaskLog

    \brief This class defines the task log model.

    \ingroup model

    \sa AbstractItem
    */
    class TaskLog : public AbstractItem
    {
      public:

        static const std::string sm_id;          //!< ...
        static const std::string sm_taskId;      //!< ...
        static const std::string sm_timer;       //!< ...
        static const std::string sm_obs;         //!< ...
        static const std::string sm_status;      //!< ...
        static const std::string sm_initialTime; //!< ...
        static const std::string sm_finalTime;   //!< ...

      public:

        /*! \brief Default constructor  */
        TaskLog();

        /*! \brief Default destructor  */
        ~TaskLog();

      public:

        ItemPropertyVector toProperties();

      public:

        /** @name Getters
        */
        //@{

        int getId();
        int getTaskId();
        int getTimer();
        std::string getObs();
        std::string getStatus();
        te::dt::DateTime* getInicialTime();
        te::dt::DateTime* getFinalTime();

        //@}

        /** @name Setters
        */
        //@{

        void setId(const int& newId);
        void setTaskId(const int& newsceneId);
        void setTimer(const int& newTimer);
        void setObs(const std::string& newObs);
        void setStatus(const std::string& newStatus);
        void setInitialTime(te::dt::DateTime* newInitialTime);
        void setFinalTime(te::dt::DateTime* newFinalTime);

      //@}

      private:

        int                                       m_taskId;
        int                                       m_timer;
        std::string                               m_obs;
        std::string                               m_status;
        te::dt::DateTime*                         m_initialTime;
        te::dt::DateTime*                         m_finalTime;

    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_TASKLOG_H

