/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */
 
 /*!
\file terraamazon/model/item/User.cpp

\brief This file defines the User model.
*/

//TerraAmazon
#include "ItemProperties.h"
#include "User.h"

//Terralib
#include <terralib/datatype/Date.h>
#include <terralib/datatype/DateTimeProperty.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::User::sm_active("active");
const std::string ta::model::User::sm_id("id");
const std::string ta::model::User::sm_projectId("project_id");
const std::string ta::model::User::sm_name("name");
const std::string ta::model::User::sm_password("password");
const std::string ta::model::User::sm_eMail("email");
const std::string ta::model::User::sm_initialDate("initial_date");
const std::string ta::model::User::sm_finalDate("final_date");

ta::model::User::User() 
  : AbstractItem(),
  m_active(false),
  m_projectId(-1),
  m_name(""),
  m_password(""),
  m_eMail("")
{
}

ta::model::User::~User()
{
  m_groupIds.clear();
}

ta::model::ItemPropertyVector ta::model::User::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    if (m_id != -1)
    {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::User::sm_id, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_id);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::User::sm_id);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::User::sm_id, mep, ipv);
    }
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::User::sm_active, te::dt::BOOLEAN_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>(m_active);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::User::sm_active);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::User::sm_active, mep, ipv);
  }

  {
    if (m_projectId != -1)
    {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::User::sm_projectId, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>(m_projectId);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::User::sm_projectId);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::User::sm_projectId, mep, ipv);
    }
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::User::sm_eMail, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_eMail);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::User::sm_eMail);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::User::sm_eMail, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::User::sm_name, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_name);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::User::sm_name);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::User::sm_name, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::User::sm_password, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_password);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::User::sm_password);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::User::sm_password, mep, ipv);
  }

  {
    if (m_initialDate.get())
    {
      te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::User::sm_initialDate, te::dt::DateTimeType::TIME_INSTANT);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::User::sm_initialDate);
      mep->setProperty(p);
      mep->setData(m_initialDate->clone());

      addProperty(ta::model::User::sm_initialDate, mep, ipv);
    }
  }

  {
    if (m_finalDate.get())
    {
      te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::User::sm_finalDate, te::dt::DateTimeType::TIME_INSTANT);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::User::sm_finalDate);
      mep->setProperty(p);
      mep->setData(m_finalDate->clone());

      addProperty(ta::model::User::sm_finalDate, mep, ipv);
    }
  }

  return ipv;
}

bool ta::model::User::isActive()
{
  return m_active;
}

int ta::model::User::getProjectId()
{
  return m_projectId;
}

std::string ta::model::User::getEMail()
{
  return m_eMail;
}

std::string ta::model::User::getName()
{
  return m_name;
}

std::string ta::model::User::getPassword()
{
  return m_password;
}

te::dt::Date* ta::model::User::getInitialDate()
{
  return m_initialDate.get();
}

te::dt::Date* ta::model::User::getFinalDate()
{
  return m_finalDate.get();
}

void ta::model::User::setIsActive(bool isActive)
{
  m_active = isActive;
}

void ta::model::User::setProjectId(int newprojId)
{
  m_projectId = newprojId;
}

void ta::model::User::setEMail(std::string newEMail)
{
  m_eMail = newEMail;
}

void ta::model::User::setName(std::string newName)
{
  m_name =  newName;
}

void ta::model::User::setPassword(std::string newPass)
{
  m_password = newPass;
}

void ta::model::User::setInitialDate(std::unique_ptr<te::dt::Date> newInitialDate)
{
  m_initialDate = std::move(newInitialDate);
}

void ta::model::User::setInitialDate(te::dt::Date* newInitialDate)
{
  m_initialDate.reset(newInitialDate);
}

void ta::model::User::setFinalDate(std::unique_ptr<te::dt::Date> newFinalDate)
{
  m_initialDate = std::move(newFinalDate);
}

void ta::model::User::setFinalDate(te::dt::Date* newFinalDate)
{
  m_initialDate.reset(newFinalDate);
}

void ta::model::User::addGroup(int groupId)
{
  m_groupIds.push_back(groupId);
}

void ta::model::User::removeGroup(int groupId)
{
  size_t size = m_groupIds.size();
  for (size_t i = 0; i < size; ++i)
  {
    if (m_groupIds[i] == groupId)
      m_groupIds.erase(m_groupIds.begin() + i);
  }
}

const std::vector<int>& ta::model::User::getGroupIds()
{
  return m_groupIds;
}

