/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

  /*!
\file terraamazon/model/item/User.h

\brief This file defines the User model.
*/

#ifndef __TERRAAMAZON_MODEL_ITEM_USER_H
#define __TERRAAMAZON_MODEL_ITEM_USER_H

// TerraAmazon
#include "AbstractItem.h"

// STL
#include <string>

namespace te
{
  namespace dt
  {
    class Date;
  }
}

namespace ta
{
  namespace model
  {
    class User : public AbstractItem
    {
    public:
      static const std::string sm_active;       //!< ...
      static const std::string sm_id;           //!< ...
      static const std::string sm_projectId;    //!< ...
      static const std::string sm_name;         //!< ...
      static const std::string sm_password;     //!< ...
      static const std::string sm_eMail;        //!< ...
      static const std::string sm_initialDate;  //!< ...
      static const std::string sm_finalDate;    //!< ...

      User();

      ~User();

      ItemPropertyVector toProperties();

      bool isActive();

      int getProjectId();

      std::string getEMail();
      std::string getName();
      std::string getPassword();

      te::dt::Date* getInitialDate();
      te::dt::Date* getFinalDate();

      void setIsActive(bool isActive);

      void setProjectId(int newprojId);

      void setEMail(std::string newEMail);
      void setName(std::string newName);
      void setPassword(std::string newPass);

      void setInitialDate(std::unique_ptr<te::dt::Date> newInitialDate);
      void setInitialDate(te::dt::Date* newInitialDate);

      void setFinalDate(std::unique_ptr<te::dt::Date> newFinalDate);
      void setFinalDate(te::dt::Date* newFinalDate);

      void addGroup(int groupId);
      void removeGroup(int groupId);
      const std::vector<int>& getGroupIds();

    private:

      bool                                      m_active;
      int                                       m_projectId;
      std::string                               m_name;
      std::string                               m_password;
      std::string                               m_eMail;
      std::unique_ptr<te::dt::Date>             m_initialDate;
      std::unique_ptr<te::dt::Date>             m_finalDate;
      std::vector<int>                          m_groupIds;  //!< The ids of the groups associeted with a user
    };
  } // end namespace model
}   // end namespace ta

#endif // __TERRAAMAZON_MODEL_ITEM_USER_H

