/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file terraamazon/GroupControlDialog.h

\brief A Qt dialog used to configure the application groups.
*/

#ifndef __TERRAAMAZON_MODEL_QT_ACCESSCONTROL_GROUPCONTROLWIDGET_H
#define __TERRAAMAZON_MODEL_QT_ACCESSCONTROL_GROUPCONTROLWIDGET_H

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// Qt
#include <QSignalMapper>
#include <QTableWidget>
#include <QWidget>

// STL
#include <memory>

// Forward declaration
namespace Ui { class GroupControlWidgetForm; }

namespace ta
{
  namespace model
  {
    class GroupFacade;

    namespace qt
    {
      class GroupControlWidget : public QWidget
      {
        Q_OBJECT
      public:

        GroupControlWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~GroupControlWidget();

        void setServer(te::da::DataSourceInfoPtr server);

      protected slots:

        void onAddGroupButtonClicked();

        void onUpdateGroupExecuted(int row);

        void onRemoveGroupExecuted(int row);

      protected:

        void addTableRow(int newRow, QTableWidget* groupTable, std::string groupName = "newGroup", int groupPrivilege = 1, bool fromDb = false);

        void added(int row);

        void loadGroups();

        void removed(int row);

      private:
        te::da::DataSourcePtr                            m_activeServer;  //!< The active server selected by the user.
        std::unique_ptr<ta::model::GroupFacade>          m_facade;        //!< The facade used to handle transactions.
        QSignalMapper*                                   m_removeMapper;  //!< The mapper used to know which group was removed.
        QSignalMapper*                                   m_updateMapper;  //!< The mapper used to know which group was update.
        std::unique_ptr<Ui::GroupControlWidgetForm>      m_ui;            //!< The widget's form.
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif // __TERRAAMAZON_MODEL_QT_ACCESSCONTROL_GROUPCONTROLWIDGET_H