/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file terraamazon/model/UserControlWidget.h

\brief A Qt widget used to configure the application users.
*/

#ifndef __TERRAAMAZON_MODEL_QT_ACCESSCONTROL_USERCONTROLWIDGET_H
#define __TERRAAMAZON_MODEL_QT_ACCESSCONTROL_USERCONTROLWIDGET_H

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

//TerraAmazon
#include "../../item/User.h"

// STL
#include <memory>

// Qt
#include <QSignalMapper>
#include <QTableWidget>
#include <QWidget>

// Forward declaration
namespace Ui { class UserControlWidgetForm; }

namespace ta
{
  namespace model
  {
    class UserFacade;

    namespace qt
    {
      class UserControlWidget : public QWidget
      {
        Q_OBJECT
        public:

          UserControlWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~UserControlWidget();

          void setUser(int userId);

          void setServer(te::da::DataSourceInfoPtr server);

        protected slots:

        void onAddUserButtonClicked();

        void onCellClicked(int row, int column);

        void onUpdateUserExecuted(int row);

        void onSortOccurred(int column);

      protected:

        void addTableRow(int newRow, QTableWidget* userTable, ta::model::User* newUser, bool fromDb = false);

        void added(int row);

        void loadUsers();

        private:
          te::da::DataSourcePtr                       m_activeServer;       //!< The active server selected by the user.
          std::unique_ptr<ta::model::User>            m_activeUser;         //!< The application's current user.
          std::unique_ptr<ta::model::UserFacade>      m_facade;             //!< The facade used to handle transactions.
          QSignalMapper*                              m_updateMapper;       //!< The mapper used to know which user was update.
          std::unique_ptr<Ui::UserControlWidgetForm>  m_ui;                 //!< The widget's form.
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif // __TERRAAMAZON_MODEL_Q;T_ACCESSCONTROL_USERCONTROLWIDGET_H