/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file CatalogManagerDialog.h

  \brief ...
  */

#ifndef __TERRAAMAZON_MODEL_CATALOGMANAGERDIALOG_H
#define __TERRAAMAZON_MODEL_CATALOGMANAGERDIALOG_H

// TerraAmazon
#include "Config.h"
#include "../../item/Catalog.h"
#include "../../item/User.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/RuleFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../facade/UserFacade.h"

// Terralib
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/maptools/DataSetLayer.h>
#include <terralib/qt/widgets/layer/utils/CreateLayerWidget.h>

// STL
#include <string>

// Qt
#include <QDialog>
#include <QSignalMapper>

// Forward declaration
namespace Ui { class CatalogManagerDialogForm; }

namespace te
{
  namespace se { class Symbolizer; }

  namespace qt
  {
    namespace widgets
    {
      // Forward declarations
      class PolygonSymbolizerProperty;

      class LineSymbolizerProperty;

      class PointSymbolizerProperty;

      class StyleControllerWidget;

      class SymbolPreviewWidget;
    }
  }
}

namespace ta
{
  namespace model
  {
    namespace qt
    {

      class CatalogManagerDialog : public QDialog
      {
        Q_OBJECT
      public:

        CatalogManagerDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~CatalogManagerDialog();

        void setServer(te::da::DataSourceInfoPtr dsInfo);

        void setUsername(std::string userName);

      public:

        void setLayers(std::list<te::map::AbstractLayerPtr> layers);

      private:

        bool importLayer(te::map::AbstractLayerPtr layer);

        bool associateLayer(te::map::AbstractLayerPtr layer);

        std::string generateStyle(te::map::AbstractLayerPtr layer);

        bool exchangeToDatabase(te::map::AbstractLayerPtr layer);

        std::string validateLayerName(std::string title);

        std::vector<te::dt::Property*> getAllowedAttributesFromLayer(ta::model::Catalog* catalog);

        std::string getActiveUserName();

      protected slots:

        void onAddLayerToolButtonPressed();

        void onUpdateButtonClicked(int row);

        void onRemoveButtonClicked(int row);

        void onCatalogStyleComboBoxActivated(int idx);

        void onCreateLayerPushButtonClicked();

        void onCurrentCellTableWidgetChanged(int currentRow, int currentColumn, int previousRow, int previousColumn);

        void onApplyStylePushButtonClicked();

        void onSymbolizerSelected(te::se::Symbolizer* s);

        void onPolygonSymbolizerChanged();

        void onLineSymbolizerChanged();

        void onPointSymbolizerChanged();

        void onMapRefresh();

      protected:

        void updateCatalogTableWidget();

        void resizeCatalogsTableWidget();

        void buildStyleWidgets();

        void setTabStatus(bool status);

      signals:

        void symbolizerChanged(te::se::Symbolizer* s);

      private:

        te::da::DataSourcePtr                                m_activeServer;       //!< The active server selected by the user.
        std::string                                          m_userName;           //!< The user of the active server.
        std::unique_ptr<ta::model::CatalogFacade>            m_catalogFacade;      //!< The facade used to handle transactions.
        std::unique_ptr<ta::model::RuleFacade>               m_ruleFacade;         //!< The facade used to handle transactions.
        std::unique_ptr<ta::model::RuleSetFacade>            m_ruleSetFacade;      //!< The facade used to handle transactions.
        std::unique_ptr<ta::model::UserFacade>                  m_userFacade;      //!< The facade used to handle transactions.
        std::unique_ptr<Ui::CatalogManagerDialogForm>                   m_ui;      //!< The widget's form.

        te::qt::widgets::CreateLayerWidget*                  m_createLayerUi;      //!< The widget's to creates a empty layer.

        te::qt::widgets::PolygonSymbolizerProperty*             m_polyWidget;
        te::qt::widgets::LineSymbolizerProperty*                m_lineWidget;
        te::qt::widgets::PointSymbolizerProperty*              m_pointWidget;
        te::qt::widgets::StyleControllerWidget*            m_styleController;

        te::qt::widgets::SymbolPreviewWidget*                m_symbolPreview;

        QSignalMapper*                                  m_updateButtonMapper;      //!<
        QSignalMapper*                                  m_removeButtonMapper;      //!<

        std::unique_ptr<te::se::Style>                               m_style;

        std::list<te::map::AbstractLayerPtr>                        m_layers;

      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta


#endif  // __TERRAAMAZON_MODEL_CATALOGMANAGERDIALOG_H
