/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file ConceptualModelUtils.h

  \brief ...
  */

#ifndef __TERRAAMAZON_MODEL_CONCEPTUALMODELUTILS_H
#define __TERRAAMAZON_MODEL_CONCEPTUALMODELUTILS_H

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/maptools/AbstractLayer.h>

namespace te
{
  namespace gm
  {
    class Envelope;
  }
}

namespace ta
{
  namespace model
  {
    /*!
    \brief This method create a TerraAmazon conceptual model at target datasource using selected cell layer.

    \ingroup model

    \param targetDs  The target Datasource.
    \param cellLayer The cell layer.
    \param res       The resolution to create the new cell layer.

    */
    void CreateConceptualModel(te::da::DataSourceInfoPtr targetDs, te::map::AbstractLayerPtr cellLayer, const double& res);

    /*!
      \brief This method create a TerraAmazon conceptual model at target datasource using the name of cell layer previously created.

      \ingroup model

      \param targetDs      The target Datasource.
      \param cellLayerName The cell layer name.
      \param res           The resolution to create the new cell layer.
      */
    void CreateConceptualModel(te::da::DataSourceInfoPtr targetDs, const std::string& cellLayerName, const double& res);

    /*!
      \brief This method create a TerraAmazon conceptual model at target datasource using informations to create the cell layer.

      \ingroup model

      \param targetDs         The target Datasource.
      \param newCellLayerName Name of the cell layer that will be created.
      \param env              The envelope in which will be based the new cell layer.
      \param res              The resolution to create the new cell layer.
      \param srid             The srid to create the new cell layer.

      */
    void CreateConceptualModel(te::da::DataSourceInfoPtr targetDs, const std::string& newCellLayerName, const te::gm::Envelope& env, const double& res, const int& srid);

    void validateDataForConceptualModel(te::da::DataSourceInfoPtr targetDs, const std::string& newCellLayerName);
  } // end namespace model
}   // end namespace ta


#endif  // __TERRAAMAZON_MODEL_CONCEPTUALMODELUTILS_H
