/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/AOIControlProjectWizardPage.h

  \brief A Qt wizard page used to configure the project from AOI Control.
*/

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_AOICONTROLPROJECTWIZARDPAGE_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_AOICONTROLPROJECTWIZARDPAGE_H

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// STL
#include <memory>

// Qt
#include <QSignalMapper>
#include <QTableWidget>
#include <QWizardPage>

// Forward declaration
namespace Ui { class AOIControlProjectWizardPageForm; }

namespace ta
{
  namespace model
  {
    class ProjectFacade;
    class ProjectSceneFacade;
    class Scene;
    class SceneFacade;

    namespace qt
    {
      class AOIControlProjectWizardPage : public QWizardPage
      {
        Q_OBJECT

      public:

        AOIControlProjectWizardPage(QWidget* parent = 0);

        virtual ~AOIControlProjectWizardPage();

        bool isComplete() const;

        void setServer(te::da::DataSourceInfoPtr server);

      protected slots:

        void onProjectComboBoxIndexChanged(int index);

        void onAddScenarioButtonClicked();

        void onLockSceneExecuted(int row);

        void onSceneFilterButttonClicked();

        void onRemScenarioButtonClicked(int row);

      protected:

        void addScene(ta::model::Scene* scene, QTableWidget* sceneTable);

        std::unique_ptr<te::da::DataSet> filter(te::da::DataSetTypePtr dataType);

        void loadData();

        void removed(int row);
        
      private:

        te::da::DataSourcePtr                               m_activeServer;     //!< The active server selected by the user.
        std::unique_ptr<ta::model::ProjectFacade>           m_projectFacade;    //!< The facade used to handle project transactions.
        std::unique_ptr<ta::model::ProjectSceneFacade>      m_projSceneFacade;  //!< The facade used to handle project & scene transactions.
        std::unique_ptr<ta::model::SceneFacade>             m_sceneFacade;      //!< The facade used to handle scene transactions.
        QSignalMapper*                                      m_lockMapper;       //!< The mapper used to know which user was locked.
        QSignalMapper*                                      m_removeMapper;     //!< The mapper used to know which scene was removed.
        std::unique_ptr<Ui::AOIControlProjectWizardPageForm>  m_ui;
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_AOICONTROLPROJECTWIZARDPAGE_H

