/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/AOIControlScenarioWizardPage.h

  \brief A Qt wizard page used to configure the scenario from AOI Control.
*/

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_AOICONTROLSCENARIOWIZARDPAGE_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_AOICONTROLSCENARIOWIZARDPAGE_H

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// Qt
#include <QWizardPage>

// STL
#include <memory>

// Forward declarations
namespace Ui { class AOIControlScenarioWizardPageForm; }


namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class DataSetTableModel;
    }
  }
}

namespace ta
{
  namespace model
  {
    class AoiLayerFacade;
    class Catalog;
    class CatalogFacade;
    class Scene;
    class SceneFacade;

    namespace qt
    {
      class AOIControlScenarioWizardPage : public QWizardPage
      {
        Q_OBJECT

      public:

        AOIControlScenarioWizardPage(QWidget* parent = 0);

        virtual ~AOIControlScenarioWizardPage();

        bool isComplete() const;

        te::da::DataSourceInfoPtr getServer();

        void setServer(te::da::DataSourceInfoPtr server);

      protected slots:

      void onAOIComboBoxIndexChanged(int index);

      void onAddSceneButtonClicked();

      void onAOIFilterButttonClicked();

      void onSceneFilterButttonClicked();

      protected:

        void addScene(ta::model::Scene* scene);

        void loadData();

        std::unique_ptr<te::da::DataSet> filter(te::da::DataSetTypePtr dataType);

      private:

        te::da::DataSourcePtr                                  m_activeServer;    //!< The active server selected by the user.
        std::unique_ptr<ta::model::AoiLayerFacade>             m_aoiLayerFacade;  //!< The facade used to handle catalog transactions.
        std::unique_ptr<ta::model::CatalogFacade>              m_catalogFacade;   //!< The facade used to handle catalog transactions.
        std::unique_ptr<ta::model::SceneFacade>                m_sceneFacade;     //!< The facade used to handle scene transactions.
        te::qt::widgets::DataSetTableModel*                    m_tableModel;      //!< The model used to preview the data.
        std::unique_ptr<Ui::AOIControlScenarioWizardPageForm>  m_ui;
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_AOICONTROLSCENARIOWIZARDPAGE_H

