/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/AOIManagementWizard.cpp

  \brief A Qt wizard page used to management the AOI.
*/

// TerraLib
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include "AOIManagementWizard.h"
#include "AOIControlProjectWizardPage.h"
#include "AOIControlScenarioWizardPage.h"

// STL
#include <cassert>

// Qt
#include <QMessageBox>

ta::model::qt::AOIManagementWizard::AOIManagementWizard(QWidget* parent)
  : QWizard(parent)
{
  //configure the wizard
  this->setWizardStyle(QWizard::ModernStyle);
  this->setWindowTitle(tr("AOI Management Wizard"));

  addPages();
}

ta::model::qt::AOIManagementWizard::~AOIManagementWizard()
{

}

bool ta::model::qt::AOIManagementWizard::validateCurrentPage()
{
  if (currentId() == PAGE_SCENE_CONTROL)
  {
    bool res = m_sceneControlWizardPage->isComplete();

    if (res)
      m_projControlWizardPage->setServer(m_sceneControlWizardPage->getServer());

    return m_sceneControlWizardPage->isComplete();
  }
  else if (currentId() == PAGE_PROJECT_CONTROL)
  {
    return m_projControlWizardPage->isComplete();
  }

  return false;
}

void ta::model::qt::AOIManagementWizard::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
    m_sceneControlWizardPage->setServer(server);
}

void ta::model::qt::AOIManagementWizard::addPages()
{
  m_sceneControlWizardPage = new AOIControlScenarioWizardPage(this);
  m_projControlWizardPage = new AOIControlProjectWizardPage(this);

  setPage(PAGE_SCENE_CONTROL, m_sceneControlWizardPage);
  setPage(PAGE_PROJECT_CONTROL, m_projControlWizardPage);
}

