/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/AttributeControlDialog.cpp

\brief A Qt dialog used to configure the attribute settings.
*/

// TerraAmazon
#include "AttributeControlWidget.h"
#include "AttributeControlDialog.h"
#include "ui_AttributeControlDialogForm.h"

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>

#include "../../../common/Utils.h"

// Qt
#include <QGridLayout>

ta::model::qt::AttributeControlDialog::AttributeControlDialog(QWidget* parent, Qt::WindowFlags f)
: QDialog(parent, f),
m_ui(new Ui::AttributeControlDialogForm)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  QGridLayout* layout = new QGridLayout(m_ui->m_widget);
  layout->setContentsMargins(0, 0, 0, 0);

  m_widget = new AttributeControlWidget(m_ui->m_widget);

  layout->addWidget(m_widget);
  
}

ta::model::qt::AttributeControlDialog::~AttributeControlDialog()
{
}

void ta::model::qt::AttributeControlDialog::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    getWidget()->setServer(server);
  }
}

ta::model::qt::AttributeControlWidget* ta::model::qt::AttributeControlDialog::getWidget()
{
  return m_widget;
}
