/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file terraamazon/AttributeControlWidget.h

  \brief A Qt widget used to configure the attribute control.
  */

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_ATTRIBUTECONTROLWIDGET_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_ATTRIBUTECONTROLWIDGET_H

// Terraamazon
#include "../../item/Catalog.h"
#include "../../item/Project.h"
#include "../../item/Rule.h"
#include "../../item/RuleSet.h"
#include "../../item/RulesLayerColumn.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/ProjectFacade.h"
#include "../../facade/RuleFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../facade/RulesLayerColumnFacade.h"

// Terralib
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/qt/widgets/property/NewPropertyWidget.h>

// STL
#include <memory>

// Qt
#include <QWidget>
#include <QSignalMapper>

// Forward declaration
namespace Ui { class AttributeControlWidgetForm; }

namespace ta
{
  namespace model
  {
    namespace qt
    {
      class AttributeControlWidget : public QWidget
      {
        Q_OBJECT
      public:

        AttributeControlWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~AttributeControlWidget();

        void setServer(te::da::DataSourceInfoPtr dsInfo);

        protected slots:

        void onAddButtonClicked(int row);

        void onCreateAttributeToolButtonPressed();

        void onCancelButtonClicked(int row);

        void onConfirmButtonClicked(int row);

        void onInputLayerComboBoxActivated(int row);

        void onProjectComboBoxActivated(int index);

        void triggerRefreshTableWidget(int idx);

        void onRemoveButtonClicked(int row);

      private:

        bool addNewAttributeToTableWidget(te::dt::Property* prop);

        void fillTableWidgetOptions(const int &row, const int &ruleSetId, bool newField = false);

        std::vector<te::dt::Property*> getAllowedAttributesFromLayer(ta::model::Catalog* catalog);

        std::vector<te::dt::Property*> getAllowedAttributesFromOutputLayer(ta::model::Catalog* catalog, const std::vector<std::string>& class_columns);

        void resizeTableWidget();

        void updateAttributeTableWidget(const int &ruleSetId);

        void initComboBoxOutputLayers(const int &projectId);

        bool isValidNewPropertyName(std::string propName, ta::model::Catalog* catalog, const std::vector<std::string>& class_columns);

        void listProjects();

        void validateInputLayerAttribute(ta::model::RulesLayerColumn* rlc, const int& row, const std::string& columnOut, const int& ruleId, const int& rulesetId);

      private:
        te::da::DataSourcePtr                                m_activeServer;                   //!< The active server selected by the user.
        std::unique_ptr<ta::model::CatalogFacade>            m_catalogFacade;                  //!< The facade used to handle transactions.
        std::unique_ptr<ta::model::ProjectFacade>            m_projectFacade;                  //!< The facade used to handle transactions.
        std::unique_ptr<ta::model::RuleFacade>               m_ruleFacade;                     //!< The facade used to handle transactions.
        std::unique_ptr<ta::model::RuleSetFacade>            m_ruleSetFacade;                  //!< The facade used to handle transactions.
        std::unique_ptr<ta::model::RulesLayerColumnFacade>   m_rulesLayerColumnFacade;         //!< The facade used to handle transactions.
        std::auto_ptr<Ui::AttributeControlWidgetForm>        m_ui;                             //!< The widget's form.
        te::qt::widgets::NewPropertyWidget*                  m_propWidget;                     //!<
        QSignalMapper*                                       m_inputLayerComboBoxMapper;       //!<
        QSignalMapper*                                       m_removeButtonMapper;             //!<
        QSignalMapper*                                       m_addButtonMapper;                //!<
        QSignalMapper*                                       m_confirmButtonMapper;            //!<
        QSignalMapper*                                       m_cancelButtonMapper;             //!<
        te::dt::Property*                                    m_currentPropertyNew;             //!<
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_ATTRIBUTECONTROLWIDGET_H

