/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ClassControlWidget.h

  \brief A Qt widget used to configure the class control.
*/

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_CLASSCONTROLWIDGET_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_CLASSCONTROLWIDGET_H

// Terralib
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/qt/widgets/utils/ColorPickerToolButton.h>

// STL
#include <memory>

// Qt
#include <QSignalMapper>
#include <QWidget>

// Forward declaration
namespace Ui { class ClassControlWidgetForm; }

namespace ta
{
  namespace model
  {
    namespace qt
    {
      class ClassControlWidget : public QWidget
      {
        Q_OBJECT

        public:

          ClassControlWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~ClassControlWidget();

        public:

          void setActiveServer(te::da::DataSourceInfoPtr dsInfo);

        public slots:

          void onProjectComboBoxActivated(int index);

          void onLayersComboBoxActivated(int index);

          void onClassAddToolButtonPressed();

          void onClassColumnAddToolButtonPressed();

          void onClassNameComboBoxEditActivated(int index);

          void onRemoveClassExecuted(int row);

          void onAddClassToolButtonPressed();

          void onRemoveClassToolButtonPressed();

          void onAddAllClassToolButtonPressed();

          void onRemoveAllClassToolButtonPressed();

          void setActionsState();

       protected:

        void listProjects();

        void listNotAssociatedClasses(const int& catalogId);

        void listAssociatedClasses(const int& catalogId);

        void fillNames();

        private:

          std::auto_ptr<Ui::ClassControlWidgetForm> m_ui;

          te::da::DataSourcePtr m_server;

          te::qt::widgets::ColorPickerToolButton* m_colorPicker;

          QSignalMapper* m_removeMapper;                   //!< The mapper used to know which action was executed.
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_CLASSCONTROLWIDGET_H

