/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/EndProcessDialog.h

  \brief A Qt dialog used to configure the end process.
*/

#ifndef __TERRAAMAZON_MODEL_QT_TASK_ENDPROCESSDIALOG_H
#define __TERRAAMAZON_MODEL_QT_TASK_ENDPROCESSDIALOG_H

// TerraAmazon
#include "Config.h"
#include "../../facade/CellFacade.h"
#include "../../facade/ProcessFacade.h"
#include "../../facade/TaskLogFacade.h"
#include "../../facade/UserFacade.h"
#include "../../item/Cell.h"
#include "../../item/Process.h"
#include "../../item/Task.h"
#include "../../item/TaskLog.h"
#include "../../item/User.h"

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// STL
#include <memory>

// Qt
#include <QDialog>

// Forward declaration
namespace Ui { class EndProcessDialogForm; }

namespace ta
{
  namespace model
  {

    namespace qt
    {

      class EndProcessDialog : public QDialog
      {
        Q_OBJECT
        public:

          EndProcessDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~EndProcessDialog();

          void setEndProcess(te::da::DataSourceInfoPtr ds, int processId); //Process* process);

        protected slots:

          void onCloseButtonClicked();

          void onOpenRadioButtonClicked();

          void onCloseRadioButtonClicked();

          void onReviewRadioButtonClicked();

          void onAllUsersCheckBoxClicked();

        private:

          std::auto_ptr<Ui::EndProcessDialogForm> m_ui;

          te::da::DataSourcePtr m_server;

          std::unique_ptr<ta::model::Process> m_process;

      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_TASK_ENDPROCESSDIALOG_H

