/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/PhaseControlDialog.h

\brief A Qt dialog used to configure the phase control.
*/

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_PHASECONTROLDIALOG_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_PHASECONTROLDIALOG_H

// TerraAmazon
#include "Config.h"

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// STL
#include <memory>

// Qt
#include <QDialog>

// Forward declaration
namespace Ui { class PhaseControlDialogForm; }

namespace ta
{
  namespace model
  {

    namespace qt
    {

      class PhaseControlDialog : public QDialog
      {
        Q_OBJECT
      public:

        PhaseControlDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~PhaseControlDialog();

        void setServer(te::da::DataSourceInfoPtr dsInfo);

        protected slots:

        void onAddPhaseButtonClicked();

        void onAddAssociationToolButtonClicked();

        void onRemoveAssociationToolButtonClicked();

        void onFinishPushButtonClicked();

      private:

        void updateTWGroupPhase(const std::string& phaseName);

        int getRowByWidgetSearch();

        void fillNames();

      private:

        te::da::DataSourcePtr                           m_activeServer;  //!< The active server selected by the user.
        std::unique_ptr<Ui::PhaseControlDialogForm>      m_ui;           //!< The widget's form.

      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_PHASECONTROLDIALOG_H
