/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file terraamazon/ProcessCellManagerDialog.h

  \brief A Qt dialog used to configure the process cell manager.
  */

#ifndef __TERRAAMAZON_MODEL_QT_PROCESSCELLMANAGERDIALOG_H
#define __TERRAAMAZON_MODEL_QT_PROCESSCELLMANAGERDIALOG_H

// Terraamazon
#include "../../item/Catalog.h"
#include "../../facade/CatalogFacade.h"
#include "../../../common/qt/CreateCellLayerWidget.h"
#include "LockedCellsWidget.h"

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>


// STL
#include <memory>

// Qt
#include <QDialog>
#include <QTableView>
#include <QSignalMapper>

// Forward declaration
namespace Ui { class ProcessCellManagerDialogForm; }

namespace ta
{
  namespace model
  {

    namespace qt
    {

      class ProcessCellManagerDialog : public QDialog
      {
        Q_OBJECT

      public:

        ProcessCellManagerDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~ProcessCellManagerDialog();

        void setServer(te::da::DataSourceInfoPtr dsInfo);

        ta::common::qt::CreateCellLayerWidget* getCreateCellLayerWidget();

        ta::model::qt::LockedCellsWidget* getLockedCellsWidget();

        protected slots:

        void onOkCreateCellLayerPushButtonClicked();

        void onRadioButtonChecked(const int& row);

        void onUpdatePushButtonClicked();

      protected:

        void listCellLayers();

      private:
        std::auto_ptr<Ui::ProcessCellManagerDialogForm> m_ui;                      //!<--
        int                                             m_rowActiveCellLayer;      //!<--
        te::da::DataSourcePtr                           m_activeServer;            //!<--
        std::unique_ptr<ta::model::CatalogFacade>       m_catalogFacade;           //!<--
        QSignalMapper*                                  m_radioButtonMapper;       //!<
        ta::common::qt::CreateCellLayerWidget*          m_createCellLayerWidget;   //!<--
        ta::model::qt::LockedCellsWidget*               m_lockedCellsWidget;       //!<--

      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROCESSCELLMANAGERDIALOG_H

