/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ProcessControlDialog.h

  \brief A Qt dialog used to configure the process control.
*/

#ifndef __TERRAAMAZON_MODEL_QT_TASK_PROCESSCONTROLDIALOG_H
#define __TERRAAMAZON_MODEL_QT_TASK_PROCESSCONTROLDIALOG_H

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>

// TerraAmazon
#include "../../Globals.h"
#include "../../facade/AoiLayerFacade.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/ProcessFacade.h"
#include "../../facade/SceneFacade.h"
#include "../../facade/TaskLogFacade.h"
#include "../../facade/UserFacade.h"
#include "../../item/AoiLayer.h"
#include "../../item/Catalog.h"
#include "../../item/Phase.h"
#include "../../item/Process.h"
#include "../../item/Scene.h"
#include "../../item/TaskLog.h"
#include "../../item/User.h"

// STL
#include <memory>

// Qt
#include <QDialog>
#include <QTableView>

// Forward declaration
namespace Ui { class ProcessControlDialogForm; }

namespace ta
{
  namespace model
  {

    namespace qt
    {

      class ProcessControlDialog : public QDialog
      {
        Q_OBJECT

        public:

          ProcessControlDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~ProcessControlDialog();

          void setProcess(te::da::DataSourceInfoPtr dsInfo, int userId, int processId);

          int getProcess();

        protected slots:

          void onAddNewTaskButtonClicked();

          void onEndProcessButtonClicked();

          void onCloseButtonClicked();

          void onSceneTableClicked(int row, int col);

          void onStartProcessButtonClicked();

          void onSelectionChanged(const QItemSelection &selected, const QItemSelection &deselected);

          void redrawSelection_slot();

        private:

          std::auto_ptr<Ui::ProcessControlDialogForm> m_ui;

          te::da::DataSourcePtr m_server;

          std::unique_ptr<ta::model::User> m_user;

          std::unique_ptr<ta::model::Process> m_process;

          te::da::DataSourceInfoPtr m_dsInfo;

          int m_selectedRow;

          bool m_manualSelect;

          void fillSceneTable();

          void redrawSelection();

          void updateSelectedRow();

          QIcon getStatusIconPixmap(std::string status);

      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_TASK_PROCESSCONTROLDIALOG_H

