/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ProjectActivationDialog.cpp

  \brief A Qt dialog used to configure the project settings.
*/

// TerraAmazon
#include "../../item/Project.h"
#include "ProjectActivationWidget.h"
#include "ProjectActivationDialog.h"
#include "ui_ProjectActivationDialogForm.h"

#include "../../../common/Utils.h"

// Qt
#include <QGridLayout>

ta::model::qt::ProjectActivationDialog::ProjectActivationDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::ProjectActivationDialogForm)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  QGridLayout* layout = new QGridLayout(m_ui->m_widget);
  layout->setContentsMargins(0, 0, 0, 0);

  m_widget = new ProjectActivationWidget(m_ui->m_widget);

  layout->addWidget(m_widget);

}

ta::model::qt::ProjectActivationDialog::~ProjectActivationDialog()
{
}

void ta::model::qt::ProjectActivationDialog::setServer(te::da::DataSourceInfoPtr dsInfo)
{
  m_widget->setServer(dsInfo);
}

int ta::model::qt::ProjectActivationDialog::getActiveProjectId()
{
  return m_widget->getActiveProjectId();
}

void ta::model::qt::ProjectActivationDialog::setActiveProjectId(int projId)
{
  m_widget->setActiveProjectId(projId);
}

ta::model::qt::ProjectActivationWidget* ta::model::qt::ProjectActivationDialog::getWidget()
{
  return m_widget;
}
