/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ProjectActivationWidget.h

  \brief A Qt widget used to configure the project settings.
*/

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTACTIVATIONWIDGET_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTACTIVATIONWIDGET_H

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// STL
#include <memory>

// Qt
#include <QWidget>

// Forward declaration
namespace Ui { class ProjectActivationWidgetForm; }

namespace ta
{
  namespace model
  {
    namespace qt
    {
      class ProjectActivationWidget : public QWidget
      {
        Q_OBJECT
      public:

        ProjectActivationWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~ProjectActivationWidget();

        void setServer(te::da::DataSourceInfoPtr dsInfo);

        int getActiveProjectId();

        void setActiveProjectId(int projId);

      protected slots:

        void onActivateTriggered();

        void onDeactivateTriggered();

      protected:
        
        void listProjects();

      private:

        std::auto_ptr<Ui::ProjectActivationWidgetForm> m_ui;

        te::da::DataSourcePtr m_server;
        std::unique_ptr<ta::model::Project> m_activeProject;
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTACTIVATIONWIDGET_H
