/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file terraamazon/ProjectControlWidget.h

  \brief A Qt widget used to configure the project settings.
  */

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTCONTROLWIDGET_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTCONTROLWIDGET_H

// TerraAmazon

#include "../../facade/CatalogFacade.h"
#include "../../facade/ProjectFacade.h"
#include "../../facade/RuleFacade.h"
#include "../../facade/RuleRuleSetFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../facade/RulesLayerColumnFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../facade/ProjectFacade.h"
#include "../../facade/ClassFacade.h"
#include "../../item_mem/CatalogMem.h"

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// STL
#include <memory>

// Qt
#include <QSignalMapper>
#include <QWidget>

// Boost
#include <boost/property_tree/ptree.hpp>

// Forward declaration
namespace Ui { class ProjectControlWidgetForm; }

namespace ta
{
  namespace model
  {
    namespace qt
    {
      class ProjectControlWidget : public QWidget
      {
        Q_OBJECT
      public:

        ProjectControlWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~ProjectControlWidget();

        protected slots:

        void onAddToolButtonPressed();

        void onUpdateToolButtonPressed(int row);

        void onRemoveToolButtonPressed(int row);

        void onExportToolButtonPressed(int row);

        void onImportToolButtonPressed();

        void onTableCellClicked(int row, int column);

      private:

        ta::model::CatalogMem* importCatalogAux(const boost::property_tree::ptree& catalog);

        boost::property_tree::ptree exportCatalogAux(const int& catalogId);

      public:

        void setActiveServer(te::da::DataSourceInfoPtr dsInfo);

        te::da::DataSourcePtr getServer();

      protected:

        void listProjects();

      private:
        std::auto_ptr<Ui::ProjectControlWidgetForm> m_ui;

        te::da::DataSourcePtr m_server;

        QSignalMapper*                                       m_updateButtonMapper;            //!<
        QSignalMapper*                                       m_removeButtonMapper;            //!<
        QSignalMapper*                                       m_exportButtonMapper;            //!<

        std::unique_ptr<ta::model::CatalogFacade>            m_catalogFacade;
        std::unique_ptr<ta::model::RuleFacade>               m_ruleFacade;
        std::unique_ptr<ta::model::RuleRuleSetFacade>        m_ruleRuleSetFacade;
        std::unique_ptr<ta::model::RulesLayerColumnFacade>   m_rulesLayerColumnFacade;
        std::unique_ptr<ta::model::RuleSetFacade>            m_rulesetFacade;
        std::unique_ptr<ta::model::ProjectFacade>            m_projectFacade;
        std::unique_ptr<ta::model::ClassFacade>              m_classFacade;
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTCONTROLWIDGET_H
