/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ProjectInformationDialog.h

  \brief A Qt dialog used for project description.
*/

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTINFORMATIONDIALOG_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTINFORMATIONDIALOG_H

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// STL
#include <memory>

// Qt
#include <QDialog>

// Forward declaration
namespace Ui { class ProjectInfoDialogForm; }

namespace ta
{
  namespace model
  {
    class Catalog;

    namespace qt
    {
      class ProjectInformationWidget;

      class ProjectInformationDialog : public QDialog
      {
        Q_OBJECT
      public:

        ProjectInformationDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~ProjectInformationDialog();

      public:

        void setActiveServer(te::da::DataSourceInfoPtr server);

      protected:

        void listProjects();

        std::string getCatalogProjection(ta::model::Catalog* catalog);

      protected slots:

      void onProjectComboBoxActivated(int index);

      private:

        std::auto_ptr<Ui::ProjectInfoDialogForm> m_ui;

        te::da::DataSourcePtr                     m_activeServer;  //!< The active server selected by the user.
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTINFORMATIONDIALOG_H

