/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ProjectManagementWizard.cpp

  \brief A Qt wizard page used to management the project.
*/

// TerraAmazon
#include "ProjectManagementWizard.h"

#include "AttributeControlWidget.h"
#include "AttributeControlWizardPage.h"
#include "ClassControlWidget.h"
#include "ClassControlWizardPage.h"
#include "ProjectControlWidget.h"
#include "ProjectControlWizardPage.h"
#include "RulesControlWidget.h"
#include "RulesControlWizardPage.h"

// TerraLib
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>


// STL
#include <cassert>

// Qt
#include <QMessageBox>

ta::model::qt::ProjectManagementWizard::ProjectManagementWizard(QWidget* parent)
  : QWizard(parent),
  m_user("")
{
  //configure the wizard
  this->setWizardStyle(QWizard::ModernStyle);
  this->setWindowTitle(tr("Project Management Wizard"));
  this->setFixedSize(640, 640);

  addPages();
}

ta::model::qt::ProjectManagementWizard::~ProjectManagementWizard()
{

}

bool ta::model::qt::ProjectManagementWizard::validateCurrentPage()
{
  if (currentPage() == m_projectControlPage)
  {
    te::da::DataSourcePtr server = m_projectControlPage->getWidget()->getServer();

    bool isComplete = m_projectControlPage->isComplete();

    if (server.get() && isComplete)
    {
      te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(server->getId());

      m_rulesControlPage->getWidget()->setActiveServer(dsInfo);
      m_rulesControlPage->getWidget()->setUser(m_user);

      return true;
    }
    return false;
  }
  else if (currentPage() == m_rulesControlPage)
  {
    te::da::DataSourcePtr server = m_projectControlPage->getWidget()->getServer();

    bool isComplete = m_rulesControlPage->isComplete();

    if (server.get() && isComplete)
    {
      te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(server->getId());

      m_attrControlPage->getWidget()->setServer(dsInfo);

      return true;
    }

    return false;
  }
  else if (currentPage() == m_attrControlPage)
  {
    te::da::DataSourcePtr server = m_projectControlPage->getWidget()->getServer();

    bool isComplete = m_attrControlPage->isComplete();

    if (server.get() && isComplete)
    {
      te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(server->getId());

      m_classControlPage->getWidget()->setActiveServer(dsInfo);

      return true;
    }

    return false;
  }
  else if (currentPage() == m_classControlPage)
  {
    return m_rulesControlPage->isComplete();
  }

  return false;
}

void ta::model::qt::ProjectManagementWizard::setActiveServer(te::da::DataSourceInfoPtr dsInfo)
{
  //set servers for project widget interface
  m_projectControlPage->getWidget()->setActiveServer(dsInfo);
}

void ta::model::qt::ProjectManagementWizard::setUser(const std::string& user)
{
  m_user = user;
}

void ta::model::qt::ProjectManagementWizard::addPages()
{
  m_projectControlPage = new ProjectControlWizardPage(this);
  m_rulesControlPage = new RulesControlWizardPage(this);
  m_attrControlPage = new AttributeControlWizardPage(this);
  m_classControlPage = new ClassControlWizardPage(this);

  addPage(m_projectControlPage);
  addPage(m_rulesControlPage);
  addPage(m_attrControlPage);
  addPage(m_classControlPage);
}

