/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ProjectManagementWizard.h

  \brief A Qt wizard page used to management the project.
*/

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTMANAGEMENTWIZARD_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTMANAGEMENTWIZARD_H

// Terralib
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// Qt
#include <QWizard>

namespace ta
{
  namespace model
  {
    namespace qt
    {
      class AttributeControlWizardPage;
      class ClassControlWizardPage;
      class ProjectControlWizardPage;
      class RulesControlWizardPage;

      class ProjectManagementWizard : public QWizard
      {
        Q_OBJECT

      public:

        ProjectManagementWizard(QWidget* parent);

        ~ProjectManagementWizard();

        virtual bool validateCurrentPage();

      public:

        void setActiveServer(te::da::DataSourceInfoPtr dsInfo);

        void setUser(const std::string& user);

      protected:

        void addPages();

      private:

        AttributeControlWizardPage* m_attrControlPage;
        ClassControlWizardPage* m_classControlPage;
        ProjectControlWizardPage* m_projectControlPage;
        RulesControlWizardPage* m_rulesControlPage;

        std::string m_user;

      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_PROJECTMANAGEMENTWIZARD_H
