/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/RulesControlDialog.cpp

  \brief A Qt dialog used to configure the project settings.
*/

// TerraAmazon
#include "RulesControlDialog.h"
#include "RulesControlWidget.h"
#include "ui_RulesControlDialogForm.h"

#include "../../../common/Utils.h"

// Qt
#include <QGridLayout>

ta::model::qt::RulesControlDialog::RulesControlDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::RulesControlDialogForm)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  QGridLayout* layout = new QGridLayout(m_ui->m_widget);
  layout->setContentsMargins(0, 0, 0, 0);

  m_rcWidget = new RulesControlWidget(m_ui->m_widget);

  layout->addWidget(m_rcWidget);

}

ta::model::qt::RulesControlDialog::~RulesControlDialog()
{
}

void ta::model::qt::RulesControlDialog::setActiveServer(te::da::DataSourceInfoPtr dsInfo)
{
  m_rcWidget->setActiveServer(dsInfo);

  this->setWindowTitle(m_rcWidget->windowTitle());
}

void ta::model::qt::RulesControlDialog::setUser(const std::string& user)
{
  m_rcWidget->setUser(user);
}