/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/RulesControlWidget.h

  \brief A Qt widget used to configure the rules control.
*/

#ifndef __TERRAAMAZON_MODEL_QT_PROJECT_RULESCONTROLWIDGET_H
#define __TERRAAMAZON_MODEL_QT_PROJECT_RULESCONTROLWIDGET_H

// Terralib
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/qt/widgets/utils/DoubleListWidget.h>

// STL
#include <memory>

// Qt
#include <QTableWidget>
#include <QSignalMapper>
#include <QWidget>

// Forward declaration
namespace Ui { class RulesControlWidgetForm; }

namespace ta
{
  namespace model
  {
    class Project;

    namespace qt
    {
      class RulesControlWidget : public QWidget
      {
        Q_OBJECT
      public:

        RulesControlWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~RulesControlWidget();


      public:

        void setActiveServer(te::da::DataSourceInfoPtr dsInfo);

        void setUser(const std::string& user);


      public slots:

        void onAddRuleToolButtonPressed();

        void onAssociateLayersToolButtonPressed();

        void onAssociateRulesToolButtonPressed();

        void onCatalogLayerComboBoxActivated(int index);

        void onProjectsComboBoxActivated(int index);

        void onProjectRulesComboBoxActivated(int index);

        void onProjectLayersComboBoxActivated(int index);

        void onRemoveRuleExecuted(int row);

        void setRuleName();

        void onCreateLayerToolButtonPressed();


      protected:

        void setOperations();

        void listCatalogs();

        void listProjects();

        void listRules();

      private:

        std::auto_ptr<Ui::RulesControlWidgetForm> m_ui;

        te::qt::widgets::DoubleListWidget* m_layersDoubleListWidget;
        te::qt::widgets::DoubleListWidget* m_rulesDoubleListWidget;

        QSignalMapper* m_removeMapper;                   //!< The mapper used to know which action was executed.

        te::da::DataSourcePtr m_server;

        std::string m_user;
      };
    } // end namespace qt
  }   // end namespace model
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_PROJECT_RULESCONTROLWIDGET_H
