/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file Workspace.h

  \brief This class models the concept of a workspace for the TerraAmazon.
*/

#ifndef __TERRAAMAZON_WORKSPACE_WORKSPACE_H
#define __TERRAAMAZON_WORKSPACE_WORKSPACE_H

// Terralib
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/BuildConfig.h>

// Qt
#include <QString>

// STL
#include <list>

namespace ta
{
  namespace workspace
  {
    struct WorkspaceMetadata
    {
      QString                                m_title;          //!< The title of the workspace.
      QString                                m_author;         //!< The author of the workspace.
      bool                                   m_changed;        //!< Flag indicating that the workspace needs to be saved.
      QString                                m_fileName;       //!< The workspace file.
      QString                                m_serversXML;     //!< The server list file.
      std::vector<te::da::DataSourceInfoPtr> m_servers;        //!< The servers of the workspace.
      std::string                            m_activeServerId; //!< The active server id of the workspace.
      int                                    m_currentProjId = -1;  //!< The current project id of the workspace.
      int                                    m_currentUserId = -1;   //!< The active user id of the workspace
    };

    void writeServers(const ta::workspace::WorkspaceMetadata& ws, std::string directory);

    std::vector<te::da::DataSourceInfoPtr> readServers(const QString& file);

    void SaveWorkspace(const WorkspaceMetadata& ws, const std::list<te::map::AbstractLayerPtr>& layers);

    void LoadWorkspace(const QString& wsFile, WorkspaceMetadata& ws, std::list<te::map::AbstractLayerPtr>& layers);
  } // end namespace workspace
}   // end namespace ta

#endif  // __TERRAAMAZON_WORKSPACE_WORKSPACE_H
