/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */
 
 /*!
\file terraamazon/ServerLoginDialog.h

\brief A Qt dialog used to configure which servers the user is connected.
*/

#ifndef __TERRAAMAZON_MODEL_QT_WORKSPACE_SERVERLOGINDIALOG_H
#define __TERRAAMAZON_MODEL_QT_WORKSPACE_SERVERLOGINDIALOG_H

// Terralib
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

//TerraAmazon
#include "../../common/Enums.h"

//STL
#include <memory>

// Qt
#include <QDialog>

// Forward declaration
namespace Ui { class ServerLoginDialogForm; }

namespace ta
{

  //Forward declaration
  namespace model
  {
    class User;
    class UserFacade;
  }

  namespace workspace
  {
    namespace qt
    {
      class ServerLoginDialog : public QDialog
      {
        Q_OBJECT

      public:

        ServerLoginDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~ServerLoginDialog();

      public:

        te::da::DataSourceInfoPtr getActiveServer();

        int getActiveUserId();

        void setActiveServer(te::da::DataSourceInfoPtr server);

        std::vector<te::da::DataSourceInfoPtr> getServers();

        void setServers(std::vector<te::da::DataSourceInfoPtr> servers);

        void setUserType(ta::common::TaUserType userType);

      protected:

        bool login(std::unique_ptr<ta::model::User> user);

      protected slots:

        void onAddToolButtonClicked(bool);

        void onOkPushButtonClicked();

      private:

        std::unique_ptr<Ui::ServerLoginDialogForm>    m_ui;            //!< The dialogs form.
        std::unique_ptr<ta::model::UserFacade>        m_facade;        //!< The facade used to handle transactions.
        te::da::DataSourceInfoPtr                     m_activeServer;  //!< The active server selected by the user.
        std::unique_ptr<ta::model::User>              m_activeUser;    //!< The user of the active server.
        int                                           m_userType;      //!< The type of user logging in the application
      };
    } // end namespace qt
  }   // end namespace workspace
}     // end namespace ta

#endif // __TERRAAMAZON_MODEL_QT_WORKSPACE_SERVERLOGINDIALOG_H
