/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */
 
 /*!
\file terraamazon/ServerManagerrDialog.h

\brief A Qt dialog used to configure which servers the user is connected to and which is the active server.
*/

#ifndef __TERRAAMAZON_MODEL_QT_WORKSPACE_SERVERMANAGERDIALOG_H
#define __TERRAAMAZON_MODEL_QT_WORKSPACE_SERVERMANAGERDIALOG_H

// TerraAmazon
#include "Config.h"
#include "../../common/TerraAmazon.h"

// Terralib
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

//STL
#include <memory>

// Qt
#include <QDialog>

// Forward declaration
namespace Ui { class ServerManagerDialogForm; }

namespace ta
{

  //Forward declaration
  namespace model
  {
    class User;
    class UserFacade;
  }

  namespace workspace
  {
    namespace qt
    {
      class ServerManagerDialog : public QDialog
      {
        Q_OBJECT
      public:

        ServerManagerDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~ServerManagerDialog();

        te::da::DataSourceInfoPtr getActiveServer();

        void setActiveServer(te::da::DataSourceInfoPtr server);

        std::vector<te::da::DataSourceInfoPtr> getServers();

        void setServers(std::vector<te::da::DataSourceInfoPtr> servers);

        int getActiveUserId();

        void setActiveUser(std::unique_ptr<ta::model::User> user);

        bool hasChanges();

        protected slots:

        void onAddToolButtonClicked(bool);

        void onOkPushButtonClicked();

        void onUpdateToolButtonClicked(bool);

        void onRemoveToolButtonClicked(bool);

      private:
        std::unique_ptr<Ui::ServerManagerDialogForm>  m_ui;            //!< The dialogs form.
        std::unique_ptr<ta::model::UserFacade>        m_facade;        //!< The facade used to handle transactions.
        te::da::DataSourceInfoPtr                     m_activeServer;  //!< The active server selected by the user.
        std::unique_ptr<ta::model::User>              m_activeUser;    //!< The user of the active server.
        bool                                          m_hasChanges;    //!< Property that holds weather changes were made to the list of servers
      };
    } // end namespace qt
  }   // end namespace workspace
}     // end namespace ta

#endif // __TERRAAMAZON_MODEL_QT_ACCESSCONTROL_USERCONTROLDIALOG_H