/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ServerRegisterDialog.h

  \brief ...
*/

#ifndef __TERRAAMAZON_MODEL_QT_WORKSPACE_SERVERREGISTERDIALOG_H
#define __TERRAAMAZON_MODEL_QT_WORKSPACE_SERVERREGISTERDIALOG_H

// TerraAmazon
#include "Config.h"

// TerraLib
#include <terralib/dataaccess/datasource/DataSourceInfo.h>

// STL
//#include <memory>

// Qt
#include <QDialog>

// Forward declaration
namespace Ui { class ServerRegisterDialogForm; }

namespace ta
{

  namespace model { class User; }

  namespace workspace
  {
    namespace qt
    {
      class ServerRegisterWidget;

      class ServerRegisterDialog : public QDialog
      {
        Q_OBJECT
        public:

          ServerRegisterDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~ServerRegisterDialog();

          std::string getHost();

          std::string getPort();

          std::string getUserName();

          std::string getPassword();

          bool savePassword();

          std::string getServerName();

          void setServer(te::da::DataSourceInfoPtr server);

          te::da::DataSourceInfoPtr getServer();

          std::unique_ptr<ta::model::User> getActiveUser();

        private:

          std::map<std::string, std::string> makeConnInfo(const std::string& host,
                                                          const std::string& port,
                                                          const std::string& user,
                                                          const std::string& password,
                                                          const std::string& dsName);

        public:

          ServerRegisterWidget* getWidget();

        protected slots:

          void onOkPushButtonClicked();

        private:

          std::unique_ptr<Ui::ServerRegisterDialogForm> m_ui;

          std::unique_ptr<ta::model::User> m_user;

          ServerRegisterWidget* m_widget;

          te::da::DataSourceInfoPtr m_server;
      };
    } // end namespace qt
  }   // end namespace workspace
}     // end namespace ta

#endif  // __TERRAAMAZON_MODEL_QT_WORKSPACE_SERVERREGISTERDIALOG_H

