/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ServerRegisterWidget.cpp

  \brief A Qt widget used to configure the Server settings.
*/

// TerraAmazon
#include "../../common/Utils.h"
#include "../../model/Globals.h"
#include "ServerRegisterWidget.h"
#include "ui_ServerRegisterWidgetForm.h"

// TerraLib
#include <terralib/common/Exception.h>
#include <terralib/common/Translator.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceFactory.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/qt/widgets/datasource/connector/AbstractDataSourceConnector.h>
#include <terralib/qt/widgets/datasource/core/DataSourceType.h>
#include <terralib/qt/widgets/datasource/core/DataSourceTypeManager.h>

// STL
#include <string>

// Qt
#include <QListWidgetItem>
#include <QMessageBox>

Q_DECLARE_METATYPE(te::da::DataSourceInfoPtr);

ta::workspace::qt::ServerRegisterWidget::ServerRegisterWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
  m_ui(new Ui::ServerRegisterWidgetForm)
{
  m_ui->setupUi(this);
  m_ui->m_passLineEdit->setEchoMode(QLineEdit::Password);

  connect(m_ui->m_loadServersBushButton, SIGNAL(clicked()), this, SLOT(onLoadServersPushButtonClicked()));
}

ta::workspace::qt::ServerRegisterWidget::~ServerRegisterWidget()
{
}

std::string ta::workspace::qt::ServerRegisterWidget::getHost()
{
  return m_ui->m_hostLineEdit->text().trimmed().toStdString();
}

std::string ta::workspace::qt::ServerRegisterWidget::getPort()
{
  return m_ui->m_portLineEdit->text().trimmed().toStdString();
}

std::string ta::workspace::qt::ServerRegisterWidget::getUser()
{
  return m_ui->m_userLineEdit->text().trimmed().toStdString();
}

std::string ta::workspace::qt::ServerRegisterWidget::getPassword()
{
  return m_ui->m_passLineEdit->text().trimmed().toStdString();
}

std::string ta::workspace::qt::ServerRegisterWidget::getServerName()
{
  return m_ui->m_serversComboBox->currentText().toStdString();
}

std::string ta::workspace::qt::ServerRegisterWidget::getServerAlias()
{
  return m_ui->m_serverAliasLineEdit->text().trimmed().toStdString();
}

void ta::workspace::qt::ServerRegisterWidget::setHost(const std::string& host)
{
  m_ui->m_hostLineEdit->setText(host.c_str());
}

void ta::workspace::qt::ServerRegisterWidget::setPort(const std::string& port)
{
  m_ui->m_portLineEdit->setText(port.c_str());
}

void ta::workspace::qt::ServerRegisterWidget::setUser(const std::string& user)
{
  m_ui->m_userLineEdit->setText(user.c_str());
}

void ta::workspace::qt::ServerRegisterWidget::setPassword(const std::string& password)
{
  m_ui->m_passLineEdit->setText(password.c_str());
}

void ta::workspace::qt::ServerRegisterWidget::setServerName(const std::string& serverName)
{
  m_ui->m_serversComboBox->addItem(serverName.c_str());
}

void ta::workspace::qt::ServerRegisterWidget::setServerAlias(const std::string& serverAlias)
{
  m_ui->m_serverAliasLineEdit->setText(serverAlias.c_str());
}

void ta::workspace::qt::ServerRegisterWidget::onLoadServersPushButtonClicked()
{
  m_ui->m_serversComboBox->clear();

  std::string user, pass, host, port;
  user = m_ui->m_userLineEdit->text().toStdString();
  pass = m_ui->m_passLineEdit->text().toStdString();
  host = m_ui->m_hostLineEdit->text().toStdString();
  port = m_ui->m_portLineEdit->text().toStdString();

  std::map<std::string, std::string> connInfo = ta::common::makePGISConnInfo(ta::model::Globals::sm_taUser, ta::model::Globals::sm_taPassword, host, port);
  std::vector<std::string> dsNames;

  try
  {
    dsNames = te::da::DataSource::getDataSourceNames("POSTGIS", connInfo);
  }
  catch (const std::exception& /*e*/)
  {
    QMessageBox::warning(this, windowTitle(), tr("Unable to load the databases. Has the conceptual model been created?"));
    return;
  }

  for (std::size_t i = 0; i < dsNames.size(); ++i)
  {
    m_ui->m_serversComboBox->addItem(dsNames[i].c_str());
  }
}