/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

#include "WorkspaceInfoDialog.h"
#include "WorkspaceInfoWidget.h"

// Qt
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGridLayout>

ta::workspace::qt::WorkspaceInfoDialog::WorkspaceInfoDialog(QWidget* parent) :
QDialog(parent)
{
  QDialog::setWindowTitle(tr("Workspace Properties"));
  QDialog::setWindowModality(Qt::ApplicationModal);

  //Buttons
  QPushButton* okButton = new QPushButton(tr("&Ok"), this);
  QPushButton* cancelButton = new QPushButton(tr("&Cancel"), this);
  QPushButton* helpButton = new QPushButton(tr("&Help"), this);

  QHBoxLayout* hlay = new QHBoxLayout;

  hlay->addWidget(helpButton);
  hlay->addSpacerItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Minimum));
  hlay->addWidget(okButton);
  hlay->addWidget(cancelButton);
  // --------------------------------------------------------------------

  // Workspace info
  m_info = new ta::workspace::qt::WorkspaceInfoWidget(this);

  QVBoxLayout* vlay = new QVBoxLayout;

  vlay->addWidget(m_info);
  vlay->addSpacerItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Minimum));
  vlay->addLayout(hlay);

  QGridLayout* grd = new QGridLayout(this);
  grd->addLayout(vlay, 0, 0, 1, 1);
  // --------------------------------------------------------------------

  // Signal/Slot Connections
  connect(okButton, SIGNAL(pressed()), SLOT(onOkPushButtonPressed()));
  connect(cancelButton, SIGNAL(pressed()), SLOT(reject()));
  connect(helpButton, SIGNAL(pressed()), SLOT(onHelpPushButtonPressed()));
}

ta::workspace::qt::WorkspaceInfoDialog::~WorkspaceInfoDialog()
{
}

void ta::workspace::qt::WorkspaceInfoDialog::setWorkspace(WorkspaceMetadata* proj)
{
  m_info->setWorkspace(proj);
}

void ta::workspace::qt::WorkspaceInfoDialog::onOkPushButtonPressed()
{
  m_info->updateWorkspaceInfo();
  QDialog::accept();
}

void ta::workspace::qt::WorkspaceInfoDialog::onHelpPushButtonPressed()
{
}
