/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file ProjectInfoDialog.h

\brief A dialog for edition of Project informations.
*/

#ifndef __TERRAAMAZON_WORKSPACE_QT_WORKSPACEINFODIALOG_H
#define __TERRAAMAZON_WORKSPACE_QT_WORKSPACEINFODIALOG_H

#include <QDialog>

namespace ta
{
  namespace workspace
  {
    // Forward declaration
    struct WorkspaceMetadata;

    namespace qt
    {
      // Forward declaration
      class WorkspaceInfoWidget;

      /*!
      \class

      \brief
      */
      class WorkspaceInfoDialog : public QDialog
      {
        Q_OBJECT

      public:

        /*!
        \brief Constructor.

        \param parent QWidget parent.
        */
        WorkspaceInfoDialog(QWidget* parent = 0);

        /*!
        \brief Destructor.
        */
        ~WorkspaceInfoDialog();

        /*!
        \brief Sets the workspace to be inspected.

        \param ws The workspace to be presented.
        */
        void setWorkspace(ta::workspace::WorkspaceMetadata* ws);

        protected slots:

        /*!
        \brief Called on ok button pressed.
        */
        void onOkPushButtonPressed();

        /*!
        \brief Called on help button pressed.
        */
        void onHelpPushButtonPressed();

      protected:

        WorkspaceInfoWidget* m_info;    //!< Pointer to the widget that contains information about workspace.
      };
    } // end namespace qt
  }   // end namespace workspace
}     // end namespace ta

#endif //__TERRAAMAZON_WORKSPACE_QT_WORKSPACEINFODIALOG_H
