/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file WorkspaceInfoWidget.h

  \brief Contains a widget for define Workspace informations.
*/

#ifndef __TERRAAMAZON_WORKSPACE_QT_WORKSPACEINFOWIDGET_H
#define __TERRAAMAZON_WORKSPACE_QT_WORKSPACEINFOWIDGET_H

#include <QWidget>

// Forward declaration
namespace Ui
{
  class WorkspaceInfoWidgetForm;
}

namespace ta
{
  namespace workspace
  {
    // Forward declaration
    struct WorkspaceMetadata;

    namespace qt
    {
      /*!
      \class WorkspaceInfoWidget

      \brief A widget for present workspace informations.
      */
      class WorkspaceInfoWidget : public QWidget
      {
      public:

        /*!
        \brief Constructor.

        \param parent QWidget parent.
        */
        WorkspaceInfoWidget(QWidget* parent = 0);

        /*!
        \brief Destructor.
        */
        ~WorkspaceInfoWidget();

        /*!
        \brief Updates the workspace being visualized.

        \param ws The workspace to be presented.
        */
        void setWorkspace(WorkspaceMetadata* ws);

        /*!
        \brief Updates the workspace informations.
        */
        void updateWorkspaceInfo();

      protected:

        WorkspaceMetadata* m_ws;      //!< Workspace being presented.

      private:

        Ui::WorkspaceInfoWidgetForm* m_ui;  //!< Pointer to the form.
      };
    } // end namespace qt
  }   // end namespace workspace
}     // end namespace ta

#endif //__TERRAAMAZON_WORKSPACE_QT_WORKSPACEINFOWIDGET_H
