/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

// TerraAmazon
#include "../../common/Utils.h"
#include "ui_WorkspaceWidgetForm.h"
#include "WorkspaceWidget.h"

// TerraLib
#include <terralib/qt/af/Utils.h>

ta::workspace::qt::WorkspaceWidget::WorkspaceWidget(QWidget* parent)
  : AbstractSettingWidget(parent),
    m_ui(new Ui::WorkspaceWidgetForm)
{
  m_ui->setupUi(this);

  m_resumeText = tr("Default author for new workspaces.");
  
  connect(m_ui->m_authorLineEdit, SIGNAL(textChanged(const QString&)), SLOT(authorChanged(const QString&)));
  connect(m_ui->m_numRecentWorkspacesSpinBox, SIGNAL(valueChanged(int)), SLOT(maxSavedChanged(int)));
  connect(m_ui->m_lastWorkspaceRadioButton, SIGNAL(clicked()), SLOT(onOpeningOptionChanged()));
  connect(m_ui->m_noneWorkspaceRadioButton, SIGNAL(clicked()), SLOT(onOpeningOptionChanged()));

  resetState();
}

void ta::workspace::qt::WorkspaceWidget::getHelpInformations(QString& ns, QString& helpFile)
{
  ns = "dpi.inpe.br.apf";
  helpFile = "apf/settings/workspace/WorkspaceConfig.html";
}

ta::workspace::qt::WorkspaceWidget::~WorkspaceWidget()
{
  delete m_ui;
}

void ta::workspace::qt::WorkspaceWidget::saveChanges()
{
  QString author = m_ui->m_authorLineEdit->text();
  int maxSaved = m_ui->m_numRecentWorkspacesSpinBox->value();

  ta::common::SaveWorkspaceInformationsOnSettings(author, maxSaved);

  ta::common::SaveOpenLastWorkspaceOnSettings(m_ui->m_lastWorkspaceRadioButton->isChecked());

  changeApplyButtonState(false);
}

void ta::workspace::qt::WorkspaceWidget::resetState()
{
  QString author;
  int maxSaved;

  ta::common::GetWorkspaceInformationsFromSettings(author, maxSaved);

  m_ui->m_authorLineEdit->setText(author);
  m_ui->m_numRecentWorkspacesSpinBox->setValue(maxSaved);

  changeApplyButtonState(false);
}

void ta::workspace::qt::WorkspaceWidget::authorChanged(const QString&)
{
  changeApplyButtonState(true);
}

void ta::workspace::qt::WorkspaceWidget::maxSavedChanged(int)
{
  changeApplyButtonState(true);
}

void ta::workspace::qt::WorkspaceWidget::onOpeningOptionChanged()
{
  changeApplyButtonState(true);
}